@@ -45,6 +45,7 @@
 #include "map/CMapDraw.h"
 #include "map/CMapItem.h"
 #include "map/CMapList.h"
+#include "print/CScreenshotDialog.h"
 #include "realtime/CRtWorkspace.h"
 #include "setup/IAppSetup.h"
 #include "tool/CImportDatabase.h"
@@ -97,7 +98,7 @@ CMainWindow::CMainWindow()
     IGisItem::init();
     wptIconManager = new CWptIconManager(this);
 
-    IUnit::setUnitType((IUnit::type_e)cfg.value("MainWindow/units",IUnit::eTypeMetric).toInt(), this);
+    IUnit::setUnitType((IUnit::type_e)cfg.value("MainWindow/units", IUnit::eTypeMetric).toInt(), this);
     IUnit::setSlopeMode((IUnit::slope_mode_e)cfg.value("Units/slopeMode", IUnit::eSlopeDegrees).toInt());
 
     QByteArray tz;
@@ -156,7 +157,7 @@ CMainWindow::CMainWindow()
         dockStates = cfg.value("dockstate").toByteArray();
     }
 
-    menuVisible = cfg.value("menuvisible",false).toBool();
+    menuVisible = cfg.value("menuvisible", false).toBool();
 
     if(windowState() == Qt::WindowFullScreen)
     {
@@ -171,13 +172,21 @@ CMainWindow::CMainWindow()
     connect(actionQuickstart,            &QAction::triggered,            this,      &CMainWindow::slotQuickstart);
     connect(actionAddMapView,            &QAction::triggered,            this,      &CMainWindow::slotAddCanvas);
     connect(actionCloneMapView,          &QAction::triggered,            this,      &CMainWindow::slotCloneCanvas);
-    connect(actionShowScale,             &QAction::changed,              this,      &CMainWindow::slotUpdateCurrentWidget);
-    connect(actionShowGrid,              &QAction::changed,              this,      static_cast<void (CMainWindow::*)()>(&CMainWindow::update));
-    connect(actionPOIText,               &QAction::changed,              this,      &CMainWindow::slotUpdateCurrentWidget);
-    connect(actionMapToolTip,            &QAction::changed,              this,      &CMainWindow::slotUpdateCurrentWidget);
-    connect(actionNightDay,              &QAction::changed,              this,      &CMainWindow::slotUpdateCurrentWidget);
-    connect(actionMinMaxTrackValues,     &QAction::changed,              this,      &CMainWindow::slotUpdateCurrentWidget);
-    connect(actionProfileIsWindow,       &QAction::toggled,              this,      &CMainWindow::slotSetProfileMode);
+    connect(actionShowGrid,              &QAction::changed,              this,      [this](){this->update();});
+    connect(actionShowScale,             &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionPOIText,               &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionMapToolTip,            &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionNightDay,              &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionShowMinMaxTrackLabels, &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionShowMinMaxSummary,     &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionShowTrackInfoTable,    &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionShowTrackInfoPoints,   &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionShowTrackSummary,      &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionShowTrackProfile,      &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionShowTrackHighlight,    &QAction::changed,              this,      &CMainWindow::slotUpdateTabWidgets);
+    connect(actionShowTrackInfoPoints,   &QAction::triggered, actionShowTrackInfoTable, &QAction::setEnabled);
+    connect(actionShowTrackProfile,      &QAction::triggered, actionProfileIsWindow,    &QAction::setEnabled);
+    connect(actionProfileIsWindow,       &QAction::triggered,            this,      &CMainWindow::slotSetProfileMode);
     connect(actionSetupMapFont,          &QAction::triggered,            this,      &CMainWindow::slotSetupMapFont);
     connect(actionSetupMapBackground,    &QAction::triggered,            this,      &CMainWindow::slotSetupMapBackground);
     connect(actionSetupGrid,             &QAction::triggered,            this,      &CMainWindow::slotSetupGrid);
@@ -198,6 +207,7 @@ CMainWindow::CMainWindow()
     connect(actionClose,                 &QAction::triggered,            this,      &CMainWindow::close);
     connect(actionCreateRoutinoDatabase, &QAction::triggered,            this,      &CMainWindow::slotCreateRoutinoDatabase);
     connect(actionPrintMap,              &QAction::triggered,            this,      &CMainWindow::slotPrintMap);
+    connect(actionTakeScreenshot,        &QAction::triggered,            this,      &CMainWindow::slotTakeScreenshot);
     connect(actionSetupWaypointIcons,    &QAction::triggered,            this,      &CMainWindow::slotSetupWptIcons);
     connect(actionCloseTab,              &QAction::triggered,            this,      &CMainWindow::slotCloseTab);
     connect(actionToggleDocks,           &QAction::triggered,            this,      &CMainWindow::slotToggleDocks);
@@ -238,7 +248,7 @@ CMainWindow::CMainWindow()
     cfg.endGroup(); // Views
     testForNoView();
 
-    CCanvas::gisLayerOpacity = cfg.value("gisLayerOpacity",1.0).toFloat();
+    CCanvas::gisLayerOpacity = cfg.value("gisLayerOpacity", 1.0).toFloat();
     widgetGisWorkspace->setOpacity(CCanvas::gisLayerOpacity);
 
     actionGeoSearch->setChecked(cfg.value("isGeosearchVisible", false).toBool());
@@ -247,11 +257,17 @@ CMainWindow::CMainWindow()
     actionPOIText->setChecked(cfg.value("POIText", true).toBool());
     actionMapToolTip->setChecked(cfg.value("MapToolTip", true).toBool());
     actionNightDay->setChecked(cfg.value("isNight", false).toBool());
-    actionMinMaxTrackValues->setChecked(cfg.value("MinMaxTrackValues", false).toBool());
+    actionShowMinMaxTrackLabels->setChecked(cfg.value("MinMaxTrackValues", false).toBool());
+    actionShowMinMaxSummary->setChecked(cfg.value("ShowMinMaxInformation", true).toBool());
+    actionShowTrackInfoTable->setChecked(cfg.value("ShowTrackInfoTable", true).toBool());
+    actionShowTrackInfoPoints->setChecked(cfg.value("ShowTrackInfoPoints", true).toBool());
+    actionShowTrackSummary->setChecked(cfg.value("ShowTrackSummary", true).toBool());
+    actionShowTrackProfile->setChecked(cfg.value("ShowTrackProfile", true).toBool());
+    actionShowTrackHighlight->setChecked(cfg.value("ShowTrackHighlight", true).toBool());
     actionFlipMouseWheel->setChecked(cfg.value("flipMouseWheel", false).toBool());
     actionProfileIsWindow->setChecked(cfg.value("profileIsWindow", false).toBool());
     mapFont = cfg.value("mapFont", font()).value<QFont>();
-    tabWidget->setCurrentIndex(cfg.value("visibleCanvas",0).toInt());
+    tabWidget->setCurrentIndex(cfg.value("visibleCanvas", 0).toInt());
     cfg.endGroup(); // Canvas
 
 
@@ -300,42 +316,53 @@ CMainWindow::CMainWindow()
     QAction * actionToggleToolBar = toolBar->toggleViewAction();
     actionToggleToolBar->setObjectName("actionToggleToolBar");
     actionToggleToolBar->setIcon(QIcon(":/icons/32x32/ToolBar.png"));
-    menuWindow->insertAction(actionSetupToolbar,actionToggleToolBar);
+    menuWindow->insertAction(actionSetupToolbar, actionToggleToolBar);
 
     QAction * actionToggleMaps = dockMaps->toggleViewAction();
     actionToggleMaps->setObjectName("actionToggleMaps");
     actionToggleMaps->setIcon(QIcon(":/icons/32x32/ToggleMaps.png"));
-    menuWindow->insertAction(actionSetupToolbar,actionToggleMaps);
+    menuWindow->insertAction(actionSetupToolbar, actionToggleMaps);
 
     QAction * actionToggleDem = dockDem->toggleViewAction();
     actionToggleDem->setObjectName("actionToggleDem");
     actionToggleDem->setIcon(QIcon(":/icons/32x32/ToggleDem.png"));
-    menuWindow->insertAction(actionSetupToolbar,actionToggleDem);
+    menuWindow->insertAction(actionSetupToolbar, actionToggleDem);
 
     QAction * actionToggleWorkspace = dockWorkspace->toggleViewAction();
     actionToggleWorkspace->setObjectName("actionToggleWorkspace");
     actionToggleWorkspace->setIcon(QIcon(":/icons/32x32/ToggleGis.png"));
-    menuWindow->insertAction(actionSetupToolbar,actionToggleWorkspace);
+    menuWindow->insertAction(actionSetupToolbar, actionToggleWorkspace);
 
     QAction * actionToggleRealtime = dockRealtime->toggleViewAction();
     actionToggleRealtime->setObjectName("actionToggleRealtime");
     actionToggleRealtime->setIcon(QIcon(":/icons/32x32/ToggleRealTime.png"));
-    menuWindow->insertAction(actionSetupToolbar,actionToggleRealtime);
+    menuWindow->insertAction(actionSetupToolbar, actionToggleRealtime);
 
     QAction * actionToggleDatabase = dockDatabase->toggleViewAction();
     actionToggleDatabase->setObjectName("actionToggleDatabase");
     actionToggleDatabase->setIcon(QIcon(":/icons/32x32/ToggleDatabase.png"));
-    menuWindow->insertAction(actionSetupToolbar,actionToggleDatabase);
+    menuWindow->insertAction(actionSetupToolbar, actionToggleDatabase);
 
     QAction * actionToggleRte = dockRte->toggleViewAction();
     actionToggleRte->setObjectName("actionToggleRte");
     actionToggleRte->setIcon(QIcon(":/icons/32x32/ToggleRouter.png"));
-    menuWindow->insertAction(actionSetupToolbar,actionToggleRte);
-
+    menuWindow->insertAction(actionSetupToolbar, actionToggleRte);
 
     menuWindow->insertSeparator(actionSetupToolbar);
 
-    QAction * separator = new QAction("---------------",this);
+    QMenu * menu = new QMenu(this);
+    menu->addAction(actionShowTrackHighlight);
+    menu->addAction(actionShowMinMaxTrackLabels);
+    menu->addAction(actionShowMinMaxSummary);
+    menu->addAction(actionShowTrackSummary);
+    menu->addAction(actionShowTrackInfoPoints);
+    menu->addAction(actionShowTrackInfoTable);
+    menu->addAction(actionShowTrackProfile);
+    menu->addAction(actionProfileIsWindow);
+    actionTrackInfo->setMenu(menu);
+
+
+    QAction * separator = new QAction("---------------", this);
     separator->setSeparator(true);
     separator->setObjectName("separator");
 
@@ -351,7 +378,15 @@ CMainWindow::CMainWindow()
                      << actionPOIText
                      << actionNightDay
                      << actionMapToolTip
-                     << actionMinMaxTrackValues
+                     << actionTrackInfo
+                     << actionShowTrackHighlight
+                     << actionShowMinMaxSummary
+                     << actionShowMinMaxTrackLabels
+                     << actionShowTrackInfoPoints
+                     << actionShowTrackInfoTable
+                     << actionShowTrackSummary
+                     << actionShowTrackProfile
+                     << actionProfileIsWindow
                      << actionSetupDEMPaths
                      << actionAbout
                      << actionHelp
@@ -368,11 +403,11 @@ CMainWindow::CMainWindow()
                      << actionVrtBuilder
                      << actionStoreView
                      << actionLoadView
-                     << actionProfileIsWindow
                      << actionClose
                      << actionCloneMapView
                      << actionCreateRoutinoDatabase
                      << actionPrintMap
+                     << actionTakeScreenshot
                      << actionSetupCoordFormat
                      << actionSetupMapBackground
                      << actionSetupWaypointIcons
@@ -388,9 +423,10 @@ CMainWindow::CMainWindow()
                      << actionToggleDocks
                      << actionToggleToolBar
                      << actionFullScreen
-                     << actionStartQMapTool;
+                     << actionStartQMapTool
+                     << actionRenameView;
 
-    QAction * separator1 = new QAction("---------------",this);
+    QAction * separator1 = new QAction("---------------", this);
     separator1->setSeparator(true);
     separator1->setObjectName("separator");
 
@@ -405,8 +441,7 @@ CMainWindow::CMainWindow()
                    << actionPOIText
                    << actionNightDay
                    << actionMapToolTip
-                   << actionMinMaxTrackValues
-                   << actionProfileIsWindow
+                   << actionTrackInfo
                    << separator1
                    << actionSetupToolbar
                    << actionToggleMaps
@@ -465,11 +500,11 @@ CMainWindow::~CMainWindow()
     {
         activeDockNames << dock->objectName();
     }
-    cfg.setValue("activedocks",activeDockNames);
+    cfg.setValue("activedocks", activeDockNames);
 
-    cfg.setValue("displaymode",static_cast<int>(displayMode));
-    cfg.setValue("dockstate",dockStates);
-    cfg.setValue("menuvisible",menuVisible);
+    cfg.setValue("displaymode", static_cast<int>(displayMode));
+    cfg.setValue("dockstate", dockStates);
+    cfg.setValue("menuvisible", menuVisible);
     cfg.endGroup();
 
     /*
@@ -511,9 +546,15 @@ CMainWindow::~CMainWindow()
     cfg.setValue("POIText", actionPOIText->isChecked());
     cfg.setValue("MapToolTip", actionMapToolTip->isChecked());
     cfg.setValue("isNight", actionNightDay->isChecked());
-    cfg.setValue("MinMaxTrackValues", actionMinMaxTrackValues->isChecked());
+    cfg.setValue("MinMaxTrackValues", actionShowMinMaxTrackLabels->isChecked());
+    cfg.setValue("ShowMinMaxInformation", actionShowMinMaxSummary->isChecked());
+    cfg.setValue("ShowTrackInfoTable", actionShowTrackInfoTable->isChecked());
+    cfg.setValue("ShowTrackInfoPoints", actionShowTrackInfoPoints->isChecked());
+    cfg.setValue("ShowTrackSummary", actionShowTrackSummary->isChecked());
+    cfg.setValue("ShowTrackProfile", actionShowTrackProfile->isChecked());
+    cfg.setValue("ShowTrackHighlight", actionShowTrackHighlight->isChecked());
     cfg.setValue("flipMouseWheel", actionFlipMouseWheel->isChecked());
-    cfg.setValue("profileIsWindow",actionProfileIsWindow->isChecked());
+    cfg.setValue("profileIsWindow", actionProfileIsWindow->isChecked());
     cfg.setValue("mapFont", mapFont);
     CMapDraw::saveMapPath(cfg);
     CDemDraw::saveDemPath(cfg);
@@ -547,6 +588,10 @@ CMainWindow::~CMainWindow()
 
     toolBarConfig->saveSettings();
     geoSearchConfig->save();
+
+    // delete icon manager explicitely to make sure temporary icon files are
+    // removed upon destruction
+    delete wptIconManager;
 }
 
 void CMainWindow::setupHomePath()
@@ -583,7 +628,12 @@ CCanvas *CMainWindow::addView(const QString& name)
     CCanvas * view = new CCanvas(tabWidget, name);
     tabWidget->addTab(view, view->objectName());
     connect(view, &CCanvas::sigMousePosition, this, &CMainWindow::slotMousePosition);
-    connect(actionMinMaxTrackValues, &QAction::triggered, view, &CCanvas::slotUpdateTrackStatistic);
+    connect(actionShowTrackHighlight, &QAction::changed,    view, [view] {view->slotUpdateTrackInfo(false);});
+    connect(actionShowMinMaxSummary, &QAction::changed,     view, [view] {view->slotUpdateTrackInfo(false);});
+    connect(actionShowTrackInfoTable, &QAction::changed,    view, [view] {view->slotUpdateTrackInfo(false);});
+    connect(actionShowTrackInfoPoints, &QAction::changed,   view, [view] {view->slotUpdateTrackInfo(true);});
+    connect(actionShowTrackSummary, &QAction::changed,      view, [view] {view->slotUpdateTrackInfo(false);});
+    connect(actionShowTrackProfile, &QAction::changed,      view, [view] {view->slotUpdateTrackInfo(false);});
 
     return view;
 }
@@ -652,9 +702,39 @@ bool CMainWindow::isMapToolTip() const
     return actionMapToolTip->isChecked();
 }
 
-bool CMainWindow::isMinMaxTrackValues() const
+bool CMainWindow::isShowMinMaxTrackLabels() const
 {
-    return actionMinMaxTrackValues->isChecked();
+    return actionShowMinMaxTrackLabels->isChecked();
+}
+
+bool CMainWindow::isShowMinMaxSummary() const
+{
+    return actionShowMinMaxSummary->isChecked();
+}
+
+bool CMainWindow::isShowTrackSummary() const
+{
+    return actionShowTrackSummary->isChecked();
+}
+
+bool CMainWindow::isShowTrackInfoTable() const
+{
+    return actionShowTrackInfoTable->isChecked() && actionShowTrackInfoPoints->isChecked();
+}
+
+bool CMainWindow::isShowTrackInfoPoints() const
+{
+    return actionShowTrackInfoPoints->isChecked();
+}
+
+bool CMainWindow::isShowTrackProfile() const
+{
+    return actionShowTrackProfile->isChecked();
+}
+
+bool CMainWindow::isShowTrackHighlight() const
+{
+    return actionShowTrackHighlight->isChecked();
 }
 
 bool CMainWindow::flipMouseWheel() const
@@ -669,12 +749,12 @@ bool CMainWindow::profileIsWindow() const
 
 void CMainWindow::addMapList(CMapList * list, const QString &name)
 {
-    tabMaps->addTab(list,name);
+    tabMaps->addTab(list, name);
 }
 
 void CMainWindow::addDemList(CDemList * list, const QString &name)
 {
-    tabDem->addTab(list,name);
+    tabDem->addTab(list, name);
 }
 
 void CMainWindow::addWidgetToTab(QWidget * w)
@@ -1073,8 +1153,9 @@ void CMainWindow::slotMousePosition(const QPointF& pos, qreal ele, qreal slope)
     if(slope != NOFLOAT)
     {
         QString val;
-        val.sprintf("%.1f", slope);
-        lblSlope->setText(tr("Slope: %1%2", "terrain").arg(val).arg("°"));
+        QString unit;
+        IUnit::self().slope2string(slope, val, unit);
+        lblSlope->setText(tr("Slope: %1%2", "terrain").arg(val).arg(unit));
         lblSlope->show();
     }
     else
@@ -1099,24 +1180,31 @@ void CMainWindow::slotMousePosition(const QPointF& pos, qreal ele, qreal slope)
     }
 }
 
-void CMainWindow::slotUpdateCurrentWidget()
+void CMainWindow::slotUpdateTabWidgets()
 {
-    CCanvas * canvas = getVisibleCanvas();
-    if(canvas)
+    const int N = tabWidget->count();
+    for(int n = 0; n < N; n++)
     {
-        canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawAll);
-        return;
-    }
+        QWidget * w = tabWidget->widget(n);
+
+        CCanvas * canvas = dynamic_cast<CCanvas*>(w);
+        if(canvas != nullptr)
+        {
+            canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawAll);
+            continue;
+        }
+
+        CDetailsTrk * trkDetails = dynamic_cast<CDetailsTrk*>(w);
+        if(trkDetails != nullptr)
+        {
+            trkDetails->updateData();
+            continue;
+        }
 
-    QWidget * w = tabWidget->currentWidget();
-    if(w)
-    {
         w->update();
-        return;
     }
 }
 
-
 void CMainWindow::slotSetupMapFont()
 {
     bool ok = false;
@@ -1203,7 +1291,7 @@ void CMainWindow::slotSetupCoordFormat()
 
 void CMainWindow::slotSetupToolbar()
 {
-    CToolBarSetupDialog dlg(this,toolBarConfig);
+    CToolBarSetupDialog dlg(this, toolBarConfig);
     dlg.exec();
 }
 
@@ -1343,6 +1431,16 @@ void CMainWindow::slotPrintMap()
     }
 }
 
+void CMainWindow::slotTakeScreenshot()
+{
+    CCanvas * canvas = getVisibleCanvas();
+    if(nullptr != canvas)
+    {
+        CScreenshotDialog dlg(*canvas, this);
+        dlg.exec();
+    }
+}
+
 void CMainWindow::slotLinkActivated(const QString& link)
 {
     if(link == "NewView")
@@ -1724,8 +1822,8 @@ void CMainWindow::slotSanityTest()
     if(pjsrc == nullptr)
     {
         QMessageBox::critical(this, tr("Fatal...")
-                              ,tr("QMapShack detected a badly installed Proj4 library. The translation tables for EPSG projections usually stored in /usr/share/proj are missing. Please contact the package maintainer of your distribution to fix it.")
-                              ,QMessageBox::Close);
+                              , tr("QMapShack detected a badly installed Proj4 library. The translation tables for EPSG projections usually stored in /usr/share/proj are missing. Please contact the package maintainer of your distribution to fix it.")
+                              , QMessageBox::Close);
 
         deleteLater();
         return;
