@@ -17,21 +17,23 @@
 
 **********************************************************************************************/
 
-#include "CMainWindow.h"
-#include "GeoMath.h"
 #include "canvas/CCanvas.h"
 #include "canvas/CCanvasSetup.h"
+#include "CMainWindow.h"
 #include "dem/CDemDraw.h"
 #include "gis/CGisDraw.h"
 #include "gis/CGisWorkspace.h"
 #include "gis/IGisLine.h"
-#include "gis/Poi.h"
 #include "gis/ovl/CGisItemOvlArea.h"
+#include "gis/Poi.h"
 #include "gis/trk/CGisItemTrk.h"
+#include "gis/trk/CTableTrkInfo.h"
 #include "grid/CGrid.h"
 #include "grid/CGridSetup.h"
+#include "GeoMath.h"
 #include "helpers/CDraw.h"
 #include "helpers/CSettings.h"
+#include "helpers/CWptIconManager.h"
 #include "map/CMapDraw.h"
 #include "mouse/CMouseAdapter.h"
 #include "mouse/CMouseEditArea.h"
@@ -42,6 +44,7 @@
 #include "mouse/CMousePrint.h"
 #include "mouse/CMouseRadiusWpt.h"
 #include "mouse/CMouseRangeTrk.h"
+#include "mouse/CMouseRuler.h"
 #include "mouse/CMouseSelect.h"
 #include "mouse/CMouseWptBubble.h"
 #include "plot/CPlotProfile.h"
@@ -49,9 +52,25 @@
 #include "units/IUnit.h"
 #include "widgets/CColorLegend.h"
 
+
 #include <QtWidgets>
 
-qreal CCanvas::gisLayerOpacity = 1.0;
+qreal CCanvas::gisLayerOpacity          = 1.0;
+
+
+#define X_OFF_STATUS          20
+#define Y_OFF_STATUS          50
+#define WIDTH_PROFILE_LARGE   300
+#define HEIGHT_PROFILE_LARGE  120
+#define WIDTH_PROFILE_SMALL   200
+#define HEIGHT_PROFILE_SMALL  80
+
+inline QSize getTrackProfileSize(int height)
+{
+    return height > 700 ?
+           QSize(WIDTH_PROFILE_LARGE, HEIGHT_PROFILE_LARGE)
+           : QSize(WIDTH_PROFILE_SMALL, HEIGHT_PROFILE_SMALL);
+}
 
 CCanvas::CCanvas(QWidget *parent, const QString &name)
     : QWidget(parent)
@@ -112,16 +131,19 @@ CCanvas::CCanvas(QWidget *parent, const QString &name)
     loadIndicator2->start();
     demLoadIndicator->show();
 
-    labelStatusMessages = new QLabel(this);
-    labelStatusMessages->setWordWrap(true);
-    labelStatusMessages->setMinimumWidth(300);
-    labelStatusMessages->hide();
+    textStatusMessages = new QTextBrowser(this);
+    textStatusMessages->setFrameStyle(QFrame::NoFrame);
+    textStatusMessages->setMinimumWidth(300);
+    textStatusMessages->hide();
 
     labelTrackStatistic = new QLabel(this);
-    labelTrackStatistic->setWordWrap(true);
-    labelTrackStatistic->setMinimumWidth(300);
+    labelTrackStatistic->setAutoFillBackground(true);
     labelTrackStatistic->hide();
 
+    labelTrackInfo = new QLabel(this);
+    labelTrackInfo->setAutoFillBackground(true);
+    labelTrackInfo->hide();
+
     connect(map, &CMapDraw::sigStartThread, mapLoadIndicator, &QLabel::show);
     connect(map, &CMapDraw::sigStopThread,  mapLoadIndicator, &QLabel::hide);
 
@@ -134,7 +156,6 @@ CCanvas::CCanvas(QWidget *parent, const QString &name)
 
     connect(timerTrackOnFocus, &QTimer::timeout, this, &CCanvas::slotCheckTrackOnFocus);
 
-
     labelHelp = new QTextBrowser(this);
     labelHelp->setOpenLinks(false);
     connect(map, &CMapDraw::sigActiveMapsChanged, labelHelp, &QLabel::setVisible);
@@ -194,7 +215,7 @@ void CCanvas::buildHelpText()
                                    "convenient download of maps, elevation data and routing data. You have to download these resources "
                                    "from various sites on your own. But this page might help you a bit</p>"
                                    );
-    addHtmlRow(msg,"<img src='://pics/DockWidgets.png'/>", msgWelcome, "rowspan='3' style='padding-right: 20px;'", "");
+    addHtmlRow(msg, "<img src='://pics/DockWidgets.png'/>", msgWelcome, "rowspan='3' style='padding-right: 20px;'", "");
 
     const QString& msgLetsStart = tr("<h2>Let's Start</h2>"
                                      "<p>As a first step make yourself comfortable by arranging the dock "
@@ -391,6 +412,11 @@ void CCanvas::setMap(const QString& filename)
     map->buildMapList(filename);
 }
 
+void CCanvas::abortMouse()
+{
+    mouse->unfocus();
+}
+
 void CCanvas::resetMouse()
 {
     mouse->setDelegate(new CMouseNormal(gis, this, mouse));
@@ -459,6 +485,11 @@ void CCanvas::setMouseWptBubble(const IGisItem::key_t& key)
     mouse->setDelegate(new CMouseWptBubble(key, gis, this, mouse));
 }
 
+void CCanvas::setMouseRuler()
+{
+    mouse->setDelegate(new CMouseRuler(gis, this, mouse));
+}
+
 void CCanvas::setMousePrint()
 {
     mouse->setDelegate(new CMousePrint(gis, this, mouse));
@@ -490,22 +521,34 @@ void CCanvas::reportStatus(const QString& key, const QString& msg)
 
     if(report.isEmpty())
     {
-        labelStatusMessages->hide();
+        textStatusMessages->hide();
     }
     else
     {
-        labelStatusMessages->show();
-        labelStatusMessages->setText(report);
-        labelStatusMessages->adjustSize();
+        textStatusMessages->setMinimumWidth(statusMessages.contains("CMouseRuler") ? 400 : 300);
+        textStatusMessages->show();
+        textStatusMessages->setText(report);
+
+        qreal h;
+        h = height() - Y_OFF_STATUS - getTrackProfileSize(height()).height() - 40;
+        h = qMin(h, textStatusMessages->document()->size().height() + 10);
+        textStatusMessages->setMinimumHeight(h);
+        textStatusMessages->setMaximumHeight(h);
     }
     update();
 }
 
 void CCanvas::resizeEvent(QResizeEvent * e)
 {
-    needsRedraw = eRedrawAll;
+    if(!setDrawContextSize(e->size()))
+    {
+        // reschedule resize event because one of the draw context threads is still running
+        // and blocking the access to internal data.
+        QApplication::postEvent(this, new QResizeEvent(e->size(), e->oldSize()));
+        return;
+    }
 
-    setDrawContextSize(e->size());
+    needsRedraw = eRedrawAll;
     QWidget::resizeEvent(e);
 
     const QRect& r = rect();
@@ -517,13 +560,13 @@ void CCanvas::resizeEvent(QResizeEvent * e)
     QPoint p2(demLoadIndicator->width()>>1, demLoadIndicator->height()>>1);
     demLoadIndicator->move(r.center() - p2);
 
-    labelStatusMessages->move(20,50);
+    textStatusMessages->move(X_OFF_STATUS, Y_OFF_STATUS);
 
-    slotUpdateTrackStatistic(CMainWindow::self().isMinMaxTrackValues());
+    slotUpdateTrackInfo(false);
     setSizeTrackProfile();
 
-    QSize s = e->size() - QSize(50,50);
-    labelHelp->move(25,25);
+    QSize s = e->size() - QSize(50, 50);
+    labelHelp->move(25, 25);
     labelHelp->resize(s);
     buildHelpText();
 }
@@ -537,7 +580,7 @@ void CCanvas::paintEvent(QPaintEvent*)
 
     QPainter p;
     p.begin(this);
-    USE_ANTI_ALIASING(p,true);
+    USE_ANTI_ALIASING(p, true);
 
     // fill the background with default pattern
     p.fillRect(rect(), backColor);
@@ -740,29 +783,38 @@ void CCanvas::keyPressEvent(QKeyEvent * e)
 
 void CCanvas::drawStatusMessages(QPainter& p)
 {
-    if(labelStatusMessages->isVisible())
+    if(textStatusMessages->isVisible())
     {
-        QRect r = labelStatusMessages->frameGeometry();
+        QRect r = textStatusMessages->frameGeometry();
         r.adjust(-5, -5, 5, 5);
         p.setPen(CDraw::penBorderGray);
-        p.setBrush(CDraw::brushBackWhite);
+        p.setBrush(textStatusMessages->palette().color(textStatusMessages->backgroundRole()));
         p.drawRoundedRect(r, RECT_RADIUS, RECT_RADIUS);
     }
 }
 
 void CCanvas::drawTrackStatistic(QPainter& p)
 {
+    p.save();
+    p.setPen(CDraw::penBorderGray);
+    p.setBrush(labelTrackStatistic->palette().color(labelTrackStatistic->backgroundRole()));
     if(labelTrackStatistic->isVisible())
     {
         QRect r = labelTrackStatistic->frameGeometry();
         r.adjust(-5, -5, 5, 5);
-        p.setPen(CDraw::penBorderGray);
-        p.setBrush(CDraw::brushBackWhite);
         p.drawRoundedRect(r, RECT_RADIUS, RECT_RADIUS);
     }
+
+    if(labelTrackInfo->isVisible())
+    {
+        QRect r = labelTrackInfo->frameGeometry();
+        r.adjust(-5, -5, 5, 5);
+        p.drawRoundedRect(r, RECT_RADIUS, RECT_RADIUS);
+    }
+    p.restore();
 }
 
-void CCanvas::drawScale(QPainter& p)
+void CCanvas::drawScale(QPainter& p, QRectF drawRect)
 {
     if(!CMainWindow::self().isScaleVisible())
     {
@@ -770,9 +822,9 @@ void CCanvas::drawScale(QPainter& p)
     }
 
     // step I: get the approximate distance for 200px in the bottom right corner
-    QPointF brc(rect().bottomRight() - QPoint(50,30));
+    QPointF brc(drawRect.bottomRight() - QPoint(50, 30));
     QPointF pt1 = brc;
-    QPointF pt2 = brc - QPoint(-200,0);
+    QPointF pt2 = brc - QPoint(-200, 0);
 
     map->convertPx2Rad(pt1);
     map->convertPx2Rad(pt2);
@@ -785,15 +837,15 @@ void CCanvas::drawScale(QPainter& p)
 
     if(0 <= b && b < log10(3.0f))
     {
-        d = 1 * qPow(10,a);
+        d = 1 * qPow(10, a);
     }
     else if(log10(3.0f) < b && b < log10(5.0f))
     {
-        d = 3 * qPow(10,a);
+        d = 3 * qPow(10, a);
     }
     else
     {
-        d = 5 * qPow(10,a);
+        d = 5 * qPow(10, a);
     }
 
     // step III: convert the scale length from [m] into [px]
@@ -805,24 +857,24 @@ void CCanvas::drawScale(QPainter& p)
     map->convertRad2Px(pt2);
 
     p.setPen(QPen(Qt::white, 9));
-    p.drawLine(pt1, pt2 + QPoint(9,0));
+    p.drawLine(pt1, pt2 + QPoint(9, 0));
     p.setPen(QPen(Qt::black, 7));
-    p.drawLine(pt1, pt2 + QPoint(9,0));
+    p.drawLine(pt1, pt2 + QPoint(9, 0));
     p.setPen(QPen(Qt::white, 5));
-    p.drawLine(pt1, pt2 + QPoint(9,0));
+    p.drawLine(pt1, pt2 + QPoint(9, 0));
 
     QVector<qreal> pattern;
     pattern << 2 << 4;
     QPen pen(Qt::black, 5, Qt::CustomDashLine);
     pen.setDashPattern(pattern);
     p.setPen(pen);
-    p.drawLine(pt1, pt2 + QPoint(9,0));
+    p.drawLine(pt1, pt2 + QPoint(9, 0));
 
 
     QPoint pt3(pt2.x() + (pt1.x() - pt2.x())/2, pt2.y());
 
     QString val, unit;
-    IUnit::self().meter2distance(d,val,unit);
+    IUnit::self().meter2distance(d, val, unit);
     CDraw::text(QString("%1 %2").arg(val).arg(unit), p, pt3, Qt::black);
 }
 
@@ -841,8 +893,8 @@ void CCanvas::slotToolTip()
     {
         return;
     }
-    QPoint p = mapToGlobal(posToolTip + QPoint(32,0));
-    QToolTip::showText(p,str);
+    QPoint p = mapToGlobal(posToolTip + QPoint(32, 0));
+    QToolTip::showText(p, str);
 }
 
 void CCanvas::slotCheckTrackOnFocus()
@@ -859,21 +911,23 @@ void CCanvas::slotCheckTrackOnFocus()
         keyTrackOnFocus.clear();
         labelTrackStatistic->clear();
         labelTrackStatistic->hide();
+        labelTrackInfo->clear();
+        labelTrackInfo->hide();
+
 
         // get access to next track object
         CGisItemTrk * trk2 = dynamic_cast<CGisItemTrk*>(CGisWorkspace::self().getItemByKey(key));
         if(nullptr == trk2)
         {
+            update();
             return;
         }
 
+        const CMainWindow& w = CMainWindow::self();
         // create new profile plot, the plot will register itself at the track
-        plotTrackProfile = new CPlotProfile(trk2, trk2->limitsGraph1, CMainWindow::self().profileIsWindow() ? IPlot::eModeWindow : IPlot::eModeIcon, this);
+        plotTrackProfile = new CPlotProfile(trk2, trk2->limitsGraph1, w.profileIsWindow() ? IPlot::eModeWindow : IPlot::eModeIcon, this);
         setSizeTrackProfile();
-        if(isVisible())
-        {
-            plotTrackProfile->show();
-        }
+        // track profile visibility is set in slotUpdateTrackInfo()
 
         colorLegend = new CColorLegend(this, trk2);
         colorLegend->setGeometry(20, 20, 40, 300);
@@ -881,32 +935,98 @@ void CCanvas::slotCheckTrackOnFocus()
         // finally store the new key as track on focus
         keyTrackOnFocus = key;
 
-        slotUpdateTrackStatistic(CMainWindow::self().isMinMaxTrackValues());
+        slotUpdateTrackInfo(false);
+        update();
     }
 }
 
-void CCanvas::slotUpdateTrackStatistic(bool show)
+void CCanvas::slotUpdateTrackInfo(bool updateVisuals)
 {
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWorkspace::self().getItemByKey(keyTrackOnFocus));
 
-    if(show && trk)
+    if(trk == nullptr)
     {
-        QString text = trk->getInfo(IGisItem::eFeatureShowName|IGisItem::eFeatureShowActivity);
-        text += trk->getInfoLimits();
+        labelTrackInfo->clear();
+        labelTrackInfo->hide();
+        labelTrackStatistic->clear();
+        labelTrackStatistic->hide();
+        return;
+    }
+
+    bool trackStatisticIsVisible = false;
+    CMainWindow& w = CMainWindow::self();
+    if(w.isShowTrackSummary() || w.isShowMinMaxSummary())
+    {
+        trackStatisticIsVisible = true;
+
+        QString text;
+        if(w.isShowTrackSummary())
+        {
+            text += trk->getInfo(IGisItem::eFeatureShowName|IGisItem::eFeatureShowActivity);
+        }
+
+        if(w.isShowMinMaxSummary())
+        {
+            text += trk->getInfoLimits();
+        }
 
-        labelTrackStatistic->setMinimumWidth((trk->getActivities().getActivityCount() > 1) ? 450 : 350);
         labelTrackStatistic->setText(text);
         labelTrackStatistic->adjustSize();
 
         labelTrackStatistic->move(rect().width() - labelTrackStatistic->width() - 20, rect().height() - labelTrackStatistic->height() - 60);
         labelTrackStatistic->show();
-        update();
     }
     else
     {
         labelTrackStatistic->clear();
         labelTrackStatistic->hide();
     }
+
+
+    if(w.isShowTrackInfoTable())
+    {
+        int cnt = 1;
+        QString text;
+
+        text += "<table>";
+        for(const CTrackData::trkpt_t& trkpt: trk->getTrackData())
+        {
+            if(trkpt.isHidden() || trkpt.desc.isEmpty())
+            {
+                continue;
+            }
+            text += "<tr><td><img src=" + CWptIconManager::self().getNumberedBullet(cnt++) + "/></td><td>" + trkpt.desc + "</td></tr>";
+        }
+        text += "</table>";
+
+        labelTrackInfo->setText(text);
+        labelTrackInfo->adjustSize();
+
+        const int x = rect().width() - labelTrackInfo->width() - 20;
+        const int y =  rect().height()
+                      - (trackStatisticIsVisible ? labelTrackStatistic->height() + 20 : 0)
+                      - labelTrackInfo->height() - 60;
+
+        labelTrackInfo->move(x, y);
+        labelTrackInfo->setVisible(cnt > 1);
+    }
+    else
+    {
+        labelTrackInfo->clear();
+        labelTrackInfo->hide();
+    }
+
+    if(isVisible() && (plotTrackProfile != nullptr))
+    {
+        plotTrackProfile->setVisible(w.isShowTrackProfile());
+    }
+
+    if(updateVisuals)
+    {
+        trk->updateVisuals(CGisItemTrk::eVisualPlot, "CCanvas::slotUpdateTrackInfo()");
+    }
+
+    update();
 }
 
 void CCanvas::moveMap(const QPointF& delta)
@@ -1100,8 +1220,8 @@ void CCanvas::setSizeTrackProfile()
         }
         else
         {
-            plotTrackProfile->resize(300,200);
-            plotTrackProfile->move(100,100);
+            plotTrackProfile->resize(300, 200);
+            plotTrackProfile->move(100, 100);
         }
 
         cfg.endGroup(); // objectName()
@@ -1110,15 +1230,7 @@ void CCanvas::setSizeTrackProfile()
     }
     else
     {
-        if(size().height() < 700)
-        {
-            plotTrackProfile->resize(200,80);
-        }
-        else
-        {
-            plotTrackProfile->resize(300,120);
-        }
-
+        plotTrackProfile->resize(getTrackProfileSize(height()));
         plotTrackProfile->move(20, height() - plotTrackProfile->height() - 20);
     }
 }
@@ -1140,19 +1252,21 @@ void CCanvas::showProfile(bool yes)
 {
     if(nullptr != plotTrackProfile)
     {
-        plotTrackProfile->setVisible(yes);
+        plotTrackProfile->setVisible(yes && CMainWindow::self().isShowTrackProfile());
     }
 }
 
-void CCanvas::setDrawContextSize(const QSize& s)
+bool CCanvas::setDrawContextSize(const QSize& s)
 {
+    bool done = true;
     for(IDrawContext * context : allDrawContext)
     {
-        context->resize(s);
+        done &= context->resize(s);
     }
+    return done;
 }
 
-void CCanvas::print(QPainter& p, const QRectF& area, const QPointF& focus)
+void CCanvas::print(QPainter& p, const QRectF& area, const QPointF& focus, bool printScale)
 {
     const QSize oldSize = size();
     const QSize newSize(area.size().toSize());
@@ -1184,11 +1298,15 @@ void CCanvas::print(QPainter& p, const QRectF& area, const QPointF& focus)
     p.resetTransform();
     // ----- start to draw fast content -----
 
-    QRect r(QPoint(0,0), area.size().toSize());
+    QRect r(QPoint(0, 0), area.size().toSize());
 
     grid->draw(p, r);
     gis->draw(p, r);
     rt->draw(p, r);
+    if(printScale)
+    {
+        drawScale(p, r);
+    }
 
     setDrawContextSize(oldSize);
 }
@@ -1231,7 +1349,7 @@ bool CCanvas::gestureEvent(QGestureEvent* e)
             if (pscale < 0.8f || pscale > 1.25f)
             {
                 const QPointF & center = pinch->centerPoint();
-                const QPointF & pos = mapFromGlobal(QPoint(center.x(),center.y()));
+                const QPointF & pos = mapFromGlobal(QPoint(center.x(), center.y()));
                 QPointF pt1 = pos;
                 map->convertPx2Rad(pt1);
                 setZoom(pscale > 1.0f, needsRedraw);
@@ -1250,3 +1368,20 @@ bool CCanvas::gestureEvent(QGestureEvent* e)
     }
     return true;
 }
+
+void CCanvas::followPosition(const QPointF& pos)
+{
+    QPointF pos_ = pos * DEG_TO_RAD;
+    convertRad2Px(pos_);
+
+    const QRect& r1 = rect();
+    QRectF r2(0, 0, r1.width() * 0.3, r1.height() * 0.3);
+    r2.moveCenter(r1.center());
+
+    if(r2.contains(pos_))
+    {
+        return;
+    }
+
+    moveMap(r1.center() - pos_);
+}
