@@ -51,6 +51,7 @@ class QTextBrowser;
 class IPlot;
 struct SGisLine;
 struct poi_t;
+class CTableTrkInfo;
 
 class CCanvas : public QWidget
 {
@@ -161,7 +162,7 @@ public:
      */
     bool findPolylineCloseBy(const QPointF& pt1, const QPointF& pt2, qint32 threshold, QPolygonF& polyline);
 
-    void print(QPainter &p, const QRectF& area, const QPointF &focus);
+    void print(QPainter &p, const QRectF& area, const QPointF &focus, bool printScale = true);
 
     /**
        @brief Set a single map file to be shown on the canvas
@@ -170,7 +171,10 @@ public:
      */
     void setMap(const QString& filename);
 
+    void followPosition(const QPointF& pos);
+
     static qreal gisLayerOpacity;
+
 signals:
     void sigMousePosition(const QPointF& pos, qreal ele, qreal slope);
     void sigZoom();
@@ -178,7 +182,8 @@ signals:
 
 public slots:
     void slotTriggerCompleteUpdate(CCanvas::redraw_e flags);
-    void slotUpdateTrackStatistic(bool show);
+    void slotUpdateTrackInfo(bool updateVisuals);
+    void slotCheckTrackOnFocus();
 
 protected:
     bool event(QEvent *) override;
@@ -197,14 +202,16 @@ protected:
 
 private slots:
     void slotToolTip();
-    void slotCheckTrackOnFocus();
-
 
 private:
     void drawStatusMessages(QPainter& p);
     void drawTrackStatistic(QPainter& p);
-    void drawScale(QPainter& p);
-    void setZoom(bool in, redraw_e &needsRedraw);
+    void drawScale(QPainter& p, QRectF drawRect);
+    void drawScale(QPainter& p)//Default use, drawRect is introduced for correct printing
+    {
+        drawScale(p, rect());
+    }
+    void setZoom(bool in, redraw_e & needsRedraw);
     void setSizeTrackProfile();
     void saveSizeTrackProfile();
     /**
@@ -260,6 +267,8 @@ private:
     /// a label with a track
     QLabel * labelTrackStatistic;
 
+    QLabel * labelTrackInfo;
+
     QTextBrowser * textStatusMessages;
     QMap<QString, QString> statusMessages;
 
