@@ -16,9 +16,9 @@
 
 **********************************************************************************************/
 
-#include "CMainWindow.h"
 #include "canvas/CCanvas.h"
 #include "config.h"
+#include "CMainWindow.h"
 #include "gis/CGisListDB.h"
 #include "gis/CGisWorkspace.h"
 #include "gis/db/CDBFolderLostFound.h"
@@ -82,7 +82,7 @@ CGisListDB::CGisListDB(QWidget *parent)
         QString type = cfg.value("type", "SQLite").toString();
         if(type == "SQLite")
         {
-            QString filename = cfg.value("filename","").toString();
+            QString filename = cfg.value("filename", "").toString();
             if(filename.isEmpty())
             {
                 QMessageBox::information(this, name, tr("Due to changes in the database system QMapShack forgot about the filename of your database '%1'. You have to select it again in the next step.").arg(name), QMessageBox::Ok);
@@ -223,7 +223,7 @@ void CGisListDB::saveDatabaseConfiguration()
             cfg.setValue("port", mysql->getPort());
             cfg.setValue("user", mysql->getUser());
             cfg.setValue("passwd", mysql->getPasswd());
-            cfg.setValue("noPasswd",mysql->hasNoPasswd());
+            cfg.setValue("noPasswd", mysql->hasNoPasswd());
             cfg.endGroup(); // name
         }
     }
@@ -584,22 +584,18 @@ void CGisListDB::slotCopyFolder()
 
     // next we need to get the target folder
     // NOTE: By pre-setting db and host, we limit the selection to the current database
-    quint64 idTarget    = 0;
+    QList<quint64> ids;
     QString db          = folder->getDBName();
     QString host        = folder->getDBHost();
 
-    CSelectDBFolder dlg(idTarget, db, host, this);
-    if(dlg.exec() == QDialog::Rejected)
+
+    CSelectDBFolder dlg(ids, db, host, this);
+    if((dlg.exec() == QDialog::Rejected) || ids.isEmpty())
     {
         return;
     }
 
-    // get a pointer to the parent folder for later use.
-    IDBFolder * target = dbfolder->getFolder(idTarget);
-    if(target == nullptr)
-    {
-        return;
-    }
+    quint64 idTarget = ids[0];
 
     // --- at this point we should have all data to perform the copy without interruption ---
 
@@ -626,7 +622,12 @@ void CGisListDB::slotCopyFolder()
     }
 
     // tell the parent folder to show all changes
-    target->update();
+    IDBFolder * target = dbfolder->getFolder(idTarget);
+    if(target != nullptr)
+    {
+        target->update();
+    }
+
     // tell other clients to show changes
     dbfolder->announceChange();
 }
@@ -651,22 +652,18 @@ void CGisListDB::slotMoveFolder()
 
     // next we need to get the target folder
     // NOTE: By pre-setting db and host, we limit the selection to the current database
-    quint64 idTarget    = 0;
+    QList<quint64> ids;
     QString db          = folder->getDBName();
     QString host        = folder->getDBHost();
 
-    CSelectDBFolder dlg(idTarget, db, host, this);
-    if(dlg.exec() == QDialog::Rejected)
+
+    CSelectDBFolder dlg(ids, db, host, this);
+    if((dlg.exec() == QDialog::Rejected) || ids.isEmpty())
     {
         return;
     }
 
-    // get a pointer to the parent folder for later use.
-    IDBFolder * target = dbfolder->getFolder(idTarget);
-    if(target == nullptr)
-    {
-        return;
-    }
+    quint64 idTarget = ids[0];
 
     // --- at this point we should have all data to perform the copy without interruption ---
 
@@ -689,7 +686,8 @@ void CGisListDB::slotMoveFolder()
             continue;
         }
 
-        if(target->isSiblingFrom(folder))
+        //if(target->isSiblingFrom(folder))
+        if(ids.contains(folder->getId()))
         {
             QMessageBox::warning(this, tr("Bad operation...."), tr("The target folder is a subfolder of the one to move. This will not work."), QMessageBox::Abort);
             continue;
@@ -717,7 +715,11 @@ void CGisListDB::slotMoveFolder()
     }
 
     // tell the parent folder to show all changes
-    target->update();
+    IDBFolder * target = dbfolder->getFolder(idTarget);
+    if(target != nullptr)
+    {
+        target->update();
+    }
     // tell other clients to show changes
     dbfolder->announceChange();
 }
@@ -982,7 +984,7 @@ void CGisListDB::slotSearchDatabase()
     }
 
     isInternalEdit--;
-    dlgSearch = new CSearchDatabase(*db,this);
+    dlgSearch = new CSearchDatabase(*db, this);
     connect(dlgSearch.data(), &CSearchDatabase::sigItemChanged, this, &CGisListDB::slotItemChanged);
     dlgSearch->exec();
     delete dlgSearch;
