@@ -17,32 +17,34 @@
 
 **********************************************************************************************/
 
+#include "canvas/CCanvas.h"
+#include "canvas/CCanvasSelect.h"
 #include "CMainWindow.h"
 #include "device/IDevice.h"
 #include "gis/CGisDatabase.h"
 #include "gis/CGisDraw.h"
 #include "gis/CGisWorkspace.h"
-#include "gis/CSetupFilter.h"
-#include "gis/IGisItem.h"
 #include "gis/db/CDBProject.h"
 #include "gis/db/CSelectDBFolder.h"
 #include "gis/db/CSetupFolder.h"
 #include "gis/gpx/CGpxProject.h"
+#include "gis/IGisItem.h"
 #include "gis/ovl/CGisItemOvlArea.h"
 #include "gis/prj/IGisProject.h"
 #include "gis/qms/CQmsProject.h"
 #include "gis/rte/CCreateRouteFromWpt.h"
 #include "gis/rte/CGisItemRte.h"
 #include "gis/rte/router/IRouter.h"
+#include "gis/search/CGeoSearchWeb.h"
 #include "gis/trk/CCombineTrk.h"
 #include "gis/trk/CGisItemTrk.h"
 #include "gis/wpt/CGisItemWpt.h"
 #include "gis/wpt/CProjWpt.h"
+#include "helpers/CInputDialog.h"
 #include "helpers/CProgressDialog.h"
 #include "helpers/CSelectCopyAction.h"
 #include "helpers/CSelectProjectDialog.h"
 #include "helpers/CSettings.h"
-#include "widgets/CColorChooser.h"
 
 #include <QtWidgets>
 #include <QtXml>
@@ -55,7 +57,7 @@ CGisWorkspace::CGisWorkspace(QMenu *menuProject, QWidget *parent)
     pSelf = this;
     setupUi(this);
 
-    lineFilter->addAction(actionClearFilter,QLineEdit::TrailingPosition);
+    lineFilter->addAction(actionClearFilter, QLineEdit::TrailingPosition);
     lineFilter->addAction(actionSetupFilter, QLineEdit::LeadingPosition);
 
     treeWks->setExternalMenu(menuProject);
@@ -72,9 +74,12 @@ CGisWorkspace::CGisWorkspace(QMenu *menuProject, QWidget *parent)
     connect(treeWks, &CGisListWks::itemSelectionChanged, this, &CGisWorkspace::slotWksItemSelectionChanged);
     connect(treeWks, &CGisListWks::sigItemDeleted, this, &CGisWorkspace::slotWksItemSelectionChanged);
 
+    connect(actionNameOnly, &QAction::triggered, this, &CGisWorkspace::slotFilterNameOnly);
+    connect(actionCompleteText, &QAction::triggered, this, &CGisWorkspace::slotFilterCompleteText);
+
     // [Issue #265] Delay the loading of the workspace to make sure the complete IUnit system
     //              is up and running.
-    QTimer::singleShot(500, treeWks, SLOT(slotLoadWorkspace()));
+    QTimer::singleShot(1000, treeWks, SLOT(slotLoadWorkspace()));
 }
 
 CGisWorkspace::~CGisWorkspace()
@@ -167,10 +172,28 @@ void CGisWorkspace::slotFilter(const QString& str)
 
 void CGisWorkspace::slotSetupFilter()
 {
-    CSetupFilter * setupFilter = new CSetupFilter(this);
-    setupFilter->adjustSize();
-    setupFilter->move(lineFilter->geometry().topLeft());
-    setupFilter->show();
+    QMenu * menu = new QMenu(lineFilter);
+    menu->addSection(tr("Apply filter to"));
+    menu->addAction(actionNameOnly);
+    menu->addAction(actionCompleteText);
+
+    QActionGroup* actionGroup = new QActionGroup(menu);
+    actionGroup->addAction(actionNameOnly);
+    actionGroup->addAction(actionCompleteText);
+
+    switch(IGisProject::filterMode)
+    {
+    case IGisProject::eFilterModeName:
+        actionNameOnly->setChecked(true);
+        break;
+
+    case IGisProject::eFilterModeText:
+        actionCompleteText->setChecked(true);
+        break;
+    }
+
+    menu->move(lineFilter->parentWidget()->mapToGlobal(lineFilter->geometry().topLeft()));
+    menu->exec();
 }
 
 void CGisWorkspace::slotSaveAll()
@@ -205,7 +228,7 @@ void CGisWorkspace::slotWksItemSelectionChanged()
 
 void CGisWorkspace::slotWksItemPressed(QTreeWidgetItem * i)
 {
-    IGisItem * item     = dynamic_cast<IGisItem*>(i);
+    IGisItem * item = dynamic_cast<IGisItem*>(i);
     if(item != nullptr)
     {
         IGisProject * project = item->getParentProject();
@@ -215,6 +238,7 @@ void CGisWorkspace::slotWksItemPressed(QTreeWidgetItem * i)
             for(CCanvas * canvas : CMainWindow::self().getCanvas())
             {
                 canvas->reportStatus("WksSelection", tr("<b>Item Selection: </b>Item selected from workspace list. Click on the map to switch back to normal mouse selection behavior."));
+                canvas->abortMouse();
             }
         }
     }
@@ -230,19 +254,85 @@ void CGisWorkspace::slotWksItemSelectionReset()
     for(CCanvas * canvas : CMainWindow::self().getCanvas())
     {
         canvas->reportStatus("WksSelection", "");
+        canvas->abortMouse();
     }
 }
 
-
-IGisProject * CGisWorkspace::selectProject()
+void CGisWorkspace::slotFilterNameOnly(bool yes)
 {
-    QString key, name;
+    if(yes)
+    {
+        IGisProject::filterMode = IGisProject::eFilterModeName;
+        applyFilter();
+    }
+}
+
+void CGisWorkspace::slotFilterCompleteText(bool yes)
+{
+    if(yes)
+    {
+        IGisProject::filterMode = IGisProject::eFilterModeText;
+        applyFilter();
+    }
+}
+
+void CGisWorkspace::slotActivityTrkByKey(const QList<IGisItem::key_t>& keys, trkact_t act)
+{
+    if(keys.isEmpty())
+    {
+        return;
+    }
+
+    if(CTrackData::trkpt_t::eAct20Bad != act)
+    {
+        QMutexLocker lock(&IGisItem::mutexItems);
+
+        QSet<IGisProject*> projects;
+        for(const IGisItem::key_t& key : keys)
+        {
+            CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(key));
+            if(trk == nullptr)
+            {
+                continue;
+            }
+
+            IGisProject * project = trk->getParentProject();
+            if(!projects.contains(project))
+            {
+                project->blockUpdateItems(true);
+                projects << project;
+            }
+
+            if(trk->isRangeSelected())
+            {
+                trk->setActivityRange(act);
+            }
+            else
+            {
+                trk->setActivity(act);
+            }
+        }
+
+        for(IGisProject * project : projects)
+        {
+            project->blockUpdateItems(false);
+        }
+    }
+}
+
+IGisProject * CGisWorkspace::selectProject(bool forceSelect)
+{
+    QString key = IGisProject::getUserFocus();
+    QString name;
     IGisProject::type_e type = IGisProject::eTypeQms;
 
-    CSelectProjectDialog dlg(key, name, type, treeWks);
-    if(dlg.exec() == QDialog::Rejected)
+    if(key.isEmpty() || forceSelect)
     {
-        return nullptr;
+        CSelectProjectDialog dlg(key, name, type, treeWks);
+        if(dlg.exec() == QDialog::Rejected)
+        {
+            return nullptr;
+        }
     }
 
     IGisProject *project = nullptr;
@@ -264,13 +354,13 @@ IGisProject * CGisWorkspace::selectProject()
     }
     else if(type == IGisProject::eTypeDb)
     {
-        quint64 idParent;
+        QList<quint64> ids;
         QString db;
         QString host;
         IDBFolder::type_e type;
 
-        CSelectDBFolder dlg1(idParent, db, host, this);
-        if(dlg1.exec() == QDialog::Rejected)
+        CSelectDBFolder dlg1(ids, db, host, this);
+        if((dlg1.exec() == QDialog::Rejected) || ids.isEmpty())
         {
             return nullptr;
         }
@@ -282,7 +372,7 @@ IGisProject * CGisWorkspace::selectProject()
         }
 
         QMutexLocker lock(&IGisItem::mutexItems);
-        CEvtW2DCreate evt(name, type, idParent, db, host);
+        CEvtW2DCreate evt(name, type, ids[0], db, host);
         CGisDatabase::self().sendEventForDb(&evt);
 
         if(evt.idChild)
@@ -601,7 +691,7 @@ void CGisWorkspace::copyItemByKey(const IGisItem::key_t &key)
         return;
     }
 
-    IGisProject *project = selectProject();
+    IGisProject *project = selectProject(true);
     if(nullptr == project)
     {
         return;
@@ -618,7 +708,7 @@ void CGisWorkspace::copyItemsByKey(const QList<IGisItem::key_t> &keys)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
 
-    IGisProject * project = selectProject();
+    IGisProject * project = selectProject(true);
     if(nullptr == project)
     {
         return;
@@ -643,26 +733,108 @@ void CGisWorkspace::copyItemsByKey(const QList<IGisItem::key_t> &keys)
     CCanvas::triggerCompleteUpdate(CCanvas::eRedrawGis);
 }
 
+void CGisWorkspace::searchWebByKey(const IGisItem::key_t &key)
+{
+    QMutexLocker lock(&IGisItem::mutexItems);
+
+    CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(getItemByKey(key));
+    if(wpt != nullptr)
+    {
+        CGeoSearchWeb::self().getMenu(wpt->getPosition(), this, true);
+    }
+}
+
 void CGisWorkspace::changeWptSymByKey(const QList<IGisItem::key_t>& keys, const QString& sym)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
 
     PROGRESS_SETUP(tr("Change waypoint symbols."), 0, keys.count(), this);
     int cnt = 0;
+
+    QSet<IGisProject*> projects;
     for(const IGisItem::key_t& key : keys)
     {
         PROGRESS(cnt++, break);
         CGisItemWpt *wpt = dynamic_cast<CGisItemWpt*>(getItemByKey(key));
         if(nullptr != wpt)
         {
+            IGisProject * project = wpt->getParentProject();
+            if(!projects.contains(project))
+            {
+                project->blockUpdateItems(true);
+                projects << project;
+            }
             wpt->setIcon(sym);
         }
     }
 
+    for(IGisProject * project : projects)
+    {
+        project->blockUpdateItems(false);
+    }
+
     emit sigChanged();
 }
 
 
+void CGisWorkspace::addEleToWptTrkByKey(const QList<IGisItem::key_t>& keys)
+{
+    CCanvas * canvas = nullptr;
+    CCanvasSelect dlg(canvas, this);
+    dlg.exec();
+
+    if(canvas == nullptr)
+    {
+        return;
+    }
+
+    QMutexLocker lock(&IGisItem::mutexItems);
+
+    QSet<IGisProject*> projects;
+    for(const IGisItem::key_t& key : keys)
+    {
+        IGisItem * item = dynamic_cast<IGisItem*>(getItemByKey(key));
+
+        IGisProject * project = item->getParentProject();
+        if(!projects.contains(project))
+        {
+            project->blockUpdateItems(true);
+            projects << project;
+        }
+
+        switch(item->type())
+        {
+        case IGisItem::eTypeTrk:
+        {
+            CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(item);
+            if(trk != nullptr)
+            {
+                trk->filterReplaceElevation(canvas);
+            }
+            break;
+        }
+
+        case IGisItem::eTypeWpt:
+        {
+            CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(item);
+            if(wpt != nullptr)
+            {
+                qreal ele = canvas->getElevationAt(wpt->getPosition() * DEG_TO_RAD);
+                wpt->setElevation(ele == NOFLOAT ? NOINT : ele);
+            }
+            break;
+        }
+        }
+    }
+
+    for(IGisProject * project : projects)
+    {
+        project->blockUpdateItems(false);
+    }
+
+    emit sigChanged();
+}
+
 void CGisWorkspace::projWptByKey(const IGisItem::key_t& key)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
@@ -746,28 +918,31 @@ void CGisWorkspace::editWptRadius(const IGisItem::key_t &key)
     }
 }
 
-void CGisWorkspace::addWptByPos(QPointF pt, const QString& label, const QString& desc) const
+void CGisWorkspace::copyWptCoordByKey(const IGisItem::key_t &key)
 {
-    QString name = label;
-    QString icon;
-    if(!CGisItemWpt::getNewWptData(pt, icon, name))
+    QMutexLocker lock(&IGisItem::mutexItems);
+    CGisItemWpt *wpt = dynamic_cast<CGisItemWpt*>(getItemByKey(key));
+    if(nullptr != wpt)
     {
-        return;
+        QString strPos;
+        QPointF pos = wpt->getPosition();
+        IUnit::degToStr(pos.x(), pos.y(), strPos);
+        QClipboard *clipboard = QApplication::clipboard();
+        clipboard->setText(strPos);
     }
+}
 
-    IGisProject * project = CGisWorkspace::self().selectProject();
+void CGisWorkspace::addWptByPos(QPointF pt, const QString& name, const QString& desc) const
+{
+    QMutexLocker lock(&IGisItem::mutexItems);
+
+    IGisProject * project = CGisWorkspace::self().selectProject(false);
     if(nullptr == project)
     {
         return;
     }
 
-    QMutexLocker lock(&IGisItem::mutexItems);
-    CGisItemWpt * wpt = new CGisItemWpt(pt, name, icon, project);
-    if(!desc.isEmpty())
-    {
-        wpt->setDescription(desc);
-    }
-    wpt->edit();
+    CGisItemWpt::newWpt(pt, name, desc, project);
 }
 
 void CGisWorkspace::focusTrkByKey(bool yes, const IGisItem::key_t& key)
@@ -825,6 +1000,19 @@ void CGisWorkspace::cutTrkByKey(const IGisItem::key_t& key)
     emit sigChanged();
 }
 
+void CGisWorkspace::addTrkInfoByKey(const IGisItem::key_t& key)
+{
+    QMutexLocker lock(&IGisItem::mutexItems);
+
+    CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(key));
+    if(nullptr != trk)
+    {
+        trk->addTrkPtDesc();
+    }
+
+    emit sigChanged();
+}
+
 void CGisWorkspace::reverseTrkByKey(const IGisItem::key_t& key)
 {
     QMutexLocker lock(&IGisItem::mutexItems);
@@ -889,28 +1077,6 @@ void CGisWorkspace::combineTrkByKey(const QList<IGisItem::key_t>& keys, const QL
     emit sigChanged();
 }
 
-void CGisWorkspace::activityTrkByKey(const QList<IGisItem::key_t>& keys)
-{
-    if(keys.isEmpty())
-    {
-        return;
-    }
-
-    quint32 flags = CActivityTrk::selectActivity(this);
-    if(0xFFFFFFFF != flags)
-    {
-        QMutexLocker lock(&IGisItem::mutexItems);
-        for(const IGisItem::key_t& key : keys)
-        {
-            CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(key));
-            if(trk != nullptr)
-            {
-                trk->setActivity(flags);
-            }
-        }
-    }
-}
-
 void CGisWorkspace::colorTrkByKey(const QList<IGisItem::key_t>& keys)
 {
     if(keys.isEmpty())
@@ -918,18 +1084,32 @@ void CGisWorkspace::colorTrkByKey(const QList<IGisItem::key_t>& keys)
         return;
     }
 
-    qint32 colorIdx = CColorChooser::selectColor(this);
+    qint32 colorIdx = IGisItem::selectColor(this);
     if(colorIdx != NOIDX)
     {
         QMutexLocker lock(&IGisItem::mutexItems);
+
+        QSet<IGisProject*> projects;
         for(const IGisItem::key_t& key : keys)
         {
             CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(getItemByKey(key));
             if(trk != nullptr)
             {
+                IGisProject * project = trk->getParentProject();
+                if(!projects.contains(project))
+                {
+                    project->blockUpdateItems(true);
+                    projects << project;
+                }
+
                 trk->setColor(colorIdx);
             }
         }
+
+        for(IGisProject * project : projects)
+        {
+            project->blockUpdateItems(false);
+        }
     }
 }
 
@@ -1074,6 +1254,34 @@ void CGisWorkspace::makeRteFromWpt(const QList<IGisItem::key_t>& keys)
     dlg.exec();
 }
 
+void CGisWorkspace::editPrxWpt(const QList<IGisItem::key_t>& keys)
+{
+    QMutexLocker lock(&IGisItem::mutexItems);
+
+    QVariant var;
+    CInputDialog dlg(this, tr("Enter new proximity range."), var, QVariant(NOFLOAT), IUnit::self().baseunit);
+    dlg.setOption(tr("Is no-go area"), false);
+    if(dlg.exec() != QDialog::Accepted)
+    {
+        return;
+    }
+
+    qreal proximity = var.toDouble()/IUnit::self().basefactor;
+    bool isNoGo = dlg.optionIsChecked();
+    for(const IGisItem::key_t& key : keys)
+    {
+        CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(getItemByKey(key));
+        if(wpt != nullptr)
+        {
+            wpt->setProximity(proximity);
+            if(wpt->isNogo() != isNoGo)
+            {
+                wpt->setNogo(isNoGo);
+            }
+        }
+    }
+}
+
 
 void CGisWorkspace::draw(QPainter& p, const QPolygonF& viewport, CGisDraw * gis)
 {
