@@ -136,9 +136,9 @@ public:
     enum mark_e
     {
         eMarkNone      = 0
-        ,eMarkChanged   = 0x00000001
-        ,eMarkNotPart   = 0x00000002
-        ,eMarkNotInDB   = 0x00000004
+        , eMarkChanged   = 0x00000001
+        , eMarkNotPart   = 0x00000002
+        , eMarkNotInDB   = 0x00000004
     };
 
     enum selection_e
@@ -154,6 +154,27 @@ public:
 
     using selflags_t = quint32;
 
+    enum color_e
+    {
+        eColorBlack         = 0
+        , eColorDarkRed     = 1
+        , eColorDarkGreen   = 2
+        , eColorDarkYellow  = 3
+        , eColorDarkBlue    = 4
+        , eColorDarkMagenta = 5
+        , eColorDarkCyan    = 6
+        , eColorLightGray   = 7
+        , eColorDarkGray    = 8
+        , eColorRed         = 9
+        , eColorGreen       = 10
+        , eColorYellow      = 11
+        , eColorBlue        = 12
+        , eColorMagenta     = 13
+        , eColorCyan        = 14
+        , eColorWhite       = 15
+        , eColorTransparent = 16
+    };
+
     struct key_t
     {
         bool operator==(const key_t& k) const
@@ -181,6 +202,10 @@ public:
     /// this mutex has to be locked when ever the item list is accessed.
     static QMutex mutexItems;
 
+    static void init();
+    static QMenu * getColorMenu(const QString &title, QObject *obj, const char *slot, QWidget * parent);
+    static qint32 selectColor(QWidget * parent);
+
     /**
        @brief If the item is part of a database project it will update itself with the database content
      */
@@ -264,6 +289,7 @@ public:
         , eFeatureShowFullText  = 0x00000002
         , eFeatureShowActivity  = 0x00000004
         , eFeatureShowDateTime  = 0x00000008
+        , eFeatureShowLinks     = 0x00000010
     };
 
     /**
@@ -391,7 +417,7 @@ public:
     virtual void drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis)
     {
     }
-    virtual void drawLabel(QPainter& p, const QPolygonF& viewport,QList<QRectF>& blockedAreas, const QFontMetricsF& fm, CGisDraw * gis) = 0;
+    virtual void drawLabel(QPainter& p, const QPolygonF& viewport, QList<QRectF>& blockedAreas, const QFontMetricsF& fm, CGisDraw * gis) = 0;
     virtual void drawHighlight(QPainter& p) = 0;
 
     virtual void gainUserFocus(bool yes) = 0;
@@ -452,6 +478,13 @@ public:
      */
     void cutHistoryBefore();
 
+    /**
+       @brief Take data of the most recent entry and apply meta information of first one
+
+       All other entries are lost
+     */
+    void squashHistory();
+
     /**
        @brief Create a clone of itself and pass back the pointer
 
@@ -538,13 +571,17 @@ public:
 
     struct color_t
     {
-        const char   *name;
+        const char   *  name;
+        const QString label;
         const QColor color;
         const QString bullet;
+        const QString line;
     };
 
-    static const color_t colorMap[];
-    static const size_t colorMapSize;
+    static const QVector<color_t>& getColorMap()
+    {
+        return colorMap;
+    }
 
 protected:
     /// set icon of QTreeWidgetItem
@@ -601,12 +638,13 @@ protected:
     enum flags_e
     {
         eFlagCreatedInQms   = 0x00000001
-        ,eFlagWriteAllowed  = 0x00000002
-        ,eFlagTainted       = 0x00000004
-        ,eFlagWptBubble     = 0x00000100
-        ,eFlagNogo          = 0x00000200
+        , eFlagWriteAllowed  = 0x00000002
+        , eFlagTainted       = 0x00000004
+        , eFlagWptBubble     = 0x00000100
+        , eFlagNogo          = 0x00000200
     };
 
+    static QVector<color_t> colorMap;
 private:
 
     void showIcon();
