@@ -27,6 +27,7 @@
 #include "gis/wpt/CGisItemWpt.h"
 #include "helpers/CLinksDialog.h"
 #include "helpers/CProgressDialog.h"
+#include "helpers/CWptIconManager.h"
 #include "helpers/Signals.h"
 #include "plot/CPlotProfile.h"
 #include "plot/CPlotTrack.h"
@@ -87,17 +88,21 @@ void CDetailsPrj::resizeEvent(QResizeEvent * e)
     timerUpdateTime->start();
 }
 
-void CDetailsPrj::getTrackProfile(CGisItemTrk * trk, QImage& image)
+void CDetailsPrj::getTrackProfile(CGisItemTrk * trk,
+                                  const CTrackData::trkpt_t * pTrkpt,
+                                  QImage& image)
 {
     CPlotProfile plot(trk, trk->limitsGraph1, IPlot::eModeIcon, this);
     plot.setSolid(true);
-    plot.save(image);
+    plot.save(image, pTrkpt);
 }
 
-void CDetailsPrj::getTrackOverview(CGisItemTrk * trk, QImage& image)
+void CDetailsPrj::getTrackOverview(CGisItemTrk * trk,
+                                   const CTrackData::trkpt_t * pTrkpt,
+                                   QImage& image)
 {
     CPlotTrack plot(trk, this);
-    plot.save(image);
+    plot.save(image, pTrkpt);
 }
 
 
@@ -195,9 +200,9 @@ void CDetailsPrj::draw(QTextDocument& doc, bool printable)
     int w = doc.textWidth();
     int nItems = 0;
 
-    QFontMetrics fm(QFont(font().family(),12));
+    QFontMetrics fm(QFont(font().family(), 12));
     int pointSize = ((10 * (w - 2 * ROOT_FRAME_MARGIN)) / (CHAR_PER_LINE *  fm.width("X")));
-    pointSize = qMax(pointSize,CMainWindow::self().getMapFont().pointSize());
+    pointSize = qMax(pointSize, CMainWindow::self().getMapFont().pointSize());
 
     QFont f = textDesc->font();
     f.setPointSize(pointSize);
@@ -207,7 +212,7 @@ void CDetailsPrj::draw(QTextDocument& doc, bool printable)
     fmtFrameStandard.setBottomMargin(5);
     fmtFrameStandard.setWidth(w - 2 * ROOT_FRAME_MARGIN);
 
-    fmtFrameTrackSummary.setBackground(Qt::white);
+    fmtFrameTrackSummary.setBackground(palette().color(QPalette::Background));
     fmtFrameTrackSummary.setBorder(1);
     fmtFrameTrackSummary.setPadding(10);
 
@@ -323,25 +328,39 @@ void CDetailsPrj::draw(QTextDocument& doc, bool printable)
         cursor1.setCharFormat(fmtCharStandard);
         cursor1.setBlockFormat(fmtBlockStandard);
 
-        QTextTable * table = cursor1.insertTable(1, 2, fmtTableHidden);
+        QTextTable * table = cursor1.insertTable(2, 2, fmtTableHidden);
 
-        QTextCursor cursor2 = table->cellAt(0,0).firstCursorPosition();
+        QTextCursor cursor2 = table->cellAt(0, 0).firstCursorPosition();
         drawInfo(cursor2, isReadOnly);
 
         if(prj.getItemCountByType(IGisItem::eTypeTrk) != 0)
         {
-            QTextCursor cursor3 = table->cellAt(0,1).firstCursorPosition();
+            QTextCursor cursor3 = table->cellAt(0, 1).firstCursorPosition();
             drawTrackSummary(cursor3, trks, isReadOnly);
         }
+
+        if(!wpts.isEmpty())
+        {
+            QTextCursor cursor3 = table->cellAt(1, 1).firstCursorPosition();
+            drawWaypointSummary(cursor3, wpts, isReadOnly);
+        }
     }
 
 
     int n=1;
     PROGRESS_SETUP(tr("Build diary..."), 0, nItems, this);
 
-    if(comboSort->currentIndex() > IGisProject::eSortRoadbookNone)
+    IGisProject::sorting_roadbook_e sorting = IGisProject::sorting_roadbook_e(comboSort->currentIndex());
+    if(sorting > IGisProject::eSortRoadbookNone)
     {
-        drawByTrack(cursor, trks, wpts, progress, n, isReadOnly);
+        if(sorting == IGisProject::eSortRoadbookTrackWithDetails)
+        {
+            drawByDetails(cursor, trks, wpts, progress, n, isReadOnly);
+        }
+        else
+        {
+            drawByTrack(cursor, trks, wpts, progress, n, isReadOnly);
+        }
     }
     else
     {
@@ -399,18 +418,76 @@ void CDetailsPrj::drawTrackSummary(QTextCursor& cursor, const QList<CGisItemTrk*
     cursor1.insertHtml(str);
 }
 
-void CDetailsPrj::addIcon(QTextTable * table, int col, int row, IGisItem * item, bool printable)
+void CDetailsPrj::drawWaypointSummary(QTextCursor& cursor, const QList<CGisItemWpt*> wpts, bool /*isReadOnly*/)
 {
-    table->cellAt(row,col).firstCursorPosition().insertImage(item->getDisplayIcon().toImage().scaledToWidth(16, Qt::SmoothTransformation));
-
-    if(!(printable||item->isReadOnly()))
+    QMap<QString, quint32> summary;
+    QMap<QString, quint32> GCsummary;
+    for(const CGisItemWpt* wpt : wpts)
     {
-        table->cellAt(row,col).lastCursorPosition().insertHtml(QString("<p><a href='edit?key=%1'><img src='://icons/16x16/EditDetails.png'/></a></p>").arg(item->getKey().item));
+        auto iconName = wpt->getIconName();
+        if(iconName.isEmpty())
+        {
+            summary["Waypoint"]++;
+        }
+        else
+        {
+            summary[iconName]++;
+        }
+
+        const CGisItemWpt::geocache_t& gc =wpt->getGeoCache();
+        if(gc.hasData)
+        {
+            GCsummary[gc.type]++;
+        }
+    }
+
+    QTextFrame * diaryFrame = cursor.insertFrame(fmtFrameTrackSummary);
+
+    QTextCursor cursor1(diaryFrame);
+
+    cursor1.setCharFormat(fmtCharStandard);
+    cursor1.setBlockFormat(fmtBlockStandard);
+
+    QString str;
+    str += tr("<b>Summary over all waypoints in project</b><br/>");
+
+    if(summary["Geocache"] != 0)
+    {
+        str+= QString::number(summary["Geocache"]) + tr(" x Geocache, consisting of: <br/>");
+        str+="<ul>";
+        for(auto key:GCsummary.keys())
+        {
+            str+= "<li>" + QString::number(GCsummary[key]) + " x " + key+ "</li>";
+        }
+        str+="</ul>";
+    }
+    for(auto key: summary.keys())
+    {
+        if(key != "Geocache")
+        {
+            str+= QString::number(summary[key]) + " x " + key+ "<br/>";
+        }
+    }
+
+    cursor1.insertHtml(str);
+}
+
+void CDetailsPrj::addIcon(QTextTable * table, int col, int row, const QPixmap& icon, const QString& key, bool isReadOnly, bool printable)
+{
+    table->cellAt(row, col).firstCursorPosition().insertImage(icon.toImage().scaledToWidth(16, Qt::SmoothTransformation));
+
+    if(!(printable || isReadOnly))
+    {
+        table->cellAt(row, col).lastCursorPosition().insertHtml(QString("<p><a href='edit?key=%1'><img src='://icons/16x16/EditDetails.png'/></a></p>").arg(key));
     }
 }
 
 
-void CDetailsPrj::drawByGroup(QTextCursor &cursor, QList<CGisItemTrk*>& trks, QList<CGisItemWpt*>& wpts, CProgressDialog& progress, int& n, bool printable)
+void CDetailsPrj::drawByGroup(QTextCursor &cursor,
+                              QList<CGisItemTrk*>& trks,
+                              QList<CGisItemWpt*>& wpts,
+                              CProgressDialog& progress,
+                              int& n, bool printable)
 {
     int cnt, w = cursor.document()->textWidth();
 
@@ -419,21 +496,21 @@ void CDetailsPrj::drawByGroup(QTextCursor &cursor, QList<CGisItemTrk*>& trks, QL
         cursor.insertHtml(tr("<h2>Waypoints</h2>"));
         QTextTable * table = cursor.insertTable(wpts.count()+1, eMax1, fmtTableStandard);
 
-        table->cellAt(0,eSym1).setFormat(fmtCharHeader);
-        table->cellAt(0,eInfo1).setFormat(fmtCharHeader);
-        table->cellAt(0,eComment1).setFormat(fmtCharHeader);
+        table->cellAt(0, eSym1).setFormat(fmtCharHeader);
+        table->cellAt(0, eInfo1).setFormat(fmtCharHeader);
+        table->cellAt(0, eComment1).setFormat(fmtCharHeader);
 
-        table->cellAt(0,eInfo1).firstCursorPosition().insertText(tr("Info"));
-        table->cellAt(0,eComment1).firstCursorPosition().insertText(tr("Comment"));
+        table->cellAt(0, eInfo1).firstCursorPosition().insertText(tr("Info"));
+        table->cellAt(0, eComment1).firstCursorPosition().insertText(tr("Comment"));
 
         cnt = 1;
         for(CGisItemWpt * wpt : wpts)
         {
             PROGRESS(n++, return );
 
-            addIcon(table, eSym1, cnt, wpt, printable);
-            table->cellAt(cnt,eInfo1).firstCursorPosition().insertHtml(wpt->getInfo(IGisItem::eFeatureShowName));
-            table->cellAt(cnt,eComment1).firstCursorPosition().insertHtml(IGisItem::createText(wpt->isReadOnly()||printable, wpt->getComment(), wpt->getDescription(), wpt->getLinks(), wpt->getKey().item));
+            addIcon(table, eSym1, cnt, wpt->getDisplayIcon(), wpt->getKey().item, wpt->isReadOnly(), printable);
+            table->cellAt(cnt, eInfo1).firstCursorPosition().insertHtml(wpt->getInfo(IGisItem::eFeatureShowName));
+            table->cellAt(cnt, eComment1).firstCursorPosition().insertHtml(IGisItem::createText(wpt->isReadOnly() || printable, wpt->getComment(), wpt->getDescription(), wpt->getLinks(), wpt->getKey().item));
             cnt++;
         }
 
@@ -445,12 +522,12 @@ void CDetailsPrj::drawByGroup(QTextCursor &cursor, QList<CGisItemTrk*>& trks, QL
         cursor.insertHtml(tr("<h2>Tracks</h2>"));
         QTextTable * table = cursor.insertTable(trks.count()+1, eMax1, fmtTableStandard);
 
-        table->cellAt(0,eSym1).setFormat(fmtCharHeader);
-        table->cellAt(0,eInfo1).setFormat(fmtCharHeader);
-        table->cellAt(0,eComment1).setFormat(fmtCharHeader);
+        table->cellAt(0, eSym1).setFormat(fmtCharHeader);
+        table->cellAt(0, eInfo1).setFormat(fmtCharHeader);
+        table->cellAt(0, eComment1).setFormat(fmtCharHeader);
 
-        table->cellAt(0,eInfo1).firstCursorPosition().insertText(tr("Info"));
-        table->cellAt(0,eComment1).firstCursorPosition().insertText(tr("Comment"));
+        table->cellAt(0, eInfo1).firstCursorPosition().insertText(tr("Info"));
+        table->cellAt(0, eComment1).firstCursorPosition().insertText(tr("Comment"));
 
         cnt = 1;
 
@@ -458,41 +535,41 @@ void CDetailsPrj::drawByGroup(QTextCursor &cursor, QList<CGisItemTrk*>& trks, QL
         {
             PROGRESS(n++, return );
 
-            addIcon(table, eSym1, cnt, trk, printable);
+            addIcon(table, eSym1, cnt, trk->getDisplayIcon(), trk->getKey().item, trk->isReadOnly(), printable);
 
             int w1 = qRound(w/3.5 > 300 ? 300 : w/3.5);
             int h1 = qRound(w1/2.0);
 
             if(w1 < 300)
             {
-                table->cellAt(cnt,eInfo1).firstCursorPosition().insertHtml(trk->getInfo(IGisItem::eFeatureShowName|IGisItem::eFeatureShowActivity));
+                table->cellAt(cnt, eInfo1).firstCursorPosition().insertHtml(trk->getInfo(IGisItem::eFeatureShowName|IGisItem::eFeatureShowActivity));
 
-                QTextTable * table1 = table->cellAt(cnt,eInfo1).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
+                QTextTable * table1 = table->cellAt(cnt, eInfo1).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
 
-                QImage profile(w1,h1,QImage::Format_ARGB32);
-                getTrackProfile(trk, profile);
-                table1->cellAt(0,0).firstCursorPosition().insertImage(profile);
+                QImage profile(w1, h1, QImage::Format_ARGB32);
+                getTrackProfile(trk, nullptr, profile);
+                table1->cellAt(0, 0).firstCursorPosition().insertImage(profile);
 
-                QImage overview(h1,h1,QImage::Format_ARGB32);
-                getTrackOverview(trk, overview);
-                table1->cellAt(0,1).firstCursorPosition().insertImage(overview);
+                QImage overview(h1, h1, QImage::Format_ARGB32);
+                getTrackOverview(trk, nullptr, overview);
+                table1->cellAt(0, 1).firstCursorPosition().insertImage(overview);
             }
             else
             {
-                QTextTable * table1 = table->cellAt(cnt,eInfo1).firstCursorPosition().insertTable(1, 3, fmtTableInfo);
+                QTextTable * table1 = table->cellAt(cnt, eInfo1).firstCursorPosition().insertTable(1, 3, fmtTableInfo);
 
-                table1->cellAt(0,0).firstCursorPosition().insertHtml(trk->getInfo(IGisItem::eFeatureShowName|IGisItem::eFeatureShowActivity));
+                table1->cellAt(0, 0).firstCursorPosition().insertHtml(trk->getInfo(IGisItem::eFeatureShowName|IGisItem::eFeatureShowActivity));
 
-                QImage profile(w1,h1,QImage::Format_ARGB32);
-                getTrackProfile(trk, profile);
-                table1->cellAt(0,1).firstCursorPosition().insertImage(profile);
+                QImage profile(w1, h1, QImage::Format_ARGB32);
+                getTrackProfile(trk, nullptr, profile);
+                table1->cellAt(0, 1).firstCursorPosition().insertImage(profile);
 
-                QImage overview(h1,h1,QImage::Format_ARGB32);
-                getTrackOverview(trk, overview);
-                table1->cellAt(0,2).firstCursorPosition().insertImage(overview);
+                QImage overview(h1, h1, QImage::Format_ARGB32);
+                getTrackOverview(trk, nullptr, overview);
+                table1->cellAt(0, 2).firstCursorPosition().insertImage(overview);
             }
 
-            table->cellAt(cnt,eComment1).firstCursorPosition().insertHtml(IGisItem::createText(trk->isReadOnly()||printable, trk->getComment(), trk->getDescription(), trk->getLinks(), trk->getKey().item));
+            table->cellAt(cnt, eComment1).firstCursorPosition().insertHtml(IGisItem::createText(trk->isReadOnly() || printable, trk->getComment(), trk->getDescription(), trk->getLinks(), trk->getKey().item));
 
             cnt++;
         }
@@ -505,6 +582,15 @@ void CDetailsPrj::drawByGroup(QTextCursor &cursor, QList<CGisItemTrk*>& trks, QL
 struct wpt_info_t
 {
     IGisItem::key_t key;
+
+    bool isReadOnly = true;
+    QString desc;
+    QString cmt;
+    QString info;
+    QList<CGisItemWpt::image_t> images;
+    QList<IGisItem::link_t> links;
+    QPixmap icon;
+
     qreal distance1 = NOFLOAT;
     qreal ascent1 = NOFLOAT;
     qreal elapsedSeconds1 = 0;
@@ -519,183 +605,324 @@ struct wpt_info_t
     qreal elapsedSeconds3 = 0;
     qreal ascent3 = NOFLOAT;
     qreal descent3 = NOFLOAT;
+
+    const CTrackData::trkpt_t * pTrkpt = nullptr;
 };
 
-void CDetailsPrj::drawByTrack(QTextCursor& cursor, QList<CGisItemTrk *> &trks, QList<CGisItemWpt *> &wpts, CProgressDialog &progress, int &n, bool printable)
-{
-    int cnt, w = cursor.document()->textWidth();
 
+QList<wpt_info_t> CDetailsPrj::getWptInfo(const CGisItemTrk& trk) const
+{
+    int cnt = 1;
+    const CTrackData::trkpt_t* lastTrkpt = nullptr;
+    QList<wpt_info_t> wptInfo;
+    wpt_info_t * lastWptInfo = nullptr;
+    bool hasValidTime = trk.getTimeStart().isValid();
+
+    const CTrackData& t = trk.getTrackData();
+    for(const CTrackData::trkpt_t& trkpt : t)
+    {
+        if(trkpt.isHidden() || (trkpt.keyWpt.item.isEmpty() && trkpt.desc.isEmpty()))
+        {
+            continue;
+        }
+
+        wptInfo << wpt_info_t();
+        wpt_info_t& info = wptInfo.last();
+        info.pTrkpt = &trkpt;
+        if(!trkpt.keyWpt.item.isEmpty())
+        {
+            CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(prj.getItemByKey(trkpt.keyWpt));
+            if(wpt != nullptr)
+            {
+                info.key        = wpt->getKey();
+                info.isReadOnly = wpt->isReadOnly();
+                info.icon       = wpt->getDisplayIcon();
+                info.desc       = wpt->getDescription();
+                info.cmt        = wpt->getComment();
+                info.links      = wpt->getLinks();
+                info.images     = wpt->getImages();
+                if(hasValidTime)
+                {
+                    info.info = wpt->getInfo(IGisItem::eFeatureShowName);
+                }
+                else
+                {
+                    info.info = wpt->getInfo(IGisItem::eFeatureShowName | IGisItem::eFeatureShowDateTime);
+                }
+            }
+            else
+            {
+                wptInfo.pop_back();
+                continue;
+            }
+        }
+        else if(!trkpt.desc.isEmpty())
+        {
+            info.info = "<b>" + trkpt.desc + "</b>";
+            if(trkpt.ele != NOINT)
+            {
+                QString val, unit;
+                IUnit::self().meter2elevation(trkpt.ele, val, unit);
+                info.info += "<br/>" + tr("Elevation: %1%2").arg(val).arg(unit);
+            }
+
+            if(!hasValidTime && trkpt.time.isValid())
+            {
+                info.info += "<br/>" + tr("Created: %1").arg(IUnit::datetime2string(trkpt.time, false, QPointF(trkpt.lon*DEG_TO_RAD, trkpt.lat*DEG_TO_RAD)));
+            }
+
+            CWptIconManager& wptMgr = CWptIconManager::self();
+            info.icon = wptMgr.loadIcon(wptMgr.getNumberedBullet(cnt++));
+        }
+        else
+        {
+            wptInfo.pop_back();
+            continue;
+        }
+        info.distance1  = trkpt.distance;
+        info.elapsedSeconds1 = trkpt.elapsedSeconds;
+        info.ascent1    = trkpt.ascent;
+        info.descent1   = trkpt.descent;
+
+        if(lastWptInfo != nullptr)
+        {
+            lastWptInfo->distance2  = trkpt.distance - lastTrkpt->distance;
+            lastWptInfo->elapsedSeconds2  = trkpt.elapsedSeconds - lastTrkpt->elapsedSeconds;
+            lastWptInfo->ascent2    = trkpt.ascent   - lastTrkpt->ascent;
+            lastWptInfo->descent2   = trkpt.descent  - lastTrkpt->descent;
+        }
+
+        info.distance3  = trk.getTotalDistance() - trkpt.distance;
+        info.elapsedSeconds3  = trk.getTotalElapsedSeconds() - trkpt.elapsedSeconds;
+        info.ascent3    = trk.getTotalAscent() - trkpt.ascent;
+        info.descent3   = trk.getTotalDescent() - trkpt.descent;
+
+        lastTrkpt       = &trkpt;
+        lastWptInfo     = &wptInfo.last();
+    }
+
+    return wptInfo;
+}
+
+QString CDetailsPrj::getNameAndTime(const wpt_info_t &info, const CGisItemTrk& trk) const
+{
+    QString str;
+    QDateTime arrivalTime = trk.getTimeStart();
+    if (arrivalTime.isValid())
+    {
+        str += info.info + "<br/>\n" +
+               tr("Arrival: ") + QString("%1").arg(IUnit::datetime2string(arrivalTime.addSecs(info.elapsedSeconds1), false));
+    }
+    else
+    {
+        str = info.info;
+    }
+
+    return str;
+}
+
+QString CDetailsPrj::getStatistics(const wpt_info_t &info) const
+{
+    QString text, val, unit;
+    text += "<table sytle='border=1px;'>";
+    text += "<tr><td></td><td><nobr>&nbsp;" + tr("From Start") + "&nbsp;</nobr></td><td><nobr>&nbsp;" + tr("To Next") + "&nbsp;</nobr></td><td><nobr>&nbsp;" + tr("To End")  + "&nbsp;</nobr></td></tr>";
+
+    text += "<tr>";
+    text += "<td>" + tr("Distance: ") + "</td>";
+    IUnit::self().meter2distance(info.distance1, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+    IUnit::self().meter2distance(info.distance2, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+    IUnit::self().meter2distance(info.distance3, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+    text += "</tr>";
+
+    text += "<tr>";
+    text += "<td>" + tr("Time: ") + "</td>";
+    IUnit::self().seconds2time(info.elapsedSeconds1, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "&nbsp;</td>";
+    IUnit::self().seconds2time(info.elapsedSeconds2, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "&nbsp;</td>";
+    IUnit::self().seconds2time(info.elapsedSeconds3, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "&nbsp;</td>";
+    text += "</tr>";
+
+    text += "<tr>";
+    text += "<td>" + tr("Ascent: ") + "</td>";
+    IUnit::self().meter2elevation(info.ascent1, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+    IUnit::self().meter2elevation(info.ascent2, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+    IUnit::self().meter2elevation(info.ascent3, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+    text += "</tr>";
+
+    text += "<tr>";
+    text += "<td>" + tr("Descent: ") + "</td>";
+    IUnit::self().meter2elevation(info.descent1, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+    IUnit::self().meter2elevation(info.descent2, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+    IUnit::self().meter2elevation(info.descent3, val, unit);
+    text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
+    text += "</tr>";
+
+    text += "</table>";
+
+    return text;
+}
+
+QImage CDetailsPrj::getImage(const wpt_info_t &info) const
+{
+    QImage image(info.images.first().pixmap);
+
+    int w = image.width();
+    int h = image.height();
+
+    if(w < h)
+    {
+        h *= 100.0 / w;
+        w  = 100;
+    }
+    else
+    {
+        h *= 200.0 / w;
+        w  = 200;
+    }
+    qDebug() << w << h;
+    return image.scaled(w, h, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+}
+
+void CDetailsPrj::drawByTrack(QTextCursor& cursor,
+                              QList<CGisItemTrk *> &trks,
+                              QList<CGisItemWpt *> &wpts,
+                              CProgressDialog &progress,
+                              int &n, bool printable)
+{
+    int w = cursor.document()->textWidth();
 
     const qreal w1 = qRound(w/3.5 > 300 ? 300 : w/3.5);
     const qreal h1 = qRound(w1/2.0);
 
-
     for(CGisItemTrk * trk : trks)
     {
-        const CTrackData::trkpt_t* lastTrkpt = nullptr;
-        wpt_info_t * lastWptInfo = nullptr;
-        QList<wpt_info_t> wptInfo;
-        const CTrackData& t = trk->getTrackData();
-        for(const CTrackData::trkpt_t& trkpt : t)
-        {
-            if(trkpt.isHidden() || trkpt.keyWpt.item.isEmpty())
-            {
-                continue;
-            }
-
-            wptInfo << wpt_info_t();
-            wpt_info_t& info = wptInfo.last();
-            info.key        = trkpt.keyWpt;
-            info.distance1  = trkpt.distance;
-            info.elapsedSeconds1 = trkpt.elapsedSeconds;
-            info.ascent1    = trkpt.ascent;
-            info.descent1   = trkpt.descent;
-
-            if(lastWptInfo != nullptr)
-            {
-                lastWptInfo->distance2  = trkpt.distance - lastTrkpt->distance;
-                lastWptInfo->elapsedSeconds2  = trkpt.elapsedSeconds - lastTrkpt->elapsedSeconds;
-                lastWptInfo->ascent2    = trkpt.ascent   - lastTrkpt->ascent;
-                lastWptInfo->descent2   = trkpt.descent  - lastTrkpt->descent;
-            }
-
-            info.distance3  = trk->getTotalDistance() - trkpt.distance;
-            info.elapsedSeconds3  = trk->getTotalElapsedSeconds() - trkpt.elapsedSeconds;
-            info.ascent3    = trk->getTotalAscent() - trkpt.ascent;
-            info.descent3   = trk->getTotalDescent() - trkpt.descent;
-
-            lastTrkpt       = &trkpt;
-            lastWptInfo     = &wptInfo.last();
-        }
+        const QList<wpt_info_t>& wptInfo = getWptInfo(*trk);
 
         cursor.insertHtml(QString("<h2>%1</h2>").arg(trk->getName()));
         QTextTable * table = cursor.insertTable(wptInfo.count()+2, eMax2, fmtTableStandard);
 
-        table->cellAt(0,eSym2).setFormat(fmtCharHeader);
-        table->cellAt(0,eInfo2).setFormat(fmtCharHeader);
-        table->cellAt(0,eData2).setFormat(fmtCharHeader);
-        table->cellAt(0,eComment2).setFormat(fmtCharHeader);
+        table->cellAt(0, eSym2).setFormat(fmtCharHeader);
+        table->cellAt(0, eInfo2).setFormat(fmtCharHeader);
+        table->cellAt(0, eData2).setFormat(fmtCharHeader);
+        table->cellAt(0, eComment2).setFormat(fmtCharHeader);
 
-        table->cellAt(0,eInfo2).firstCursorPosition().insertText(tr("Info"));
-        table->cellAt(0,eComment2).firstCursorPosition().insertText(tr("Comment"));
-
-        cnt = 1;
+        table->cellAt(0, eInfo2).firstCursorPosition().insertText(tr("Info"));
+        table->cellAt(0, eData2).firstCursorPosition().insertText(tr("Statistics"));
+        table->cellAt(0, eComment2).firstCursorPosition().insertText(tr("Comment"));
 
+        int cnt = 1;
         for(const wpt_info_t &info : wptInfo)
         {
             PROGRESS(n++, return );
 
-            CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(prj.getItemByKey(info.key));
-            if(wpt != nullptr)
+            addIcon(table, eSym2, cnt, info.icon, info.key.item, info.isReadOnly, printable);
+            table->cellAt(cnt, eInfo2).firstCursorPosition().insertHtml(getNameAndTime(info, *trk));
+
+            QTextTable * table1 = table->cellAt(cnt, eData2).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
+            table1->cellAt(0, 0).firstCursorPosition().insertHtml(getStatistics(info));
+
+            if(!info.images.isEmpty())
             {
-                addIcon(table, eSym2, cnt, wpt, printable);
-                QString timeStr = "";
-                QDateTime arrivalTime = trk->getTimeStart();
-                if (arrivalTime.isValid())
-                {
-                    timeStr += wpt->getInfo(IGisItem::eFeatureShowName) + "<br/>\n" +
-                               tr("Arrival: ") + QString("%1").arg(IUnit::datetime2string(arrivalTime.addSecs(info.elapsedSeconds1), false));
-                }
-                else
-                {
-                    timeStr = wpt->getInfo(IGisItem::eFeatureShowName | IGisItem::eFeatureShowDateTime);
-                }
-                table->cellAt(cnt,eInfo2).firstCursorPosition().insertHtml(timeStr);
-                QTextTable * table1 = table->cellAt(cnt,eData2).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
-
-                QString text, val, unit;
-                text += "<table sytle='border=1px;'>";
-                text += "<tr><td></td><td><nobr>&nbsp;" + tr("From Start") + "&nbsp;</nobr></td><td><nobr>&nbsp;" + tr("To Next") + "&nbsp;</nobr></td><td><nobr>&nbsp;" + tr("To End")  + "&nbsp;</nobr></td></tr>";
-
-                text += "<tr>";
-                text += "<td>" + tr("Distance: ") + "</td>";
-                IUnit::self().meter2distance(info.distance1, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
-                IUnit::self().meter2distance(info.distance2, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
-                IUnit::self().meter2distance(info.distance3, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
-                text += "</tr>";
-
-                text += "<tr>";
-                text += "<td>" + tr("Time: ") + "</td>";
-                IUnit::self().seconds2time(info.elapsedSeconds1, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "&nbsp;</td>";
-                IUnit::self().seconds2time(info.elapsedSeconds2, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "&nbsp;</td>";
-                IUnit::self().seconds2time(info.elapsedSeconds3, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "&nbsp;</td>";
-                text += "</tr>";
-
-                text += "<tr>";
-                text += "<td>" + tr("Ascent: ") + "</td>";
-                IUnit::self().meter2elevation(info.ascent1, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
-                IUnit::self().meter2elevation(info.ascent2, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
-                IUnit::self().meter2elevation(info.ascent3, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
-                text += "</tr>";
-
-                text += "<tr>";
-                text += "<td>" + tr("Descent: ") + "</td>";
-                IUnit::self().meter2elevation(info.descent1, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
-                IUnit::self().meter2elevation(info.descent2, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
-                IUnit::self().meter2elevation(info.descent3, val, unit);
-                text += "<td>"+ QString("%1%2").arg(val).arg(unit) + "</td>";
-                text += "</tr>";
-
-                text += "</table>";
-
-                table1->cellAt(0,0).firstCursorPosition().insertHtml(text);
-
-                const QList<CGisItemWpt::image_t>& images = wpt->getImages();
-                if(!images.isEmpty())
-                {
-                    QImage image(images.first().pixmap);
-                    qDebug() << image.size();
-
-                    int w = image.width();
-                    int h = image.height();
-
-                    if(w < h)
-                    {
-                        h *= 100.0 / w;
-                        w  = 100;
-                    }
-                    else
-                    {
-                        h *= 200.0 / w;
-                        w  = 200;
-                    }
-                    qDebug() << w << h;
-                    image = image.scaled(w,h,Qt::KeepAspectRatio, Qt::SmoothTransformation);
-
-
-                    table1->cellAt(0,1).firstCursorPosition().insertImage(image);
-                }
-
-                table->cellAt(cnt,eComment2).firstCursorPosition().insertHtml(IGisItem::createText(wpt->isReadOnly()||printable, wpt->getComment(), wpt->getDescription(), wpt->getLinks(), wpt->getKey().item));
+                table1->cellAt(0, 1).firstCursorPosition().insertImage(getImage(info));
             }
+
+            table->cellAt(cnt, eComment2).firstCursorPosition().insertHtml(IGisItem::createText(info.isReadOnly || printable, info.cmt, info.desc, info.links, info.key.item));
+
             cnt++;
         }
 
-        addIcon(table, eSym1, cnt, trk, printable);
-        table->cellAt(cnt,eInfo2).firstCursorPosition().insertHtml(trk->getInfo(IGisItem::eFeatureShowName|IGisItem::eFeatureShowActivity));
+        addIcon(table, eSym1, cnt, trk->getDisplayIcon(), trk->getKey().item, trk->isReadOnly(), printable);
+        table->cellAt(cnt, eInfo2).firstCursorPosition().insertHtml(trk->getInfo(IGisItem::eFeatureShowName|IGisItem::eFeatureShowActivity));
 
-        QTextTable * table1 = table->cellAt(cnt,eData2).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
+        QTextTable * table1 = table->cellAt(cnt, eData2).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
 
-        QImage profile(w1,h1,QImage::Format_ARGB32);
-        getTrackProfile(trk, profile);
-        table1->cellAt(0,0).firstCursorPosition().insertImage(profile);
+        QImage profile(w1, h1, QImage::Format_ARGB32);
+        getTrackProfile(trk, nullptr, profile);
+        table1->cellAt(0, 0).firstCursorPosition().insertImage(profile);
 
-        QImage overview(h1,h1,QImage::Format_ARGB32);
-        getTrackOverview(trk, overview);
-        table1->cellAt(0,1).firstCursorPosition().insertImage(overview);
+        QImage overview(h1, h1, QImage::Format_ARGB32);
+        getTrackOverview(trk, nullptr, overview);
+        table1->cellAt(0, 1).firstCursorPosition().insertImage(overview);
 
-        table->cellAt(cnt,eComment2).firstCursorPosition().insertHtml(IGisItem::createText(trk->isReadOnly()||printable, trk->getComment(), trk->getDescription(), trk->getLinks(), trk->getKey().item));
+        table->cellAt(cnt, eComment2).firstCursorPosition().insertHtml(IGisItem::createText(trk->isReadOnly() || printable, trk->getComment(), trk->getDescription(), trk->getLinks(), trk->getKey().item));
+
+        cursor.setPosition(table->lastPosition() + 1);
+    }
+}
+
+void CDetailsPrj::drawByDetails(QTextCursor& cursor,
+                                QList<CGisItemTrk *> &trks,
+                                QList<CGisItemWpt *> &wpts,
+                                CProgressDialog &progress,
+                                int &n, bool printable)
+{
+    int w = cursor.document()->textWidth();
+
+    const qreal w1 = qRound(w/3.5 > 300 ? 300 : w/3.5);
+    const qreal h1 = qRound(w1/2.0);
+
+    for(CGisItemTrk * trk : trks)
+    {
+        const QList<wpt_info_t>& wptInfo = getWptInfo(*trk);
+
+        cursor.insertHtml(QString("<h2>%1</h2>").arg(trk->getName()));
+        QTextTable * table = cursor.insertTable(wptInfo.count()+2, eMax2, fmtTableStandard);
+
+        table->cellAt(0, eSym2).setFormat(fmtCharHeader);
+        table->cellAt(0, eInfo2).setFormat(fmtCharHeader);
+        table->cellAt(0, eData2).setFormat(fmtCharHeader);
+        table->cellAt(0, eComment2).setFormat(fmtCharHeader);
+
+        table->cellAt(0, eInfo2).firstCursorPosition().insertText(tr("Info"));
+        table->cellAt(0, eData2).firstCursorPosition().insertText(tr("Statistics"));
+
+        int cnt = 1;
+        for(const wpt_info_t &info : wptInfo)
+        {
+            PROGRESS(n++, return );
+
+            // 1st column
+            addIcon(table, eSym2, cnt, info.icon, info.key.item, info.isReadOnly, printable);
+
+            // 2nd column
+            QTextTable * table1 = table->cellAt(cnt, eInfo2).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
+            table1->cellAt(0, 0).firstCursorPosition().insertHtml(getNameAndTime(info, *trk));
+            if(!info.images.isEmpty())
+            {
+                table1->cellAt(0, 1).firstCursorPosition().insertImage(getImage(info));
+            }
+
+            // 3rd column
+            table->cellAt(cnt, eData2).firstCursorPosition().insertHtml(getStatistics(info));
+
+            // 4th column
+            QTextTable * table2 = table->cellAt(cnt, eComment2).lastCursorPosition().insertTable(1, 2, fmtTableInfo);
+            QImage profile(w1, h1, QImage::Format_ARGB32);
+            getTrackProfile(trk, info.pTrkpt, profile);
+            table2->cellAt(0, 0).firstCursorPosition().insertImage(profile);
+            QImage overview(h1, h1, QImage::Format_ARGB32);
+            getTrackOverview(trk, info.pTrkpt, overview);
+            table2->cellAt(0, 1).firstCursorPosition().insertImage(overview);
+
+            // next row
+            cnt++;
+        }
+
+        addIcon(table, eSym1, cnt, trk->getDisplayIcon(), trk->getKey().item, trk->isReadOnly(), printable);
+        table->cellAt(cnt, eInfo2).firstCursorPosition().insertHtml(trk->getInfo(IGisItem::eFeatureShowName|IGisItem::eFeatureShowActivity));
+
+        table->mergeCells(cnt, eData2, 1, 2);
+        table->cellAt(cnt, eData2).firstCursorPosition().insertHtml(IGisItem::createText(trk->isReadOnly() || printable, trk->getComment(), trk->getDescription(), trk->getLinks(), trk->getKey().item));
 
         cursor.setPosition(table->lastPosition() + 1);
     }
@@ -710,21 +937,21 @@ void CDetailsPrj::drawArea(QTextCursor& cursor, QList<CGisItemOvlArea *> &areas,
     cursor.insertHtml(tr("<h2>Areas</h2>"));
     QTextTable * table = cursor.insertTable(areas.count()+1, eMax1, fmtTableStandard);
 
-    table->cellAt(0,eSym1).setFormat(fmtCharHeader);
-    table->cellAt(0,eInfo1).setFormat(fmtCharHeader);
-    table->cellAt(0,eComment1).setFormat(fmtCharHeader);
+    table->cellAt(0, eSym1).setFormat(fmtCharHeader);
+    table->cellAt(0, eInfo1).setFormat(fmtCharHeader);
+    table->cellAt(0, eComment1).setFormat(fmtCharHeader);
 
-    table->cellAt(0,eInfo1).firstCursorPosition().insertText(tr("Info"));
-    table->cellAt(0,eComment1).firstCursorPosition().insertText(tr("Comment"));
+    table->cellAt(0, eInfo1).firstCursorPosition().insertText(tr("Info"));
+    table->cellAt(0, eComment1).firstCursorPosition().insertText(tr("Comment"));
 
     int cnt = 1;
     for(CGisItemOvlArea * area : areas)
     {
         PROGRESS(n++, return );
 
-        addIcon(table, eSym1, cnt, area, printable);
-        table->cellAt(cnt,eInfo1).firstCursorPosition().insertHtml(area->getInfo(IGisItem::eFeatureShowName));
-        table->cellAt(cnt,eComment1).firstCursorPosition().insertHtml(IGisItem::createText(area->isReadOnly()||printable, area->getComment(), area->getDescription(), area->getLinks(), area->getKey().item));
+        addIcon(table, eSym1, cnt, area->getDisplayIcon(), area->getKey().item, area->isReadOnly(), printable);
+        table->cellAt(cnt, eInfo1).firstCursorPosition().insertHtml(area->getInfo(IGisItem::eFeatureShowName));
+        table->cellAt(cnt, eComment1).firstCursorPosition().insertHtml(IGisItem::createText(area->isReadOnly() || printable, area->getComment(), area->getDescription(), area->getLinks(), area->getKey().item));
         cnt++;
     }
 
@@ -740,21 +967,21 @@ void CDetailsPrj::drawRoute(QTextCursor& cursor, QList<CGisItemRte *> &rtes, CPr
     cursor.insertHtml(tr("<h2>Routes</h2>"));
     QTextTable * table = cursor.insertTable(rtes.count()+1, eMax1, fmtTableStandard);
 
-    table->cellAt(0,eSym1).setFormat(fmtCharHeader);
-    table->cellAt(0,eInfo1).setFormat(fmtCharHeader);
-    table->cellAt(0,eComment1).setFormat(fmtCharHeader);
+    table->cellAt(0, eSym1).setFormat(fmtCharHeader);
+    table->cellAt(0, eInfo1).setFormat(fmtCharHeader);
+    table->cellAt(0, eComment1).setFormat(fmtCharHeader);
 
-    table->cellAt(0,eInfo1).firstCursorPosition().insertText(tr("Info"));
-    table->cellAt(0,eComment1).firstCursorPosition().insertText(tr("Comment"));
+    table->cellAt(0, eInfo1).firstCursorPosition().insertText(tr("Info"));
+    table->cellAt(0, eComment1).firstCursorPosition().insertText(tr("Comment"));
 
     int cnt = 1;
     for(CGisItemRte * rte : rtes)
     {
         PROGRESS(n++, return );
 
-        addIcon(table, eSym1, cnt, rte, printable);
-        table->cellAt(cnt,eInfo1).firstCursorPosition().insertHtml(rte->getInfo(IGisItem::eFeatureShowName));
-        table->cellAt(cnt,eComment1).firstCursorPosition().insertHtml(IGisItem::createText(rte->isReadOnly()||printable, rte->getComment(), rte->getDescription(), rte->getLinks(), rte->getKey().item));
+        addIcon(table, eSym1, cnt, rte->getDisplayIcon(), rte->getKey().item, rte->isReadOnly(), printable);
+        table->cellAt(cnt, eInfo1).firstCursorPosition().insertHtml(rte->getInfo(IGisItem::eFeatureShowName));
+        table->cellAt(cnt, eComment1).firstCursorPosition().insertHtml(IGisItem::createText(rte->isReadOnly() || printable, rte->getComment(), rte->getDescription(), rte->getLinks(), rte->getKey().item));
         cnt++;
     }
 
