@@ -22,9 +22,9 @@
 #include "gis/CGisDraw.h"
 #include "gis/CGisListWks.h"
 #include "gis/CGisWorkspace.h"
-#include "gis/IGisItem.h"
 #include "gis/fit/CFitProject.h"
 #include "gis/gpx/CGpxProject.h"
+#include "gis/IGisItem.h"
 #include "gis/ovl/CGisItemOvlArea.h"
 #include "gis/prj/CDetailsPrj.h"
 #include "gis/prj/IGisProject.h"
@@ -59,6 +59,8 @@ const QString IGisProject::filedialogFilterFIT    = "Garmin FIT Format (*.fit)";
 const QString IGisProject::filedialogSaveFilters = filedialogFilterGPX + ";; " + filedialogFilterQLB + ";; " + filedialogFilterQMS + ";; " + filedialogFilterTCX;
 const QString IGisProject::filedialogLoadFilters = filedialogAllSupported + ";; " + filedialogFilterGPX + ";; " + filedialogFilterTCX + ";; " + filedialogFilterSML + ";; " + filedialogFilterLOG + ";; " + filedialogFilterQLB + ";; " + filedialogFilterQMS + ";; " + filedialogFilterSLF + ";; " + filedialogFilterFIT;
 
+QString IGisProject::keyUserFocus;
+
 IGisProject::filter_mode_e IGisProject::filterMode = IGisProject::eFilterModeName;
 
 IGisProject::IGisProject(type_e type, const QString &filename, CGisListWks *parent)
@@ -106,6 +108,10 @@ IGisProject::IGisProject(type_e type, const QString &filename, IDevice *parent)
 IGisProject::~IGisProject()
 {
     delete dlgDetails;
+    if(key == keyUserFocus)
+    {
+        keyUserFocus.clear();
+    }
 }
 
 IGisProject * IGisProject::create(const QString filename, CGisListWks * parent)
@@ -230,7 +236,7 @@ QString IGisProject::getDeviceKey() const
 
 QPixmap IGisProject::getIcon() const
 {
-    return icon(CGisListWks::eColumnIcon).pixmap(22,22);
+    return icon(CGisListWks::eColumnIcon).pixmap(22, 22);
 }
 
 qint32 IGisProject::isOnDevice() const
@@ -310,7 +316,7 @@ void IGisProject::setChanged()
 {
     if(autoSave)
     {
-        setText(CGisListWks::eColumnDecoration,"A");
+        setText(CGisListWks::eColumnDecoration, "A");
 
         if(!autoSavePending)
         {
@@ -320,7 +326,7 @@ void IGisProject::setChanged()
     }
     else
     {
-        setText(CGisListWks::eColumnDecoration,"*");
+        setText(CGisListWks::eColumnDecoration, "*");
     }
     updateItems();
 }
@@ -512,7 +518,7 @@ void IGisProject::setupName(const QString &defaultName)
 
 void IGisProject::markAsSaved()
 {
-    setText(CGisListWks::eColumnDecoration,autoSave ? "A" : "");
+    setText(CGisListWks::eColumnDecoration, autoSave ? "A" : "");
     for(int i = 0; i < childCount(); i++)
     {
         IGisItem * item = dynamic_cast<IGisItem*>(child(i));
@@ -1215,3 +1221,17 @@ bool IGisProject::findPolylineCloseBy(const QPointF& pt1, const QPointF& pt2, qi
     }
     return !polyline.isEmpty();
 }
+
+void IGisProject::gainUserFocus(bool yes)
+{
+    if(yes)
+    {
+        setIcon(CGisListWks::eColumnName, QIcon("://icons/32x32/Focus.png"));
+        keyUserFocus = key;
+    }
+    else
+    {
+        setIcon(CGisListWks::eColumnName, QIcon());
+        keyUserFocus.clear();
+    }
+}
