@@ -29,7 +29,7 @@
 #include <QtWidgets>
 
 #define VER_TRK         quint8(5)
-#define VER_WPT         quint8(2)
+#define VER_WPT         quint8(3)
 #define VER_RTE         quint8(3)
 #define VER_AREA        quint8(1)
 #define VER_LINK        quint8(1)
@@ -38,7 +38,7 @@
 #define VER_RTEPT       quint8(2)
 #define VER_RTESUBPT    quint8(2)
 #define VER_WPT_T       quint8(1)
-#define VER_GC_T        quint8(2)
+#define VER_GC_T        quint8(3)
 #define VER_GCLOG_T     quint8(1)
 #define VER_IMAGE       quint8(1)
 #define VER_PROJECT     quint8(5)
@@ -250,6 +250,7 @@ QDataStream& operator<<(QDataStream& stream, const CGisItemWpt::geocache_t& geoc
         stream << geocache.ownerId;
         stream << geocache.type;
         stream << geocache.container;
+        stream << geocache.attributes;
         stream << quint8(geocache.shortDescIsHtml);
         stream << geocache.shortDesc;
         stream << quint8(geocache.longDescIsHtml);
@@ -277,7 +278,7 @@ QDataStream& operator>>(QDataStream& stream, CGisItemWpt::geocache_t& geocache)
         geocache.available = tmp8;
         stream >> tmp8;
         geocache.archived = tmp8;
-        if(version>1)
+        if(version > 1)
         {
             stream >> tmp8;
             geocache.needsMaintenance = tmp8;
@@ -287,13 +288,17 @@ QDataStream& operator>>(QDataStream& stream, CGisItemWpt::geocache_t& geocache)
         if(version == 1)
         {
             QString trash;
-            stream >> trash; //status string
+            stream >> trash; //status string was only used in version 1
         }
         stream >> geocache.name;
         stream >> geocache.owner;
         stream >> geocache.ownerId;
         stream >> geocache.type;
         stream >> geocache.container;
+        if(version > 2)
+        {
+            stream>>geocache.attributes;
+        }
         stream >> tmp8;
         geocache.shortDescIsHtml = tmp8;
         stream >> geocache.shortDesc;
@@ -312,7 +317,7 @@ QDataStream& operator>>(QDataStream& stream, CGisItemWpt::geocache_t& geocache)
 QDataStream& operator<<(QDataStream& stream, const CGisItemWpt::image_t& image)
 {
     QBuffer imgBuf;
-    image.pixmap.save(&imgBuf,"JPEG");
+    image.pixmap.save(&imgBuf, "JPEG");
 
     stream << VER_IMAGE;
     stream << imgBuf.data();
@@ -336,7 +341,7 @@ QDataStream& operator>>(QDataStream& stream, CGisItemWpt::image_t& image)
     stream >> image.filePath;
     stream >> image.fileName;
 
-    image.pixmap.load(&imgBuf,"JPEG");
+    image.pixmap.load(&imgBuf, "JPEG");
 
     return stream;
 }
@@ -547,7 +552,7 @@ QDataStream& CGisItemTrk::operator>>(QDataStream& stream) const
 
     stream.writeRawData(MAGIC_TRK, MAGIC_SIZE);
     stream << VER_TRK;
-    stream << qCompress(buffer,9);
+    stream << qCompress(buffer, 9);
     return stream;
 }
 
@@ -559,9 +564,9 @@ QDataStream& CGisItemTrk::operator<<(QDataStream& stream)
     qint64 pos = dev->pos();
 
     char magic[10];
-    stream.readRawData(magic,MAGIC_SIZE);
+    stream.readRawData(magic, MAGIC_SIZE);
 
-    if(strncmp(magic,MAGIC_TRK,MAGIC_SIZE))
+    if(strncmp(magic, MAGIC_TRK, MAGIC_SIZE))
     {
         dev->seek(pos);
         return stream;
@@ -647,9 +652,9 @@ QDataStream& CGisItemWpt::operator<<(QDataStream& stream)
     qint64 pos = dev->pos();
 
     char magic[10];
-    stream.readRawData(magic,MAGIC_SIZE);
+    stream.readRawData(magic, MAGIC_SIZE);
 
-    if(strncmp(magic,MAGIC_WPT,MAGIC_SIZE))
+    if(strncmp(magic, MAGIC_WPT, MAGIC_SIZE))
     {
         dev->seek(pos);
         return stream;
@@ -675,6 +680,12 @@ QDataStream& CGisItemWpt::operator<<(QDataStream& stream)
         in >> widthBubble;
     }
 
+    if(version <= 2 && geocache.hasData)
+    {
+        //If the geocache was saved with an old Version of QMS recalculate it's key to make sure geocaches with the same id are treated as being the same
+        key.item="";
+        genKey();
+    }
     setIcon();
     setText   (CGisListWks::eColumnName, getName());
     setToolTip(CGisListWks::eColumnName, getInfo(IGisItem::eFeatureShowName));
@@ -703,7 +714,7 @@ QDataStream& CGisItemWpt::operator>>(QDataStream& stream) const
 
     stream.writeRawData(MAGIC_WPT, MAGIC_SIZE);
     stream << VER_WPT;
-    stream << qCompress(buffer,9);
+    stream << qCompress(buffer, 9);
 
     return stream;
 }
@@ -716,9 +727,9 @@ QDataStream& CGisItemRte::operator<<(QDataStream& stream)
     qint64 pos = dev->pos();
 
     char magic[10];
-    stream.readRawData(magic,MAGIC_SIZE);
+    stream.readRawData(magic, MAGIC_SIZE);
 
-    if(strncmp(magic,MAGIC_RTE,MAGIC_SIZE))
+    if(strncmp(magic, MAGIC_RTE, MAGIC_SIZE))
     {
         dev->seek(pos);
         return stream;
@@ -790,7 +801,7 @@ QDataStream& CGisItemRte::operator>>(QDataStream& stream) const
 
     stream.writeRawData(MAGIC_RTE, MAGIC_SIZE);
     stream << VER_RTE;
-    stream << qCompress(buffer,9);
+    stream << qCompress(buffer, 9);
 
     return stream;
 }
@@ -803,9 +814,9 @@ QDataStream& CGisItemOvlArea::operator<<(QDataStream& stream)
     qint64 pos = dev->pos();
 
     char magic[10];
-    stream.readRawData(magic,MAGIC_SIZE);
+    stream.readRawData(magic, MAGIC_SIZE);
 
-    if(strncmp(magic,MAGIC_AREA,MAGIC_SIZE))
+    if(strncmp(magic, MAGIC_AREA, MAGIC_SIZE))
     {
         dev->seek(pos);
         return stream;
@@ -868,7 +879,7 @@ QDataStream& CGisItemOvlArea::operator>>(QDataStream& stream) const
 
     stream.writeRawData(MAGIC_AREA, MAGIC_SIZE);
     stream << VER_AREA;
-    stream << qCompress(buffer,9);
+    stream << qCompress(buffer, 9);
 
     return stream;
 }
@@ -880,9 +891,9 @@ QDataStream& IGisProject::operator<<(QDataStream& stream)
     qint64 pos = dev->pos();
 
     char magic[10];
-    stream.readRawData(magic,MAGIC_SIZE);
+    stream.readRawData(magic, MAGIC_SIZE);
 
-    if(strncmp(magic,MAGIC_PROJ,MAGIC_SIZE))
+    if(strncmp(magic, MAGIC_PROJ, MAGIC_SIZE))
     {
         dev->seek(pos);
         return stream;
@@ -1018,7 +1029,7 @@ QDataStream& IGisProject::operator>>(QDataStream& stream) const
         stream << VER_ITEM;
         stream << quint8(item->type());
         stream << item->getHistory();
-        stream << quint8(item->data(1,Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
+        stream << quint8(item->data(1, Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
         stream << item->getLastDatabaseHash();
     }
     for(int i = 0; i < childCount(); i++)
@@ -1031,7 +1042,7 @@ QDataStream& IGisProject::operator>>(QDataStream& stream) const
         stream << VER_ITEM;
         stream << quint8(item->type());
         stream << item->getHistory();
-        stream << quint8(item->data(1,Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
+        stream << quint8(item->data(1, Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
         stream << item->getLastDatabaseHash();
     }
     for(int i = 0; i < childCount(); i++)
@@ -1044,7 +1055,7 @@ QDataStream& IGisProject::operator>>(QDataStream& stream) const
         stream << VER_ITEM;
         stream << quint8(item->type());
         stream << item->getHistory();
-        stream << quint8(item->data(1,Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
+        stream << quint8(item->data(1, Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
         stream << item->getLastDatabaseHash();
     }
     for(int i = 0; i < childCount(); i++)
@@ -1057,7 +1068,7 @@ QDataStream& IGisProject::operator>>(QDataStream& stream) const
         stream << VER_ITEM;
         stream << quint8(item->type());
         stream << item->getHistory();
-        stream << quint8(item->data(1,Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
+        stream << quint8(item->data(1, Qt::UserRole).toUInt() & IGisItem::eMarkChanged);
         stream << item->getLastDatabaseHash();
     }
 
@@ -1071,9 +1082,9 @@ QDataStream& CDBProject::operator<<(QDataStream& stream)
     qint64 pos = dev->pos();
 
     char magic[10];
-    stream.readRawData(magic,MAGIC_SIZE);
+    stream.readRawData(magic, MAGIC_SIZE);
 
-    if(strncmp(magic,MAGIC_PROJ,MAGIC_SIZE))
+    if(strncmp(magic, MAGIC_PROJ, MAGIC_SIZE))
     {
         dev->seek(pos);
         return stream;
