@@ -18,24 +18,24 @@
 **********************************************************************************************/
 
 
-#include "CMainWindow.h"
-#include "GeoMath.h"
 #include "canvas/CCanvas.h"
+#include "CMainWindow.h"
 #include "gis/CGisDraw.h"
 #include "gis/CGisListWks.h"
-#include "gis/WptIcons.h"
 #include "gis/prj/IGisProject.h"
 #include "gis/rte/CDetailsRte.h"
 #include "gis/rte/CGisItemRte.h"
 #include "gis/rte/CScrOptRte.h"
 #include "gis/trk/CGisItemTrk.h"
+#include "GeoMath.h"
 #include "helpers/CDraw.h"
 #include "helpers/CDraw.h"
+#include "helpers/CWptIconManager.h"
 #include "units/IUnit.h"
 
+#include <proj_api.h>
 #include <QtWidgets>
 #include <QtXml>
-#include <proj_api.h>
 
 const QPen CGisItemRte::penBackground(Qt::white, 5, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
 IGisItem::key_t CGisItemRte::keyUserFocus;
@@ -51,7 +51,7 @@ void CGisItemRte::rtept_t::updateIcon()
     }
     else
     {
-        icon = getWptIconByName(sym, focus);
+        icon = CWptIconManager::self().getWptIconByName(sym, focus);
     }
 }
 /// used to create a copy of route with new parent
@@ -269,7 +269,7 @@ void CGisItemRte::deriveSecondaryData()
         }
     }
 
-    boundingRect = QRectF(QPointF(west * DEG_TO_RAD, north * DEG_TO_RAD), QPointF(east * DEG_TO_RAD,south * DEG_TO_RAD));
+    boundingRect = QRectF(QPointF(west * DEG_TO_RAD, north * DEG_TO_RAD), QPointF(east * DEG_TO_RAD, south * DEG_TO_RAD));
 }
 
 void CGisItemRte::edit()
@@ -345,7 +345,7 @@ void CGisItemRte::toTrack()
 
 void CGisItemRte::setSymbol()
 {
-    IGisItem::setIcon(QPixmap("://icons/48x48/Route.png").scaled(22,22, Qt::KeepAspectRatio, Qt::SmoothTransformation));
+    IGisItem::setIcon(QPixmap("://icons/48x48/Route.png").scaled(22, 22, Qt::KeepAspectRatio, Qt::SmoothTransformation));
 }
 
 void CGisItemRte::setName(const QString& str)
@@ -458,6 +458,18 @@ QString CGisItemRte::getInfo(quint32 feature) const
         }
     }
 
+    if((feature & eFeatureShowLinks) && !rte.links.isEmpty())
+    {
+        for(const link_t& link : rte.links)
+        {
+            if(link.type.isEmpty() || (link.type == "text/html"))
+            {
+                str += "<br/>\n";
+                str += QString("<a href='%1'>%2</a>").arg(link.uri.toString()).arg(link.text);
+            }
+        }
+    }
+
     return str + "</div>";
 }
 
@@ -535,7 +547,7 @@ void CGisItemRte::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
     QPointF p2 = viewport[2];
     gis->convertRad2Px(p1);
     gis->convertRad2Px(p2);
-    QRectF extViewport(p1,p2);
+    QRectF extViewport(p1, p2);
 
     QVector<qint32>  points;
     QVector<QPixmap> icons;
@@ -579,11 +591,11 @@ void CGisItemRte::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
         switch(points[i])
         {
         case 1:
-            p.drawEllipse(line[i],7,7);
+            p.drawEllipse(line[i], 7, 7);
             break;
 
         case 2:
-            p.drawEllipse(line[i],5,5);
+            p.drawEllipse(line[i], 5, 5);
             break;
         }
     }
@@ -592,7 +604,7 @@ void CGisItemRte::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
     p.setBrush(hasUserFocus() ? penForegroundFocus.color() : penForeground.color());
     if (!isNogo())
     {
-        CDraw::arrows(line, extViewport, p, 10, 80, 1.0);
+        CDraw::arrows(line, extViewport, p, 10, 120, 1.0);
     }
     p.drawPolyline(line);
 
@@ -608,7 +620,7 @@ void CGisItemRte::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
         {
         case 1:
             p.setBrush(Qt::red);
-            p.drawEllipse(line[i],5,5);
+            p.drawEllipse(line[i], 5, 5);
             if(focus[n] != NOPOINTF)
             {
                 p.drawPixmap(line[i] - focus[n], icons[n]);
@@ -618,7 +630,7 @@ void CGisItemRte::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
 
         case 2:
             p.setBrush(Qt::cyan);
-            p.drawEllipse(line[i],3,3);
+            p.drawEllipse(line[i], 3, 3);
             break;
         }
     }
@@ -653,18 +665,19 @@ void CGisItemRte::drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis)
         str += "\n" + mouseMoveFocus->instruction;
 
         // calculate bounding box of text
-        QFont f = CMainWindow::self().getMapFont();
+        const CMainWindow& w = CMainWindow::self();
+        QFont f = w.getMapFont();
         QFontMetrics fm(f);
-        QRect rectText = fm.boundingRect(QRect(0,0,500,0), Qt::AlignLeft|Qt::AlignTop|Qt::TextWordWrap, str);
+        QRect rectText = fm.boundingRect(QRect(0, 0, 500, 0), Qt::AlignLeft|Qt::AlignTop|Qt::TextWordWrap, str);
         rectText.adjust(-5, -5, 5, 5);
-        rectText.moveBottomLeft(anchor.toPoint() + QPoint(-50,-50));
+        rectText.moveBottomLeft(anchor.toPoint() + QPoint(-50, -50));
 
         p.setFont(f);
-        CDraw::bubble(p, rectText, anchor.toPoint(), 18 /* px */, 21 /* px */);
+        CDraw::bubble(p, rectText, anchor.toPoint(), w.palette().color(QPalette::Background), 18 /* px */, 21 /* px */);
 
         p.save();
-        p.translate(5,5);
-        p.setPen(Qt::darkBlue);
+        p.translate(5, 5);
+        p.setPen(w.palette().color(QPalette::Foreground));
         p.drawText(rectText, str);
         p.restore();
     }
@@ -688,7 +701,7 @@ void CGisItemRte::drawLabel(QPainter& p, const QPolygonF& viewport, QList<QRectF
         //p.drawPixmap(pt, rtept.icon);
 
         QRectF rect = fm.boundingRect(rtept.name);
-        rect.adjust(-2,-2,2,2);
+        rect.adjust(-2, -2, 2, 2);
 
         // place label on top
         rect.moveCenter(pt + QPointF(rtept.icon.width()/2, -fm.height()));
@@ -713,6 +726,7 @@ void CGisItemRte::drawLabel(QPainter& p, const QPolygonF& viewport, QList<QRectF
             }
         }
 
+
         CDraw::text(rtept.name, p, rect.toRect(), Qt::darkBlue);
         blockedAreas << rect;
     }
@@ -727,7 +741,7 @@ void CGisItemRte::drawHighlight(QPainter& p)
         return;
     }
 
-    p.setPen(QPen(QColor(255,0,0,100),11,Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
+    p.setPen(QPen(QColor(255, 0, 0, 100), 11, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
     p.drawPolyline(line);
 }
 
@@ -761,10 +775,14 @@ void CGisItemRte::setDataFromPolyline(const SGisLine &l)
     QMutexLocker lock(&mutexItems);
     mouseMoveFocus = nullptr;
 
-    readRouteDataFromGisLine(l);
-
+    // [Issue #436] Add histrory entry befor the new GIS line is stored
+    // because this will update the current history entry several times.
+    // Therefore the new entry must be in place else the updated route
+    // is stored in the previous entry as well.
     flags |= eFlagTainted;
     changed(tr("Changed route points."), "://icons/48x48/LineMove.png");
+
+    readRouteDataFromGisLine(l);
 }
 
 void CGisItemRte::getPolylineFromData(SGisLine& l) const
@@ -1164,47 +1182,47 @@ void CGisItemRte::setResultFromBRouter(const QDomDocument &xml, const QString &o
             subpt.type              = subpt_t::eTypeJunct;
             subpt.instruction       = xmlManeuver.firstChildElement("desc").text();
             const QString &command = xmlManeuver.firstChildElement("extensions").firstChildElement("turn").text(); // command
-            if(command=="TU")        // u-turn
+            if(command == "TU")        // u-turn
             {
                 subpt.bearing = 180;
             }
-            else if(command=="TSHL") // turn sharp left
+            else if(command == "TSHL") // turn sharp left
             {
                 subpt.bearing = -135;
             }
-            else if(command=="TL")   // turn left
+            else if(command == "TL")   // turn left
             {
                 subpt.bearing = -90;
             }
-            else if(command=="TSLL") // turn slight left
+            else if(command == "TSLL") // turn slight left
             {
                 subpt.bearing = -45;
             }
-            else if(command=="KL")   // keep left
+            else if(command == "KL")   // keep left
             {
                 subpt.bearing = 0;
             }
-            else if(command=="C")    // straight
+            else if(command == "C")    // straight
             {
                 subpt.bearing = 0;
             }
-            else if(command=="KR")   // keep right
+            else if(command == "KR")   // keep right
             {
                 subpt.bearing = 0;
             }
-            else if(command=="TSLR") // turn slight right
+            else if(command == "TSLR") // turn slight right
             {
                 subpt.bearing = 45;
             }
-            else if(command=="TR")   // turn right
+            else if(command == "TR")   // turn right
             {
                 subpt.bearing = 90;
             }
-            else if(command=="TSHR") // turn sharp right
+            else if(command == "TSHR") // turn sharp right
             {
                 subpt.bearing = 135;
             }
-            else if(command=="TRU")  // u-turn
+            else if(command == "TRU")  // u-turn
             {
                 subpt.bearing = 180;
             }
@@ -1241,7 +1259,7 @@ void CGisItemRte::setResultFromBRouter(const QDomDocument &xml, const QString &o
             }
         }
         routePoint.ele = shape[startIdx].ele;
-        routePoint.subpts = shape.mid(startIdx,minDistIdx-startIdx);
+        routePoint.subpts = shape.mid(startIdx, minDistIdx-startIdx);
         routePoint.fakeSubpt.lon = routePoint.lon;
         routePoint.fakeSubpt.lat = routePoint.lat;
         routePoint.fakeSubpt.ele = routePoint.ele;
