@@ -20,19 +20,18 @@
 
 #include "helpers/CSettings.h"
 #include "setup/IAppSetup.h"
+#include <QJSEngine>
 #include <QMessageBox>
-#include <QWebFrame>
-#include <QWebPage>
-#include <QtScript>
-#include <QtWebKit>
+#include <QNetworkReply>
+#include <QWebEnginePage>
 
 CRouterBRouterSetup::CRouterBRouterSetup(QObject *parent)
     : QObject(parent)
 {
     networkAccessManager = new QNetworkAccessManager(this);
-    profilesWebPage = new QWebPage(this);
+    profilesWebPage = new QWebEnginePage(this);
     connect(networkAccessManager, &QNetworkAccessManager::finished, this, &CRouterBRouterSetup::slotOnlineRequestFinished);
-    connect(profilesWebPage, &QWebPage::loadFinished, this, &CRouterBRouterSetup::slotLoadOnlineProfilesRequestFinished);
+    connect(profilesWebPage, &QWebEnginePage::loadFinished, this, &CRouterBRouterSetup::slotLoadOnlineProfilesRequestFinished);
 }
 
 CRouterBRouterSetup::~CRouterBRouterSetup()
@@ -43,13 +42,13 @@ void CRouterBRouterSetup::load()
 {
     SETTINGS;
     cfg.beginGroup("Route/brouter");
-    installMode = modeFromString(cfg.value("installMode",stringFromMode(defaultInstallMode)).toString());
-    expertMode = cfg.value("expertMode",defaultExpertMode).toBool();
+    installMode = modeFromString(cfg.value("installMode", stringFromMode(defaultInstallMode)).toString());
+    expertMode = cfg.value("expertMode", defaultExpertMode).toBool();
     onlineWebUrl = cfg.value("onlineWebUrl", defaultOnlineWebUrl).toString();
     onlineServiceUrl = cfg.value("onlineServiceUrl", defaultOnlineServiceUrl).toString();
     onlineProfilesUrl = cfg.value("onlineProfilesUrl", defaultOnlineProfilesUrl).toString();
     localDir = cfg.value("localDir", defaultLocalDir).toString();
-    localJavaExecutable = cfg.value("localJava",findJava()).toString();
+    localJavaExecutable = cfg.value("localJava", findJava()).toString();
     localProfileDir = cfg.value("localProfileDir", defaultLocalProfileDir).toString();
     localCustomProfileDir = cfg.value("localCustomProfileDir", defaultLocalCustomProfileDir).toString();
     localSegmentsDir = cfg.value("localSegmentsDir", defaultLocalSegmentsDir).toString();
@@ -59,8 +58,8 @@ void CRouterBRouterSetup::load()
     localNumberThreads = cfg.value("localNumberThreads", defaultLocalNumberThreads).toString();
     localMaxRunningTime = cfg.value("localMaxRunningTime", defaultLocalMaxRunningTime).toString();
     localJavaOpts = cfg.value("localJavaOpts", defaultLocalJavaOpts).toString();
-    binariesUrl = cfg.value("binariesUrl",defaultBinariesUrl).toString();
-    segmentsUrl = cfg.value("segmentsUrl",defaultSegmentsUrl).toString();
+    binariesUrl = cfg.value("binariesUrl", defaultBinariesUrl).toString();
+    segmentsUrl = cfg.value("segmentsUrl", defaultSegmentsUrl).toString();
     onlineProfiles.clear();
     int size = cfg.beginReadArray("online");
     for (int i=0; i < size; i++)
@@ -97,7 +96,7 @@ void CRouterBRouterSetup::save()
 {
     SETTINGS;
     cfg.beginGroup("Route/brouter");
-    cfg.setValue("expertMode",expertMode);
+    cfg.setValue("expertMode", expertMode);
     cfg.setValue("installMode", stringFromMode(installMode));
     cfg.setValue("onlineWebUrl", onlineWebUrl);
     cfg.setValue("onlineServiceUrl", onlineServiceUrl);
@@ -126,7 +125,7 @@ void CRouterBRouterSetup::save()
     for (int i=0; i < localProfiles.size(); i++)
     {
         cfg.setArrayIndex(i);
-        cfg.setValue("profile",localProfiles.at(i));
+        cfg.setValue("profile", localProfiles.at(i));
     }
     cfg.endArray();
     cfg.endGroup();
@@ -229,7 +228,7 @@ void CRouterBRouterSetup::profileUp(const QString &profile)
         if (index > 0)
         {
             localProfiles.removeAt(index);
-            localProfiles.insert(index-1,profile);
+            localProfiles.insert(index-1, profile);
             emit sigProfilesChanged();
         }
     }
@@ -240,7 +239,7 @@ void CRouterBRouterSetup::profileUp(const QString &profile)
         if (index > 0)
         {
             onlineProfiles.removeAt(index);
-            onlineProfiles.insert(index-1,profile);
+            onlineProfiles.insert(index-1, profile);
             emit sigProfilesChanged();
         }
     }
@@ -254,7 +253,7 @@ void CRouterBRouterSetup::profileDown(const QString &profile)
         if (index > -1 && index < localProfiles.size()-1)
         {
             localProfiles.removeAt(index);
-            localProfiles.insert(index+1,profile);
+            localProfiles.insert(index+1, profile);
             emit sigProfilesChanged();
         }
     }
@@ -265,7 +264,7 @@ void CRouterBRouterSetup::profileDown(const QString &profile)
         if (index > -1 && index < onlineProfiles.size()-1)
         {
             onlineProfiles.removeAt(index);
-            onlineProfiles.insert(index+1,profile);
+            onlineProfiles.insert(index+1, profile);
             emit sigProfilesChanged();
         }
     }
@@ -342,7 +341,7 @@ QStringList CRouterBRouterSetup::getProfiles() const
 
 void CRouterBRouterSetup::loadLocalOnlineProfiles() const
 {
-    profilesWebPage->mainFrame()->load(QUrl(onlineProfilesUrl));
+    profilesWebPage->load(QUrl(onlineProfilesUrl));
 }
 
 void CRouterBRouterSetup::loadOnlineConfig() const
@@ -351,8 +350,8 @@ void CRouterBRouterSetup::loadOnlineConfig() const
     const QString &configHost = configUrl.host();
 
     QNetworkReply * reply = networkAccessManager->get(QNetworkRequest(configUrl));
-    reply->setProperty("configHost",configHost);
-    reply->setProperty("type",eTypeConfig);
+    reply->setProperty("configHost", configHost);
+    reply->setProperty("type", eTypeConfig);
 }
 
 void CRouterBRouterSetup::slotOnlineRequestFinished(QNetworkReply *reply)
@@ -384,7 +383,7 @@ void CRouterBRouterSetup::loadOnlineConfigFinished(QNetworkReply *reply)
 
         const QString jsConfig(reply->readAll());
 
-        QScriptEngine engine;
+        QJSEngine engine;
 
         const QString &jsSetup = QString( \
             "(function(){\
@@ -397,29 +396,29 @@ void CRouterBRouterSetup::loadOnlineConfigFinished(QNetworkReply *reply)
                        BR = {};\
                       })();").arg(configHost);
 
-        engine.evaluate(jsSetup).toString();
-        engine.evaluate(jsConfig).toString();
-        if (engine.hasUncaughtException())
+        engine.evaluate(jsSetup);
+        const QJSValue &val = engine.evaluate(jsConfig);
+        if (val.isError())
         {
-            emitOnlineConfigScriptError(engine.uncaughtException());
+            emitOnlineConfigScriptError(val);
             return;
         }
 
-        const QScriptValue &br = engine.globalObject().property("BR");
-        if (!br.isValid() || br.isError())
+        const QJSValue &br = engine.globalObject().property("BR");
+        if (!br.isObject() || br.isError())
         {
             emitOnlineConfigScriptError(br);
             return;
         }
-        const QScriptValue &conf = br.property("conf");
-        if (!conf.isValid() || conf.isError())
+        const QJSValue &conf = br.property("conf");
+        if (!conf.isObject() || conf.isError())
         {
             emitOnlineConfigScriptError(conf);
             return;
         }
 
-        const QScriptValue &host = conf.property("host").toString();
-        if (!host.isValid() || host.isError())
+        const QJSValue &host = conf.property("host");
+        if (!host.isString() || host.isError())
         {
             emitOnlineConfigScriptError(host);
             return;
@@ -428,8 +427,8 @@ void CRouterBRouterSetup::loadOnlineConfigFinished(QNetworkReply *reply)
         {
             onlineServiceUrl = host.toString();
         }
-        const QScriptValue &url = conf.property("profilesUrl").toString();
-        if (!url.isValid() || url.isError())
+        const QJSValue &url = conf.property("profilesUrl");
+        if (!url.isString() || url.isError())
         {
             emitOnlineConfigScriptError(url);
             return;
@@ -439,19 +438,19 @@ void CRouterBRouterSetup::loadOnlineConfigFinished(QNetworkReply *reply)
             onlineProfilesUrl = url.toString();
         }
 
-        const QScriptValue &profiles = conf.property("profiles");
-        if (!profiles.isValid() || profiles.isError())
+        const QJSValue &profiles = conf.property("profiles");
+        if (!profiles.isArray() || profiles.isError())
         {
             emitOnlineConfigScriptError(profiles);
             return;
         }
-        const qint32 len = profiles.property("length").toInt32();
+        const qint32 len = profiles.property("length").toInt();
 
         QStringList onlineProfilesLoaded;
-        for(qint32 i=0; i<len; i++)
+        for(qint32 i=0; i < len; i++)
         {
-            const QScriptValue &profile = profiles.property(i);
-            if (!profile.isValid() || profile.isError())
+            const QJSValue &profile = profiles.property(i);
+            if (!profile.isString() || profile.isError())
             {
                 emitOnlineConfigScriptError(profile);
                 return;
@@ -484,30 +483,34 @@ void CRouterBRouterSetup::slotLoadOnlineProfilesRequestFinished(bool ok)
     }
     else
     {
-        const QWebElement &htmlElement = profilesWebPage->mainFrame()->documentElement();
-        const QWebElementCollection &anchorElements = htmlElement.findAll("table tr td a");
+        profilesWebPage->runJavaScript(
+            "var profiles = [];"
+            "var xpathResult = document.evaluate('.//@href',document.body,null,XPathResult.UNORDERED_NODE_ITERATOR_TYPE,null);"
+            "var href = xpathResult.iterateNext();"
+            "while(href) {"
+            "  var pmatch = href.value.match(/(\\S+)\\.brf/);"
+            "  if (pmatch != null) {"
+            "    profiles.push(pmatch[1]);"
+            "  }"
+            "  href = xpathResult.iterateNext();"
+            "}"
+            "profiles;",
+            [this](const QVariant& v) { afterSlotLoadOnlineProfilesRequestFinishedRunJavascript(v); }
+            );
+    }
+}
 
-        if (anchorElements.count() == 0)
-        {
-            emitNetworkError(tr("%1 invalid result").arg(onlineProfilesUrl));
-            return;
-        }
-
-        const QRegExp rxProfileName("(\\S+)\\.brf");
-
-        QStringList onlineProfilesLoaded;
-        for (const QWebElement &anchorElement : anchorElements)
-        {
-            const QString &profileName = anchorElement.toPlainText();
-            //only anchors matching the desired pattern
-            if (rxProfileName.indexIn(profileName) > -1)
-            {
-                onlineProfilesLoaded << rxProfileName.cap(1);
-            }
-        }
+void CRouterBRouterSetup::afterSlotLoadOnlineProfilesRequestFinishedRunJavascript(const QVariant &v)
+{
+    QStringList onlineProfilesLoaded = v.toStringList();
 
+    if (onlineProfilesLoaded.size() == 0)
+    {
+        emitNetworkError(tr("%1 invalid result").arg(onlineProfilesUrl));
+    }
+    else
+    {
         mergeOnlineProfiles(onlineProfilesLoaded);
-
         emit sigProfilesChanged();
     }
 }
@@ -531,14 +534,14 @@ void CRouterBRouterSetup::mergeOnlineProfiles(const QStringList &onlineProfilesL
     }
 }
 
-void CRouterBRouterSetup::emitOnlineConfigScriptError(const QScriptValue &error) const
+void CRouterBRouterSetup::emitOnlineConfigScriptError(const QJSValue &error) const
 {
-    emit sigError(tr("Error parsing online-config:"),error.toString());
+    emit sigError(tr("Error parsing online-config:"), error.toString());
 }
 
 void CRouterBRouterSetup::emitNetworkError(QString error) const
 {
-    emit sigError(tr("Network error:"),error);
+    emit sigError(tr("Network error:"), error);
 }
 
 void CRouterBRouterSetup::displayProfileAsync(const QString &profile) const
@@ -569,7 +572,7 @@ void CRouterBRouterSetup::displayOnlineProfileAsync(const QString &profile) cons
 void CRouterBRouterSetup::loadOnlineProfileAsync(const QString &profile, const profileRequest_e mode) const
 {
     QNetworkReply * reply = networkAccessManager->get(QNetworkRequest(QUrl(onlineProfilesUrl + profile +".brf")));
-    reply->setProperty("type",eTypeProfile);
+    reply->setProperty("type", eTypeProfile);
     reply->setProperty("profile", profile);
     reply->setProperty("request", mode);
 }
