@@ -17,8 +17,8 @@
 
 **********************************************************************************************/
 
-#include "CMainWindow.h"
 #include "canvas/CCanvas.h"
+#include "CMainWindow.h"
 #include "gis/rte/router/brouter/CRouterBRouterSetup.h"
 #include "gis/rte/router/brouter/CRouterBRouterTilesSelect.h"
 #include "gis/rte/router/brouter/CRouterBRouterTilesSelectArea.h"
@@ -26,8 +26,7 @@
 #include "gis/rte/router/brouter/CRouterBRouterTilesStatus.h"
 #include <QNetworkReply>
 #include <QNetworkRequest>
-#include <QWebElement>
-#include <QWebFrame>
+#include <QWebEnginePage>
 
 const QPoint CRouterBRouterTilesSelect::noTile = QPoint(INT_MIN, INT_MIN);
 const int CRouterBRouterTilesSelect::minTileLat = -180;
@@ -35,6 +34,13 @@ const int CRouterBRouterTilesSelect::maxTileLat =  180;
 const int CRouterBRouterTilesSelect::minTileLon =  -85;
 const int CRouterBRouterTilesSelect::maxTileLon =   85;
 const int CRouterBRouterTilesSelect::tileSize   =    5;
+//pattern for filenames of tiles: 'E10_N20.rd5'
+const QString CRouterBRouterTilesSelect::patternTileName = QString("([EW])(\\d{1,3})_([NS])(\\d{1,3})\\.rd5$");
+const QRegExp CRouterBRouterTilesSelect::regExpTileName = QRegExp(CRouterBRouterTilesSelect::patternTileName);
+//pattern for tiles date parsing: '16-Feb-2017 20:48  '
+const QRegExp CRouterBRouterTilesSelect::regExpDate = QRegExp("(\\d{1,2})-(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)-(\\d{4}) (\\d{1,2}):(\\d{2})");
+//pattern for tiles size parsing: 8.2M 271K 9.3K
+const QRegExp CRouterBRouterTilesSelect::regExpSize = QRegExp(" {0,2}(\\d{1,3}|\\d\\.\\d)([KMG])");
 
 CRouterBRouterTilesSelect::CRouterBRouterTilesSelect(QWidget *parent)
     : QWidget(parent)
@@ -44,14 +50,14 @@ CRouterBRouterTilesSelect::CRouterBRouterTilesSelect(QWidget *parent)
         for (int y = minTileLon; y < maxTileLon; y += tileSize)
         {
             CRouterBRouterTilesStatus * status = new CRouterBRouterTilesStatus(this);
-            tilesDownloadStatus.insert(fileNameFromTile(QPoint(x,y)),status);
+            tilesDownloadStatus.insert(fileNameFromTile(QPoint(x, y)), status);
         }
     }
 
     outerLayout = new QVBoxLayout(this);
-    outerLayout->setContentsMargins(0,0,0,0);
+    outerLayout->setContentsMargins(0, 0, 0, 0);
     widgetSelect = new QWidget(this);
-    widgetSelect->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
+    widgetSelect->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
     outerLayout->addWidget(widgetSelect);
     widgetSelect->show();
 
@@ -61,17 +67,17 @@ CRouterBRouterTilesSelect::CRouterBRouterTilesSelect(QWidget *parent)
 
     QSettings view(temp.fileName(), QSettings::IniFormat);
     view.clear();
-    view.setValue("map/zoomIndex",16);
-    view.setValue("scales",1);
-    view.setValue("proj","+proj=merc");
-    view.setValue("grid/proj","+proj=longlat +datum=WGS84 +no_defs");
+    view.setValue("map/zoomIndex", 16);
+    view.setValue("scales", 1);
+    view.setValue("proj", "+proj=merc");
+    view.setValue("grid/proj", "+proj=longlat +datum=WGS84 +no_defs");
 
-    CCanvas * canvas = new CCanvas(widgetSelect,"BRouterTileDownload");
+    CCanvas * canvas = new CCanvas(widgetSelect, "BRouterTileDownload");
     // clone canvas by a temporary configuration file
     canvas->loadConfig(view);
     canvas->setMap("://map/World.gemf");
 
-    selectArea = new CRouterBRouterTilesSelectArea(widgetSelect,canvas);
+    selectArea = new CRouterBRouterTilesSelectArea(widgetSelect, canvas);
 
     QLayout * selectLayout = new CRouterBRouterTilesSelectLayout(widgetSelect);
     selectLayout->addWidget(canvas);
@@ -111,7 +117,7 @@ CRouterBRouterTilesSelect::CRouterBRouterTilesSelect(QWidget *parent)
 
     tilesDownloadManager = new QNetworkAccessManager(this);
 
-    tilesWebPage = new QWebPage(this);
+    tilesWebPage = new QWebEnginePage(this);
 
     connect(pushClearSelection, &QPushButton::clicked, this, &CRouterBRouterTilesSelect::slotClearSelection);
     connect(pushDeleteSelection, &QPushButton::clicked, this, &CRouterBRouterTilesSelect::slotDeleteSelected);
@@ -119,7 +125,7 @@ CRouterBRouterTilesSelect::CRouterBRouterTilesSelect(QWidget *parent)
     connect(pushDownload, &QPushButton::clicked, this, &CRouterBRouterTilesSelect::slotDownload);
     connect(selectArea, &CRouterBRouterTilesSelectArea::sigTileClicked, this, &CRouterBRouterTilesSelect::slotTileClicked);
     connect(selectArea, &CRouterBRouterTilesSelectArea::sigTileToolTipChanged, this, &CRouterBRouterTilesSelect::slotTileToolTipChanged);
-    connect(tilesWebPage, &QWebPage::loadFinished, this, &CRouterBRouterTilesSelect::slotLoadOnlineTilesRequestFinished);
+    connect(tilesWebPage, &QWebEnginePage::loadFinished, this, &CRouterBRouterTilesSelect::slotLoadOnlineTilesRequestFinished);
     connect(tilesDownloadManager, &QNetworkAccessManager::finished, this, &CRouterBRouterTilesSelect::slotDownloadFinished);
 }
 
@@ -146,7 +152,7 @@ void CRouterBRouterTilesSelect::slotSelectOutdated()
 {
     bool changed(false);
 
-    for (QHash<QString,CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
+    for (QHash<QString, CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
          it != tilesDownloadStatus.constEnd();
          ++it)
     {
@@ -167,7 +173,7 @@ void CRouterBRouterTilesSelect::slotDeleteSelected()
 {
     bool changed(false);
 
-    for (QHash<QString,CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
+    for (QHash<QString, CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
          it != tilesDownloadStatus.constEnd();
          ++it)
     {
@@ -199,7 +205,7 @@ void CRouterBRouterTilesSelect::slotDeleteSelected()
 void CRouterBRouterTilesSelect::slotClearSelection()
 {
     bool changed(false);
-    for (QHash<QString,CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
+    for (QHash<QString, CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
          it != tilesDownloadStatus.constEnd();
          ++it)
     {
@@ -223,7 +229,7 @@ void CRouterBRouterTilesSelect::updateButtons() const
     bool hasUnselectedOutdated(false);
     bool hasSelectedForDownload(false);
 
-    for (QHash<QString,CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
+    for (QHash<QString, CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
          it != tilesDownloadStatus.constEnd();
          ++it)
     {
@@ -258,7 +264,7 @@ void CRouterBRouterTilesSelect::initialize()
 {
     try
     {
-        for (QHash<QString,CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
+        for (QHash<QString, CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
              it != tilesDownloadStatus.constEnd();
              ++it)
         {
@@ -277,17 +283,16 @@ void CRouterBRouterTilesSelect::initialize()
         }
 
         const QStringList &segments = dir.entryList();
-        const QRegExp rxTileName("([EW])(\\d{1,3})_([NS])(\\d{1,3})\\.rd5$");
         for (const QString &segment : segments)
         {
-            if (rxTileName.indexIn(segment) > -1)
+            if (regExpTileName.indexIn(segment) > -1)
             {
                 const QPoint& tile = tileFromFileName(segment);
                 if (tile != noTile)
                 {
                     CRouterBRouterTilesStatus * status = getTileStatus(tile);
 
-                    const QFileInfo &info = QFileInfo(dir,segment);
+                    const QFileInfo &info = QFileInfo(dir, segment);
                     status->localDate = info.created();
                     status->localSize = info.size();
                     status->isLocal = true;
@@ -296,7 +301,7 @@ void CRouterBRouterTilesSelect::initialize()
         }
         clearError();
         initialized = true;
-        tilesWebPage->mainFrame()->load(QUrl(setup->segmentsUrl));
+        tilesWebPage->load(QUrl(setup->segmentsUrl));
     }
     catch (const QString &msg)
     {
@@ -308,100 +313,111 @@ void CRouterBRouterTilesSelect::initialize()
 
 void CRouterBRouterTilesSelect::slotLoadOnlineTilesRequestFinished(bool ok)
 {
-    try
+    if (!ok)
     {
-        if (!ok)
+        segmentsError(tr("Network Error"));
+        update();
+        return;
+    }
+    tilesWebPage->runJavaScript(
+        QString(
+            "var tiles = [];"
+            "var xpathResult = document.evaluate('.//a',document.body,null,XPathResult.UNORDERED_NODE_ITERATOR_TYPE,null);"
+            "var reTileName = /").append(patternTileName).append("/;")
+        .append(
+            "var anchor = xpathResult.iterateNext();"
+            "while(anchor) {"
+            "  if(reTileName.test(anchor.innerHTML)) {"
+            "    var dateElement = anchor.parentNode.nextSibling;"
+            "    var sizeElement = dateElement.nextSibling;"
+            "    var tile = {};"
+            "    tile.name = anchor.innerHTML;"
+            "    tile.date = dateElement.innerHTML;"
+            "    tile.size = sizeElement.innerHTML;"
+            "    tiles.push(tile);"
+            "  }"
+            "  anchor = xpathResult.iterateNext();"
+            "}"
+            "tiles;"),
+        [this](const QVariant &v) { afterSlotLoadOnlineTilesRequestFinishedRunJavascript(v); }
+        );
+}
+
+void CRouterBRouterTilesSelect::afterSlotLoadOnlineTilesRequestFinishedRunJavascript(const QVariant &v)
+{
+    const QList<QVariant> & results = v.toList();
+    if (results.count() == 0)
+    {
+        segmentsError(tr("invalid result, no files found"));
+        update();
+        return;
+    }
+
+    for (const QVariant & result : results)
+    {
+        const QMap<QString, QVariant> & tileMap = result.toMap();
+
+        const QString &tileName = tileMap.value("name").toString();
+        //only anchors matching the desired pattern
+        if (regExpTileName.indexIn(tileName) > -1)
         {
-            throw tr("Network Error");
-        }
-        else
-        {
-            const QWebElement &htmlElement = tilesWebPage->mainFrame()->documentElement();
-            const QWebElementCollection &anchorElements = htmlElement.findAll("table tr td a");
+            const QPoint &tile = tileFromFileName(tileName);
 
-            if (anchorElements.count() == 0)
+            if (tile != noTile)
             {
-                throw tr("invalid result, no files found");
-            }
-
-            // 'E10_N20.rd5'
-            const QRegExp rxTileName("([EW])(\\d{1,3})_([NS])(\\d{1,3})\\.rd5");
-
-            // '16-Feb-2017 20:48  '
-            // const QString dateFormat = "d-MMM-yyyy H:mm";
-            // QDateFormat conversion depends on user-locale, doesn't work here
-            const QRegExp rxDate("(\\d{1,2})-(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)-(\\d{4}) (\\d{1,2}):(\\d{2})");
-
-            // 8.2M 271K 9.3K
-            const QRegExp rxSize(" {0,2}(\\d{1,3}|\\d\\.\\d)([KMG])");
-
-            for (const QWebElement &anchorElement : anchorElements)
-            {
-                const QString &tileName = anchorElement.toPlainText();
-                //only anchors matching the desired pattern
-                if (rxTileName.indexIn(tileName) > -1)
+                CRouterBRouterTilesStatus * status = getTileStatus(tile);
+                if (status != nullptr)
                 {
-                    const QWebElement &dateElement = anchorElement.parent().nextSibling();
-                    const QWebElement &sizeElement = dateElement.nextSibling();
+                    status->isRemote = true;
 
-                    const QPoint &tile = tileFromFileName(tileName);
-
-                    if (tile != noTile)
+                    const QString &date = tileMap.value("date").toString();
+                    if (regExpDate.indexIn(date) < 0)
                     {
-                        CRouterBRouterTilesStatus * status = getTileStatus(tile);
-                        if (status != nullptr)
-                        {
-                            status->isRemote = true;
+                        segmentsError(tr("cannot parse: %1 is not a date").arg(date));
+                        update();
+                        return;
+                    }
+                    int day = regExpDate.cap(1).toInt();
+                    const QString &monthStr = regExpDate.cap(2);
+                    int month = monthStr == "Jan" ? 1 :
+                                monthStr == "Feb" ? 2 :
+                                monthStr == "Mar" ? 3 :
+                                monthStr == "Apr" ? 4 :
+                                monthStr == "May" ? 5 :
+                                monthStr == "Jun" ? 6 :
+                                monthStr == "Jul" ? 7 :
+                                monthStr == "Aug" ? 8 :
+                                monthStr == "Sep" ? 9 :
+                                monthStr == "Oct" ? 10 :
+                                monthStr == "Nov" ? 11 :
+                                monthStr == "Dec" ? 12 :
+                                0;
+                    int year = regExpDate.cap(3).toInt();
+                    int hour = regExpDate.cap(4).toInt();
+                    int min  = regExpDate.cap(5).toInt();
 
-                            const QString &date = dateElement.toPlainText();
-                            if (rxDate.indexIn(date) < 0)
-                            {
-                                throw tr("cannot parse: %1 is not a date").arg(date);
-                            }
-                            int day = rxDate.cap(1).toInt();
-                            const QString &monthStr = rxDate.cap(2);
-                            int month = monthStr == "Jan" ? 1 :
-                                        monthStr == "Feb" ? 2 :
-                                        monthStr == "Mar" ? 3 :
-                                        monthStr == "Apr" ? 4 :
-                                        monthStr == "May" ? 5 :
-                                        monthStr == "Jun" ? 6 :
-                                        monthStr == "Jul" ? 7 :
-                                        monthStr == "Aug" ? 8 :
-                                        monthStr == "Sep" ? 9 :
-                                        monthStr == "Oct" ? 10 :
-                                        monthStr == "Nov" ? 11 :
-                                        12;
-                            int year = rxDate.cap(3).toInt();
-                            int hour = rxDate.cap(4).toInt();
-                            int min  = rxDate.cap(5).toInt();
+                    status->remoteDate = QDateTime(QDate(year, month, day), QTime(hour, min, 0));
 
-                            status->remoteDate = QDateTime(QDate(year,month,day),QTime(hour,min,0));
-
-                            const QString &size = sizeElement.toPlainText();
-                            if (rxSize.indexIn(size) < 0)
-                            {
-                                throw tr("cannot parse: %1 is not a valid size").arg(size);
-                            }
-                            status->remoteSize = rxSize.cap(1).toFloat() * (rxSize.cap(2) == "M" ? 1048576 :
-                                                                            rxSize.cap(2) == "G" ? 1073741824 :
-                                                                            rxSize.cap(2) == "K" ? 1024 :
-                                                                            1);
-                            if (status->isLocal && status->remoteDate > status->localDate)
-                            {
-                                status->isOutdated = true;
-                            }
-                        }
+                    const QString &size = tileMap.value("size").toString();
+                    if (regExpSize.indexIn(size) < 0)
+                    {
+                        segmentsError(tr("cannot parse: %1 is not a valid size").arg(size));
+                        update();
+                        return;
+                    }
+                    status->remoteSize = regExpSize.cap(1).toFloat() * (regExpSize.cap(2) == "M" ? 1048576 :
+                                                                        regExpSize.cap(2) == "G" ? 1073741824 :
+                                                                        regExpSize.cap(2) == "K" ? 1024 :
+                                                                        1);
+                    if (status->isLocal && status->remoteDate > status->localDate)
+                    {
+                        status->isOutdated = true;
                     }
                 }
             }
         }
-        clearError();
-    }
-    catch (const QString &msg)
-    {
-        error(tr("Error retrieving available routing data from %1: %2").arg(setup->segmentsUrl).arg(msg));
     }
+    clearError();
     update();
 }
 
@@ -439,12 +455,10 @@ QString CRouterBRouterTilesSelect::formatSize(const qint64 size)
 
 QPoint CRouterBRouterTilesSelect::tileFromFileName(const QString &fileName)
 {
-    // 'E10_N20.rd5'
-    const QRegExp rxTileName("([EW])(\\d{1,3})_([NS])(\\d{1,3})\\.rd5");
-    if (rxTileName.indexIn(fileName) > -1)
+    if (regExpTileName.indexIn(fileName) > -1)
     {
-        return QPoint(rxTileName.cap(2).toInt() * (rxTileName.cap(1) == "E" ? 1 : -1)
-                      ,rxTileName.cap(4).toInt() * (rxTileName.cap(3) == "N" ? 1 : -1));
+        return QPoint(regExpTileName.cap(2).toInt() * (regExpTileName.cap(1) == "E" ? 1 : -1)
+                      , regExpTileName.cap(4).toInt() * (regExpTileName.cap(3) == "N" ? 1 : -1));
     }
     else
     {
@@ -454,9 +468,9 @@ QPoint CRouterBRouterTilesSelect::tileFromFileName(const QString &fileName)
 
 QString CRouterBRouterTilesSelect::fileNameFromTile(const QPoint tile)
 {
-    return QString("%1%2_%3%4.rd5").arg(tile.x()<0 ? "W" : "E")
+    return QString("%1%2_%3%4.rd5").arg(tile.x() < 0 ? "W" : "E")
            .arg(abs(tile.x()))
-           .arg(tile.y()<0 ? "S" : "N")
+           .arg(tile.y() < 0 ? "S" : "N")
            .arg(abs(tile.y()));
 }
 
@@ -469,7 +483,7 @@ void CRouterBRouterTilesSelect::slotDownload()
 {
     bool changed(false);
 
-    for (QHash<QString,CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constBegin();
+    for (QHash<QString, CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constBegin();
          it != tilesDownloadStatus.constEnd();
          ++it)
     {
@@ -524,7 +538,7 @@ void CRouterBRouterTilesSelect::slotDownloadReadReady()
             try
             {
                 const QString &fileName = reply->property("tile").toString();
-                QHash<QString,CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constFind(fileName);
+                QHash<QString, CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constFind(fileName);
                 if (it == tilesDownloadStatus.constEnd())
                 {
                     throw tr("no valid request for filename %1").arg(fileName);
@@ -564,7 +578,7 @@ void CRouterBRouterTilesSelect::slotDownloadFinished(QNetworkReply* reply)
     reply->deleteLater();
 
     const QString &fileName = reply->property("tile").toString();
-    QHash<QString,CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constFind(fileName);
+    QHash<QString, CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constFind(fileName);
     if (it == tilesDownloadStatus.constEnd())
     {
         error(tr("no valid request for filename %1").arg(fileName));
@@ -654,7 +668,7 @@ void CRouterBRouterTilesSelect::updateStatus()
 
     bool downloading(false);
 
-    for (QHash<QString,CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constBegin();
+    for (QHash<QString, CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constBegin();
          it != tilesDownloadStatus.constEnd();
          ++it)
     {
@@ -690,7 +704,7 @@ void CRouterBRouterTilesSelect::updateStatus()
                          .arg(numOutdated).arg(formatSize(sizeOutdated))
                          .arg(numOutstanding).arg(formatSize(sizeOutstanding)));
     statusProgress->setVisible(downloading);
-    statusProgress->setRange(0,sizeDownloadMax);
+    statusProgress->setRange(0, sizeDownloadMax);
     statusProgress->setValue(sizeDownloaded);
 
     if (this->downloading != downloading || (numOutstanding > 0) != downloadSelected)
@@ -707,6 +721,11 @@ void CRouterBRouterTilesSelect::error(const QString &error) const
     errorLabel->setVisible(true);
 }
 
+void CRouterBRouterTilesSelect::segmentsError(const QString &msg) const
+{
+    error(tr("Error retrieving available routing data from %1: %2").arg(setup->segmentsUrl).arg(msg));
+}
+
 void CRouterBRouterTilesSelect::clearError() const
 {
     errorLabel->setVisible(false);
@@ -714,7 +733,7 @@ void CRouterBRouterTilesSelect::clearError() const
 
 CRouterBRouterTilesStatus * CRouterBRouterTilesSelect::getTileStatus(QPoint tile) const
 {
-    QHash<QString,CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constFind(fileNameFromTile(tile));
+    QHash<QString, CRouterBRouterTilesStatus*>::const_iterator it = tilesDownloadStatus.constFind(fileNameFromTile(tile));
     if (it != tilesDownloadStatus.constEnd())
     {
         return it.value();
@@ -731,7 +750,7 @@ void CRouterBRouterTilesSelect::updateTiles() const
     QVector<QPoint> outstandingTiles;
     QVector<QPoint> selectedTiles;
 
-    for (QHash<QString,CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
+    for (QHash<QString, CRouterBRouterTilesStatus * >::const_iterator it = tilesDownloadStatus.constBegin();
          it != tilesDownloadStatus.constEnd();
          ++it)
     {
