@@ -25,6 +25,7 @@
 #include "gis/trk/filter/CFilterDouglasPeuker.h"
 #include "gis/trk/filter/CFilterInterpolateElevation.h"
 #include "gis/trk/filter/CFilterInvalid.h"
+#include "gis/trk/filter/CFilterLoopsCut.h"
 #include "gis/trk/filter/CFilterMedian.h"
 #include "gis/trk/filter/CFilterNewDate.h"
 #include "gis/trk/filter/CFilterObscureDate.h"
@@ -35,6 +36,7 @@
 #include "gis/trk/filter/CFilterSplitSegment.h"
 #include "gis/trk/filter/CFilterSubPt2Pt.h"
 #include "gis/trk/filter/CFilterTerrainSlope.h"
+#include "gis/trk/filter/CFilterZeroSpeedDriftCleaner.h"
 #include "helpers/CLinksDialog.h"
 #include "helpers/CSettings.h"
 #include "helpers/Signals.h"
@@ -43,36 +45,33 @@
 #include "units/IUnit.h"
 #include "widgets/CTextEditWidget.h"
 
-#include <QtWidgets>
 #include <proj_api.h>
-
-#include <functional>
-
-using std::bind;
+#include <QtWidgets>
 
 /* base case: add the filter specified in template parameter */
 template<typename filter>
-static void addFilters(QTreeWidgetItem *itemGroup, CGisItemTrk& trk)
+static void addFilters(QTreeWidgetItem *itemGroup, CGisItemTrk& trk, qint32& minWidth)
 {
     QTreeWidgetItem *item = new QTreeWidgetItem(itemGroup);
     itemGroup->treeWidget()->setItemWidget(item, /* column = */ 0, new filter(trk, itemGroup->treeWidget()));
+    minWidth = qMax(minWidth, itemGroup->treeWidget()->itemWidget(item, 0)->sizeHint().width());
 }
 
 template<typename filter1, typename filter2, typename ... remainingFilters>
-static void addFilters(QTreeWidgetItem *itemGroup, CGisItemTrk& trk)
+static void addFilters(QTreeWidgetItem *itemGroup, CGisItemTrk& trk, qint32& minWidth)
 {
-    addFilters<filter1>(itemGroup, trk);
-    addFilters<filter2, remainingFilters ...>(itemGroup, trk);
+    addFilters<filter1>(itemGroup, trk, minWidth);
+    addFilters<filter2, remainingFilters ...>(itemGroup, trk, minWidth);
 }
 
 template<typename ... filters>
-static void addFilterGroup(QTreeWidget *widget, CGisItemTrk& trk, const QString &groupText, const QString &groupIcon)
+static void addFilterGroup(QTreeWidget *widget, CGisItemTrk& trk, const QString &groupText, const QString &groupIcon, qint32& minWidth)
 {
     QTreeWidgetItem *itemGroup = new QTreeWidgetItem(widget);
     itemGroup->setIcon(/* column = */ 0, QIcon(groupIcon));
     itemGroup->setText(/* column = */ 0, groupText);
 
-    addFilters<filters ...>(itemGroup, trk);
+    addFilters<filters ...>(itemGroup, trk, minWidth);
 }
 
 CDetailsTrk::CDetailsTrk(CGisItemTrk& trk)
@@ -80,22 +79,32 @@ CDetailsTrk::CDetailsTrk(CGisItemTrk& trk)
     , trk(trk)
 {
     setupUi(this);
-
-    QPixmap icon(14,14);
-
-    for(int i=0; i < TRK_N_COLORS; ++i)
+    QPixmap icon(14, 14);
+    const int N = IGisItem::getColorMap().count();
+    for(int i=0; i < N; ++i)
     {
-        icon.fill(IGisItem::colorMap[i].color);
-        comboColor->addItem(icon, IGisItem::colorMap[i].name, IGisItem::colorMap[i].color);
+        const IGisItem::color_t& color = IGisItem::getColorMap()[i];
+        icon.fill(color.color);
+        comboColor->addItem(icon, color.label, color.color);
     }
 
+    labelHelpTrackPointInfo->setText(tr( "<b>Track Point Information</b><br/>"
+                                         "To document your tracks you can add a short description to a track "
+                                         "point. All points with description will be marked along the track "
+                                         "with auto-numbered bullets.<br/>"
+                                         "You can add a description by a right click on a track point in the graphs. "
+                                         "Or you do a left click on a track point in the map view.<br/>"
+                                         "With the first track point you add information to this text will be replaced "
+                                         "by a table. Use it to edit and delete track point information.<br/>"
+                                         ));
+
     widgetColorLayout->setAlignment(Qt::AlignTop);
 
     widgetColorActivity->setTrack(&trk);
 
-    updateData();
-
-    treeWidget->setTrack(&trk);
+    connect(treeTrackPointInfo, &CTableTrkInfo::sigHasTrkPtInfo, this, &CDetailsTrk::slotHasTrkPtInfo);
+    treeTrackPoint->setTrack(&trk);
+    treeTrackPointInfo->setTrack(&trk);
 
     plot1 = new CPlotProfile(&trk, trk.limitsGraph1, IPlot::eModeNormal, this);
     plot2 = new CPlot(&trk, trk.limitsGraph2, this);
@@ -106,7 +115,7 @@ CDetailsTrk::CDetailsTrk(CGisItemTrk& trk)
         plot->setMinimumSize(QSize(0, 100));
         plot->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
         plot->show();
-        layoutPlot->addWidget(plot);
+        framePlot->layout()->addWidget(plot);
 
         const CTrackData::trkpt_t *ptRange1 = nullptr, *ptRange2 = nullptr;
         trk.getMouseRange(ptRange1, ptRange2);
@@ -123,7 +132,9 @@ CDetailsTrk::CDetailsTrk(CGisItemTrk& trk)
     checkGraph1->setChecked(cfg.value("showGraph1", true).toBool());
     checkGraph2->setChecked(cfg.value("showGraph2", true).toBool());
     checkGraph3->setChecked(cfg.value("showGraph3", true).toBool());
-    splitter->restoreState (cfg.value("splitterSizes").toByteArray());
+    splitter1->restoreState (cfg.value("splitterSizes1").toByteArray());
+    splitter2->restoreState (cfg.value("splitterSizes2").toByteArray());
+    splitter3->restoreState (cfg.value("splitterSizes3").toByteArray());
     tabWidget->setCurrentIndex(cfg.value("visibleTab", 0).toInt());
     cfg.endGroup();
 
@@ -143,7 +154,7 @@ CDetailsTrk::CDetailsTrk(CGisItemTrk& trk)
     connect(plot2,            &CPlot::sigMouseClickState,          this, &CDetailsTrk::slotMouseClickState);
     connect(plot3,            &CPlot::sigMouseClickState,          this, &CDetailsTrk::slotMouseClickState);
 
-    connect(spinLineWidth,    static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged), this, &CDetailsTrk::slotLineWidth);
+    connect(spinLineWidth,    static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &CDetailsTrk::slotLineWidth);
     connect(checkWithArrows,  &QCheckBox::toggled, this, &CDetailsTrk::slotWithArrows);
 
     setupStyleLimits(trk.colorSourceLimit, toolLimitAutoStyle, toolLimitUsrStyle, toolLimitSysStyle, spinLimitLow, spinLimitHigh);
@@ -168,19 +179,25 @@ CDetailsTrk::CDetailsTrk(CGisItemTrk& trk)
     loadGraphSource(comboGraph2, 2, CKnownExtension::internalSpeedDist);
     loadGraphSource(comboGraph3, 3, CKnownExtension::internalProgress);
 
-    addFilterGroup<CFilterDouglasPeuker, CFilterInvalid, CFilterReset, CFilterDelete>
-        (treeFilter, trk, tr("Reduce visible track points"), "://icons/48x48/PointHide.png");
+    // this will the largest minimum width of all filter widgets
+    qint32 minWidth = 0;
+    addFilterGroup<CFilterDouglasPeuker, CFilterZeroSpeedDriftCleaner, CFilterInvalid, CFilterReset, CFilterDelete>
+        (treeFilter, trk, tr("Reduce visible track points"), "://icons/48x48/PointHide.png", minWidth);
 
     addFilterGroup<CFilterMedian, CFilterInterpolateElevation, CFilterReplaceElevation, CFilterOffsetElevation>
-        (treeFilter, trk, tr("Change elevation of track points"), "://icons/48x48/SetEle.png");
+        (treeFilter, trk, tr("Change elevation of track points"), "://icons/48x48/SetEle.png", minWidth);
 
     addFilterGroup<CFilterNewDate, CFilterObscureDate, CFilterSpeed>
-        (treeFilter, trk, tr("Change timestamp of track points"), "://icons/48x48/Time.png");
+        (treeFilter, trk, tr("Change timestamp of track points"), "://icons/48x48/Time.png", minWidth);
 
-    addFilterGroup<CFilterDeleteExtension, CFilterSplitSegment, CFilterSubPt2Pt, CFilterTerrainSlope, CFilterChangeStartPoint>
-        (treeFilter, trk, tr("Miscellaneous"), "://icons/48x48/CSrcUnknown.png");
+    addFilterGroup<CFilterDeleteExtension, CFilterSplitSegment, CFilterSubPt2Pt, CFilterTerrainSlope, CFilterChangeStartPoint, CFilterLoopsCut>
+        (treeFilter, trk, tr("Miscellaneous"), "://icons/48x48/CSrcUnknown.png", minWidth);
+
+    // limit tree widget horizontal size to the filter widget with the largest minimum size
+    treeFilter->setMinimumWidth(minWidth + treeFilter->indentation());
 
     slotShowPlots();
+    updateData();
 }
 
 CDetailsTrk::~CDetailsTrk()
@@ -190,7 +207,9 @@ CDetailsTrk::~CDetailsTrk()
     cfg.setValue("showGraph1",    checkGraph1->isChecked());
     cfg.setValue("showGraph2",    checkGraph2->isChecked());
     cfg.setValue("showGraph3",    checkGraph3->isChecked());
-    cfg.setValue("splitterSizes", splitter->saveState());
+    cfg.setValue("splitterSizes1", splitter1->saveState());
+    cfg.setValue("splitterSizes2", splitter2->saveState());
+    cfg.setValue("splitterSizes3", splitter3->saveState());
     cfg.setValue("visibleTab",    tabWidget->currentIndex());
     cfg.endGroup();
 
@@ -202,6 +221,12 @@ CDetailsTrk::~CDetailsTrk()
     trk.clearDlgDetails();
 }
 
+void CDetailsTrk::slotHasTrkPtInfo(bool yes)
+{
+    labelHelpTrackPointInfo->setVisible(!yes);
+    treeTrackPointInfo->setVisible(yes);
+}
+
 void CDetailsTrk::slotSetLimitModeStyle(CLimit::mode_e mode, bool on)
 {
     if(!on)
@@ -248,12 +273,9 @@ void CDetailsTrk::setupGraphLimits(CLimit& limit, QToolButton * toolLimitAutoGra
     connect(toolLimitAutoGraph, &QToolButton::toggled, spinMinGraph, &QDoubleSpinBox::setDisabled);
     connect(toolLimitAutoGraph, &QToolButton::toggled, spinMaxGraph, &QDoubleSpinBox::setDisabled);
 
-    /* creates "lambdas" which look like:
-     *   (bool on) { slotSetLimitModeGraph(CLimit::eModeAuto, &limit, spinMinGraph, spinMaxGraph, on); }
-     */
-    auto limitAutoFunc = bind(&CDetailsTrk::slotSetLimitModeGraph, this, CLimit::eModeAuto, &limit, spinMinGraph, spinMaxGraph, std::placeholders::_1);
-    auto limitUserFunc = bind(&CDetailsTrk::slotSetLimitModeGraph, this, CLimit::eModeUser, &limit, spinMinGraph, spinMaxGraph, std::placeholders::_1);
-    auto limitSysFunc  = bind(&CDetailsTrk::slotSetLimitModeGraph, this, CLimit::eModeSys,  &limit, spinMinGraph, spinMaxGraph, std::placeholders::_1);
+    auto limitAutoFunc = [this, &limit, spinMinGraph, spinMaxGraph](bool checked){slotSetLimitModeGraph(CLimit::eModeAuto, &limit, spinMinGraph, spinMaxGraph, checked);};
+    auto limitUserFunc = [this, &limit, spinMinGraph, spinMaxGraph](bool checked){slotSetLimitModeGraph(CLimit::eModeUser, &limit, spinMinGraph, spinMaxGraph, checked);};
+    auto limitSysFunc = [this, &limit, spinMinGraph, spinMaxGraph](bool checked){slotSetLimitModeGraph(CLimit::eModeSys, &limit, spinMinGraph, spinMaxGraph, checked);};
 
     connect(toolLimitAutoGraph, &QToolButton::toggled, this, limitAutoFunc);
     connect(toolLimitUsrGraph,  &QToolButton::toggled, this, limitUserFunc);
@@ -262,7 +284,7 @@ void CDetailsTrk::setupGraphLimits(CLimit& limit, QToolButton * toolLimitAutoGra
     connect(spinMinGraph, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged), &limit, &CLimit::setMin);
     connect(spinMaxGraph, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged), &limit, &CLimit::setMax);
 
-    auto limitChangedFunc = bind(&CDetailsTrk::setupLimits, this, &limit, spinMinGraph, spinMaxGraph);
+    auto limitChangedFunc = [this, &limit, spinMinGraph, spinMaxGraph](){setupLimits(&limit, spinMinGraph, spinMaxGraph);};
     connect(&limit, &CLimit::sigChanged, this, limitChangedFunc);
 }
 
@@ -296,17 +318,16 @@ void CDetailsTrk::setupStyleLimits(CLimit& limit, QToolButton *toolLimitAuto, QT
     connect(toolLimitAuto, &QToolButton::toggled, spinMax, &QDoubleSpinBox::setDisabled);
     connect(toolLimitAuto, &QToolButton::toggled, spinMin, &QDoubleSpinBox::setDisabled);
 
-    connect(toolLimitAuto, &QToolButton::toggled, bind(&CDetailsTrk::slotSetLimitModeStyle, this, CLimit::eModeAuto, std::placeholders::_1));
-    connect(toolLimitUsr,  &QToolButton::toggled, bind(&CDetailsTrk::slotSetLimitModeStyle, this, CLimit::eModeUser, std::placeholders::_1));
-    connect(toolLimitSys,  &QToolButton::toggled, bind(&CDetailsTrk::slotSetLimitModeStyle, this, CLimit::eModeSys,  std::placeholders::_1));
+    connect(toolLimitAuto, &QToolButton::toggled, [this](bool checked){slotSetLimitModeStyle(CLimit::eModeAuto, checked);});
+    connect(toolLimitUsr,  &QToolButton::toggled, [this](bool checked){slotSetLimitModeStyle(CLimit::eModeUser, checked);});
+    connect(toolLimitSys,  &QToolButton::toggled, [this](bool checked){slotSetLimitModeStyle(CLimit::eModeSys, checked);});
 
     connect(spinMax,       &CDoubleSpinBox::valueChangedByStep, this, &CDetailsTrk::slotColorLimitHighChanged);
     connect(spinMax,       &CDoubleSpinBox::editingFinished,    this, &CDetailsTrk::slotColorLimitHighChanged);
     connect(spinMin,       &CDoubleSpinBox::valueChangedByStep, this, &CDetailsTrk::slotColorLimitLowChanged);
     connect(spinMin,       &CDoubleSpinBox::editingFinished,    this, &CDetailsTrk::slotColorLimitLowChanged);
 
-    auto limitChangedFunc = bind(&CDetailsTrk::setupLimits, this, &limit, spinMin, spinMax);
-    connect(&limit,        &CLimit::sigChanged, this, limitChangedFunc);
+    connect(&limit, &CLimit::sigChanged, this, [this, &limit, spinMin, spinMax] {setupLimits(&limit, spinMin, spinMax);});
 }
 
 void CDetailsTrk::loadGraphSource(QComboBox * comboBox, qint32 n, const QString cfgDefault)
@@ -325,7 +346,7 @@ void CDetailsTrk::loadGraphSource(QComboBox * comboBox, qint32 n, const QString
     // signals are unblocked by now changing the combobox will trigger a graph update
     SETTINGS;
     cfg.beginGroup("TrackDetails");
-    int i = comboBox->findData(cfg.value(QString("propGraph%1").arg(n),cfgDefault).toString());
+    int i = comboBox->findData(cfg.value(QString("propGraph%1").arg(n), cfgDefault).toString());
     if(i != NOIDX)
     {
         comboBox->setCurrentIndex(i);
@@ -357,6 +378,8 @@ void CDetailsTrk::updateData()
     bool isReadOnly = trk.isReadOnly();
     bool isNogo = trk.isNogo();
 
+    frameTrackInfo->setVisible(CMainWindow::self().isShowTrackInfoPoints());
+
     tabWidget->widget(eTabFilter)->setEnabled(!isReadOnly);
 
     labelTainted->setVisible(trk.isTainted());
@@ -378,8 +401,7 @@ void CDetailsTrk::updateData()
     trk.getActivities().printSummary(str);
     labelActivityInfo->setText(str);
 
-    quint32 flags = trk.getActivities().getAllFlags();
-    bool hasActivity = 0 != (flags & CTrackData::trkpt_t::eActMask);
+    bool hasActivity = !trk.getActivities().getAllActivities().isEmpty();
     labelActivityHelp->setVisible(!hasActivity);
     labelActivityInfo->setVisible(hasActivity);
     pushSetActivities->setEnabled(!isReadOnly);
@@ -398,6 +420,10 @@ void CDetailsTrk::updateData()
         }
     }
 
+    plot1->updateData();
+    plot2->updateData();
+    plot3->updateData();
+
     X______________BlockAllSignals______________X(this);
 
     spinLineWidth->setValue(trk.lineScale.val().toDouble());
@@ -431,7 +457,7 @@ void CDetailsTrk::updateData()
     bool enabledColorize = !source.isEmpty() && (source != "activity");
     bool enabledActivity = source == "activity";
 
-    comboColor->setVisible(!(enabledColorize||enabledActivity));
+    comboColor->setVisible(!(enabledColorize || enabledActivity));
     widgetColorLabel->setVisible(enabledColorize);
     widgetColorLabel->setEnabled(enabledColorize);
     toolLimitAutoStyle->setEnabled(enabledColorize);
@@ -492,7 +518,6 @@ void CDetailsTrk::updateData()
     }
 
     enableTabFilter();
-
     originator = false;
     CCanvas::restoreOverrideCursor("CDetailsTrk::updateData");
 }
@@ -540,9 +565,9 @@ void CDetailsTrk::setMouseClickFocus(const CTrackData::trkpt_t *pt)
 {
     if(nullptr != pt)
     {
-        treeWidget->blockSignals(true);
-        treeWidget->setCurrentItem(treeWidget->topLevelItem(pt->idxTotal));
-        treeWidget->blockSignals(false);
+        treeTrackPoint->blockSignals(true);
+        treeTrackPoint->setCurrentItem(treeTrackPoint->topLevelItem(pt->idxTotal));
+        treeTrackPoint->blockSignals(false);
     }
 }
 
@@ -592,6 +617,7 @@ void CDetailsTrk::slotShowPlots()
     plot1->setVisible(checkGraph1->isChecked());
     plot2->setVisible(checkGraph2->isChecked());
     plot3->setVisible(checkGraph3->isChecked());
+    updateData();
 }
 
 void CDetailsTrk::slotColorChanged(int idx)
@@ -669,11 +695,7 @@ void CDetailsTrk::slotLinkActivated(const QUrl& url)
 
 void CDetailsTrk::slotSetActivities()
 {
-    quint32 flags = CActivityTrk::selectActivity(this);
-    if(0xFFFFFFFF != flags)
-    {
-        trk.setActivity(flags);
-    }
+    CActivityTrk::getMenu(trk.getKey(), this, true);
 }
 
 void CDetailsTrk::setupGraph(CPlot * plot, const CLimit& limit, const QString& source, QDoubleSpinBox * spinMin, QDoubleSpinBox * spinMax)
@@ -724,7 +746,7 @@ void CDetailsTrk::slotLineWidthMode(bool isUser)
     spinLineWidth->setValue(trk.lineScale.val().toDouble());
 }
 
-void CDetailsTrk::slotLineWidth(qreal f)
+void CDetailsTrk::slotLineWidth(int f)
 {
     trk.lineScale = f;
 }
