@@ -17,24 +17,25 @@
 **********************************************************************************************/
 
 #include "CMainWindow.h"
-#include "GeoMath.h"
 #include "gis/CGisDraw.h"
 #include "gis/CGisWorkspace.h"
 #include "gis/prj/IGisProject.h"
 #include "gis/trk/CCutTrk.h"
 #include "gis/trk/CDetailsTrk.h"
 #include "gis/trk/CGisItemTrk.h"
+#include "gis/trk/CInvalidTrk.h"
 #include "gis/trk/CKnownExtension.h"
 #include "gis/trk/CPropertyTrk.h"
 #include "gis/trk/CScrOptTrk.h"
 #include "gis/wpt/CGisItemWpt.h"
+#include "GeoMath.h"
 #include "helpers/CDraw.h"
 #include "helpers/CProgressDialog.h"
 #include "helpers/CSettings.h"
 
+#include <proj_api.h>
 #include <QtWidgets>
 #include <QtXml>
-#include <proj_api.h>
 
 #define DEFAULT_COLOR       4
 #define MIN_DIST_CLOSE_TO   10
@@ -43,6 +44,119 @@
 #define WPT_FOCUS_DIST_IN   (50*50)
 #define WPT_FOCUS_DIST_OUT  (200*200)
 
+namespace
+{
+// helper to declutter and draw clusters of track info points
+class cluster
+{
+public:
+    cluster(int number, const QRect& r) : box(r)
+    {
+        elements.append({number});
+    }
+    int size() const {return elements.size();}
+
+    static void addToCluster(const QRect& r, QList<cluster>& clusters, int number)
+    {
+        for(cluster& cluster : clusters)
+        {
+            if(cluster.box.intersects(r))
+            {
+                cluster.box &= r;
+                cluster.elements.append(number);
+                return;
+            }
+        }
+        clusters.append({number, r});
+    }
+
+    static void draw(const QList<cluster>&clusters, QPainter& p, int size)
+    {
+        for(const cluster& cluster : clusters)
+        {
+            const int N = cluster.size();
+            if(N >= MAX_CONST_SIZE)
+            {
+                continue;
+            }
+
+            const QPointF* constellation = constellations[N];
+            for(int n = 0; n < N; n++)
+            {
+                CDraw::number(cluster.elements[n], size, p, cluster.box.center() + constellation[n] * size, Qt::black);
+            }
+        }
+    }
+
+private:
+    QRect box;
+    QList<int> elements;
+
+    static const QPointF point1[];
+    static const QPointF point2[];
+    static const QPointF point3[];
+    static const QPointF point4[];
+    static const QPointF point5[];
+    static const QPointF point6[];
+    static const QPointF point7[];
+    static const QPointF point8[];
+    static const QPointF point9[];
+
+    static constexpr int MAX_CONST_SIZE = 9;
+    static const QPointF * constellations[MAX_CONST_SIZE];
+};
+
+const QPointF cluster::point1[] = {
+    {0, 0}
+};
+
+const QPointF cluster::point2[] = {
+    {-0.5, 0}, {0.5, 0}
+};
+
+const QPointF cluster::point3[] = {
+    {  -1, 0}, {  0, 0}, {1, 0}
+};
+
+const QPointF cluster::point4[] = {
+    {-1, -0.5}, {0, -0.5}, {1, -0.5},
+    {-1, 0.5}
+};
+
+const QPointF cluster::point5[] = {
+    {-1, -0.5}, {0, -0.5}, {1, -0.5},
+    {-1, 0.5}, {0, 0.5}
+};
+
+const QPointF cluster::point6[] = {
+    {-1, -0.5}, {0, -0.5}, {1, -0.5},
+    {-1, 0.5}, {0, 0.5}, {1, 0.5}
+};
+
+const QPointF cluster::point7[] = {
+    {-1, -1}, {0, -1}, {1, -1},
+    {-1, 0}, {0, 0}, {1, 0},
+    {-1, 1}
+};
+
+const QPointF cluster::point8[] = {
+    {-1, -1}, {0, -1}, {1, -1},
+    {-1, 0}, {0, 0}, {1, 0},
+    {-1, 1}, {0, 1}
+};
+
+const QPointF cluster::point9[] = {
+    {-1, -1}, {0, -1}, {1, -1},
+    {-1, 0}, {0, 0}, {1, 0},
+    {-1, 1}, {0, 1}, {1, 1}
+};
+
+const QPointF * cluster::constellations[MAX_CONST_SIZE] =
+{
+    point1, point2, point3, point4, point5, point6, point7, point8, point9
+};
+
+
 struct trkwpt_t
 {
     QString name;
@@ -50,6 +164,7 @@ struct trkwpt_t
     qreal y = 0;
     IGisItem::key_t key;
 };
+}
 
 IGisItem::key_t CGisItemTrk::keyUserFocus;
 
@@ -121,17 +236,7 @@ CGisItemTrk::CGisItemTrk(const QDomNode& xml, IGisProject *project)
     setupHistory();
     updateDecoration(eMarkNone, eMarkNone);
 
-    if((cntInvalidPoints != 0) && (cntInvalidPoints < cntVisiblePoints))
-    {
-        int res = QMessageBox::question(CMainWindow::self().getBestWidgetForParent(), tr("Invalid points...."),
-                                        tr("The track '%1' has %2 invalid points out of %3 visible points. "
-                                           "Do you want to hide invalid points now?").arg(getName()).arg(cntInvalidPoints).arg(cntVisiblePoints),
-                                        QMessageBox::Yes|QMessageBox::No, QMessageBox::No);
-        if(res == QMessageBox::Yes)
-        {
-            filterRemoveInvalidPoints();
-        }
-    }
+    checkForInvalidPoints();
 }
 
 CGisItemTrk::CGisItemTrk(const QString& filename, IGisProject * project)
@@ -147,6 +252,8 @@ CGisItemTrk::CGisItemTrk(const QString& filename, IGisProject * project)
 
     setupHistory();
     updateDecoration(eMarkNone, eMarkNone);
+
+    checkForInvalidPoints();
 }
 
 CGisItemTrk::CGisItemTrk(const history_t& hist, const QString &dbHash, IGisProject * project)
@@ -173,6 +280,8 @@ CGisItemTrk::CGisItemTrk(CTrackData& trkdata, IGisProject *project)
     setupHistory();
     deriveSecondaryData();
     updateDecoration(eMarkNone, eMarkNone);
+
+    checkForInvalidPoints();
 }
 
 CGisItemTrk::CGisItemTrk(CFitStream& stream, IGisProject * project)
@@ -186,6 +295,8 @@ CGisItemTrk::CGisItemTrk(CFitStream& stream, IGisProject * project)
     setupHistory();
     deriveSecondaryData();
     updateDecoration(eMarkNone, eMarkNone);
+
+    checkForInvalidPoints();
 }
 
 
@@ -327,6 +438,11 @@ QString CGisItemTrk::getInfoLimits() const
 
     for(const QString& key : keys)
     {
+        if(key == CKnownExtension::internalSpeedTime)  // Output of "Speed*" will already be covered by internalSpeedDist
+        {                                              // No need to show it twice
+            continue;
+        }
+
         const CKnownExtension& ext = CKnownExtension::get(key);
         const limits_t& limit = extrema[key];
 
@@ -358,14 +474,14 @@ QString CGisItemTrk::getInfo(quint32 feature) const
 
     QString str = "<div>";
 
-    qint32 actCnt       = activities.getActivityCount();
-    quint32 actFlags    = activities.getAllFlags();
+    qint32 actCnt               = activities.getActivityCount();
+    const QSet<trkact_t>& acts  = activities.getAllActivities();
 
     if(feature & eFeatureShowName)
     {
-        if((actCnt == 1) && actFlags)
+        if((actCnt == 1) && (acts.toList().first() != CTrackData::trkpt_t::eAct20None))
         {
-            const CActivityTrk::desc_t& desc = activities.getDescriptor(actFlags);
+            const CActivityTrk::desc_t& desc = activities.getDescriptor(acts.toList().first());
             str += QString("<img src='%1'/>&nbsp;").arg(desc.iconSmall);
         }
         str += "<b>" + getName() + "</b>";
@@ -495,6 +611,17 @@ QString CGisItemTrk::getInfo(quint32 feature) const
         }
     }
 
+    if((feature & eFeatureShowLinks) && !trk.links.isEmpty())
+    {
+        for(const link_t& link : trk.links)
+        {
+            if(link.type.isEmpty() || (link.type == "text/html"))
+            {
+                str += "<br/>\n";
+                str += QString("<a href='%1'>%2</a>").arg(link.uri.toString()).arg(link.text);
+            }
+        }
+    }
 
     return str + "</div>";
 }
@@ -850,6 +977,7 @@ void CGisItemTrk::updateExtremaAndExtensions()
         existingExtensions << CKnownExtension::internalSpeedDist;
         existingExtensions << CKnownExtension::internalSpeedTime;
         extrema[CKnownExtension::internalSpeedDist] = extremaSpeed;
+        extrema[CKnownExtension::internalSpeedTime] = extremaSpeed;
     }
 
     if(numeric_limits<qreal>::max() != extremaProgress.min)
@@ -905,8 +1033,8 @@ void CGisItemTrk::consolidatePoints()
             continue;
         }
 
-        seg.pts.first().unsetFlag(CTrackData::trkpt_t::eSubpt);
-        seg.pts.last().unsetFlag(CTrackData::trkpt_t::eSubpt);
+        seg.pts.first().unsetFlag(CTrackData::trkpt_t::eFlagSubpt);
+        seg.pts.last().unsetFlag(CTrackData::trkpt_t::eFlagSubpt);
     }
 }
 
@@ -1023,7 +1151,7 @@ void CGisItemTrk::deriveSecondaryData()
         lastTrkpt = &trkpt;
     }
 
-    boundingRect = QRectF(QPointF(west * DEG_TO_RAD, north * DEG_TO_RAD), QPointF(east * DEG_TO_RAD,south * DEG_TO_RAD));
+    boundingRect = QRectF(QPointF(west * DEG_TO_RAD, north * DEG_TO_RAD), QPointF(east * DEG_TO_RAD, south * DEG_TO_RAD));
 
     for(int p = 0; p < lintrk.size(); p++)
     {
@@ -1126,7 +1254,7 @@ void CGisItemTrk::deriveSecondaryData()
 
     setupInterpolation(interp.valid, interp.Q);
 
-    updateVisuals(eVisualPlot|eVisualDetails|eVisualProject|eVisualColorAct|eVisualTrkTable, "deriveSecondaryData()");
+    updateVisuals(eVisualPlot|eVisualDetails|eVisualProject|eVisualColorAct|eVisualTrkTable|eVisualTrkInfo, "deriveSecondaryData()");
 
 //    qDebug() << "--------------" << getName() << "------------------";
 //    qDebug() << "allValidFlags" << hex << allValidFlags;
@@ -1150,17 +1278,24 @@ void CGisItemTrk::findWaypointsCloseBy(CProgressDialog& progress, quint32& curre
 
     bool withDoubles = project->getSortingRoadbook() != IGisProject::eSortRoadbookTrackWithoutDouble;
 
+
+    qreal north = -90 * DEG_TO_RAD;
+    qreal south = 90 * DEG_TO_RAD;
+    qreal west = 180 * DEG_TO_RAD;
+    qreal east = -180 * DEG_TO_RAD;
     QVector<pointDP> line;
     // combine all segments to a single line
     for(CTrackData::trkpt_t& pt : trk)
     {
         pt.keyWpt.clear();
-        if(pt.isHidden())
-        {
-            continue;
-        }
         pointDP dp(pt.lon * DEG_TO_RAD, pt.lat * DEG_TO_RAD, 0);
-        dp.idx = pt.idxVisible;
+        dp.idx = pt.idxTotal;
+
+        north = qMax(north, dp.y);
+        south = qMin(south, dp.y);
+        west = qMin(west, dp.x);
+        east = qMax(east, dp.x);
+
         line << dp;
     }
 
@@ -1169,6 +1304,9 @@ void CGisItemTrk::findWaypointsCloseBy(CProgressDialog& progress, quint32& curre
         return;
     }
 
+    constexpr qreal OFFSET = 0.1 * DEG_TO_RAD;
+    QRectF _boundingRect(QPointF(west - OFFSET, north + OFFSET), QPointF(east + OFFSET, south - OFFSET));
+
     // convert coordinates of all waypoints into meter coordinates relative to the first track point
     point3D pt0 = line[0];
     QList<trkwpt_t> trkwpts;
@@ -1180,7 +1318,7 @@ void CGisItemTrk::findWaypointsCloseBy(CProgressDialog& progress, quint32& curre
             continue;
         }
 
-        if(!boundingRect.contains(wpt->getBoundingRect().topLeft()))
+        if(!_boundingRect.contains(wpt->getBoundingRect().topLeft()))
         {
             continue;
         }
@@ -1209,6 +1347,7 @@ void CGisItemTrk::findWaypointsCloseBy(CProgressDialog& progress, quint32& curre
         pt1.y = qSin(a1 * DEG_TO_RAD) * d;
     }
 
+    bool doDeriveData = false;
     numberOfAttachedWpt = 0;
     for(const trkwpt_t &trkwpt : trkwpts)
     {
@@ -1230,11 +1369,16 @@ void CGisItemTrk::findWaypointsCloseBy(CProgressDialog& progress, quint32& curre
             }
             else if(withDoubles && (d > WPT_FOCUS_DIST_OUT))
             {
-                CTrackData::trkpt_t * trkpt = const_cast<CTrackData::trkpt_t*>(trk.getTrkPtByVisibleIndex(index));
+                CTrackData::trkpt_t * trkpt = const_cast<CTrackData::trkpt_t*>(trk.getTrkPtByTotalIndex(index));
                 if(trkpt)
                 {
                     ++numberOfAttachedWpt;
                     trkpt->keyWpt = trkwpt.key;
+                    if(trkpt->isHidden())
+                    {
+                        trkpt->unsetFlag(CTrackData::trkpt_t::eFlagHidden);
+                        doDeriveData = true;
+                    }
                 }
 
                 index = NOIDX;
@@ -1250,15 +1394,24 @@ void CGisItemTrk::findWaypointsCloseBy(CProgressDialog& progress, quint32& curre
 
         if(index != NOIDX)
         {
-            CTrackData::trkpt_t * trkpt = const_cast<CTrackData::trkpt_t*>(trk.getTrkPtByVisibleIndex(index));
+            CTrackData::trkpt_t * trkpt = const_cast<CTrackData::trkpt_t*>(trk.getTrkPtByTotalIndex(index));
             if(trkpt)
             {
                 ++numberOfAttachedWpt;
                 trkpt->keyWpt = trkwpt.key;
+                if(trkpt->isHidden())
+                {
+                    trkpt->unsetFlag(CTrackData::trkpt_t::eFlagHidden);
+                    doDeriveData = true;
+                }
             }
         }
     }
 
+    if(doDeriveData)
+    {
+        deriveSecondaryData();
+    }
     updateVisuals(eVisualDetails|eVisualPlot, "findWaypointsCloseBy()");
 }
 
@@ -1315,6 +1468,8 @@ bool CGisItemTrk::cut()
         return false;
     }
 
+    mouseMoveFocus = nullptr;
+
     qint32 idxMouse             = mouseClickFocus->idxTotal;
     CCutTrk::mode_e mode        = dlg.getMode();
     CCutTrk::cutmode_e cutMode  = dlg.getCutMode();
@@ -1403,6 +1558,49 @@ bool CGisItemTrk::cut()
     return askToDeleteOriginal;
 }
 
+bool CGisItemTrk::addTrkPtDesc()
+{
+    if(nullptr == mouseClickFocus)
+    {
+        return false;
+    }
+
+    const QString& desc = QInputDialog::getText(CMainWindow::self().getBestWidgetForParent(), tr("Track Point Info..."),
+                                                tr("Enter some text to be attached to this track point:"));
+
+    if(desc.isEmpty())
+    {
+        return false;
+    }
+
+    if(trk.setTrkPtDesc(mouseClickFocus->idxTotal, desc))
+    {
+        changed(tr("Add track point desc.: %1").arg(desc), "://icons/48x48/I.png");
+        return true;
+    }
+    return false;
+}
+
+bool CGisItemTrk::setTrkPtDesc(int idxTotal, const QString& desc)
+{
+    if(trk.setTrkPtDesc(idxTotal, desc))
+    {
+        changed(tr("Changed track point desc.: %1").arg(desc), "://icons/48x48/I.png");
+        return true;
+    }
+    return false;
+}
+
+bool CGisItemTrk::delTrkPtDesc(const QList<int>& idxTotal)
+{
+    if(trk.delTrkPtDesc(idxTotal))
+    {
+        changed(tr("Removed track point desc."), "://icons/48x48/DeleteMultiple.png");
+        return true;
+    }
+    return false;
+}
+
 void CGisItemTrk::reverse()
 {
     QString name = getName() + "_rev";
@@ -1533,9 +1731,10 @@ void CGisItemTrk::hideSelectedPoints()
     // iterate over all segments and delete points between idx1 and idx2
     for(CTrackData::trkpt_t& trkpt : trk)
     {
-        if((idx1 < trkpt.idxTotal) && (trkpt.idxTotal < idx2))
+        if((idx1 < trkpt.idxTotal) && (trkpt.idxTotal < idx2)
+           && trkpt.desc.isEmpty() && trkpt.keyWpt.item.isEmpty())
         {
-            trkpt.setFlag(CTrackData::trkpt_t::eHidden);
+            trkpt.setFlag(CTrackData::trkpt_t::eFlagHidden);
         }
     }
     resetMouseRange();
@@ -1569,7 +1768,7 @@ void CGisItemTrk::showSelectedPoints()
     {
         if(isInRange(trkpt.idxTotal, idx1, idx2))
         {
-            trkpt.unsetFlag(CTrackData::trkpt_t::eHidden);
+            trkpt.unsetFlag(CTrackData::trkpt_t::eFlagHidden);
         }
     }
 
@@ -1598,6 +1797,7 @@ void CGisItemTrk::copySelectedPoints() const
     new CGisItemTrk(name, idx1, idx2, trk, project);
 }
 
+
 void CGisItemTrk::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF> &blockedAreas, CGisDraw *gis)
 {
     QMutexLocker lock(&mutexItems);
@@ -1605,7 +1805,7 @@ void CGisItemTrk::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
     lineSimple.clear();
     lineFull.clear();
 
-    if(!isVisible(boundingRect, viewport,gis))
+    if(!isVisible(boundingRect, viewport, gis))
     {
         return;
     }
@@ -1620,7 +1820,7 @@ void CGisItemTrk::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
     QPointF p2 = viewport[2];
     gis->convertRad2Px(p1);
     gis->convertRad2Px(p2);
-    QRectF extViewport(p1,p2);
+    QRectF extViewport(p1, p2);
 
     if(mode == eModeNormal)
     {
@@ -1690,7 +1890,8 @@ void CGisItemTrk::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
     QList<QPolygonF> lines;
     splitLineToViewport(lineSimple, extViewport, lines);
 
-    if(key == keyUserFocus)
+    const CMainWindow& w = CMainWindow::self();
+    if(key == keyUserFocus && w.isShowTrackHighlight())
     {
         p.setPen(QPen(Qt::red, penWidthHi, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
         for(const QPolygonF &l : lines)
@@ -1704,9 +1905,11 @@ void CGisItemTrk::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
     for(const QPolygonF &l : lines)
     {
         p.drawPolyline(l);
-        if(!isNogo() && showArrows.val().toBool())
+        const QRectF& bounding = l.boundingRect();
+        auto area = bounding.width() * bounding.height();
+        if(!isNogo() && showArrows.val().toBool() && (area > (100 * 100)))
         {
-            CDraw::arrows(l, extViewport, p, 10, 80, lineScale.val().toDouble());
+            CDraw::arrows(l, extViewport, p, 2, 120, lineScale.val().toDouble());
         }
     }
 
@@ -1738,15 +1941,10 @@ void CGisItemTrk::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
     }
 
     // -------------------------
-
+    bool skipDecorations = (!keyUserFocus.item.isEmpty() && (key != keyUserFocus));
     // draw min/max labels
-    if(CMainWindow::self().isMinMaxTrackValues())
+    if(w.isShowMinMaxTrackLabels() && !skipDecorations)
     {
-        if(!keyUserFocus.item.isEmpty() && (key != keyUserFocus))
-        {
-            return;
-        }
-
         for(const QString& key : extrema.keys())
         {
             if(key == CKnownExtension::internalProgress)
@@ -1768,6 +1966,35 @@ void CGisItemTrk::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
             p.drawEllipse(posMax, 5, 5);
         }
     }
+
+    // draw info points
+    if(w.isShowTrackInfoPoints() && !skipDecorations)
+    {
+        const QFont& f  = w.getMapFont();
+        const int pointSize = f.pointSize();
+        const int size = (pointSize + (f.bold() ? 3 : 2)) * 2;
+        p.setFont(f);
+
+        quint32 cnt = 1;
+        QList<cluster> clusters;
+        for(const CTrackData::trkpt_t &trkpt : trk)
+        {
+            if(trkpt.desc.isEmpty() || trkpt.isHidden())
+            {
+                continue;
+            }
+
+            QPointF pos(trkpt.lon, trkpt.lat);
+            pos *= DEG_TO_RAD;
+            gis->convertRad2Px(pos);
+
+            QRect r(0, 0, size, size);
+            r.moveCenter(pos.toPoint());
+            cluster::addToCluster(r, clusters, cnt++);
+        }
+
+        cluster::draw(clusters, p, size);
+    }
 }
 
 
@@ -1775,8 +2002,8 @@ void CGisItemTrk::drawLimitLabels(limit_type_e type, const QString& label, const
 {
     const QString& fullLabel = (type == eLimitTypeMin ? tr("min.") : tr("max.")) + " " + label;
     QRectF rect = fm.boundingRect(fullLabel);
-    rect.moveBottomLeft(pos.toPoint() + QPoint(10,-10));
-    rect.adjust(-4,-2,4,2);
+    rect.moveBottomLeft(pos.toPoint() + QPoint(10, -10));
+    rect.adjust(-4, -2, 4, 2);
 
     qint32 baseWidth    = 10;
     qint32 basePos      = 10;
@@ -1784,17 +2011,17 @@ void CGisItemTrk::drawLimitLabels(limit_type_e type, const QString& label, const
 
     if(CDraw::doesOverlap(blockedAreas, rect))
     {
-        rect.moveBottomRight(pos.toPoint() + QPoint(-10,-10));
+        rect.moveBottomRight(pos.toPoint() + QPoint(-10, -10));
         basePos = rect.width() - 10;
 
         if(CDraw::doesOverlap(blockedAreas, rect))
         {
-            rect.moveTopLeft(pos.toPoint() + QPoint(10,10));
+            rect.moveTopLeft(pos.toPoint() + QPoint(10, 10));
             basePos = 10;
 
             if(CDraw::doesOverlap(blockedAreas, rect))
             {
-                rect.moveTopRight(pos.toPoint() + QPoint(-10,10));
+                rect.moveTopRight(pos.toPoint() + QPoint(-10, 10));
                 basePos = rect.width() - 10;
 
                 if(CDraw::doesOverlap(blockedAreas, rect))
@@ -1805,15 +2032,14 @@ void CGisItemTrk::drawLimitLabels(limit_type_e type, const QString& label, const
         }
     }
 
-    CDraw::bubble(p, rect.toRect(), pos.toPoint(), baseWidth, basePos, (key == keyUserFocus) ? CDraw::penBorderRed : CDraw::penBorderGray);
+    CDraw::bubble(p, rect.toRect(), pos.toPoint(), Qt::white, baseWidth, basePos, (key == keyUserFocus) ? CDraw::penBorderRed : CDraw::penBorderGray);
     CDraw::text(fullLabel, p, rect.toRect(), type == eLimitTypeMin ? Qt::darkGreen : Qt::darkRed);
     blockedAreas << rect;
 }
 
-void CGisItemTrk::setPen(QPainter& p, QPen& pen, quint32 flag) const
+void CGisItemTrk::setPen(QPainter& p, QPen& pen, trkact_t act) const
 {
-    flag &= CTrackData::trkpt_t::eActMask;
-    pen.setColor((flag == 0) ? color : CActivityTrk::getDescriptor(flag).color);
+    pen.setColor((act == CTrackData::trkpt_t::eAct20None) ? color : CActivityTrk::getDescriptor(act).color);
     p.setPen(pen);
 }
 
@@ -1835,14 +2061,14 @@ void CGisItemTrk::drawColorizedByActivity(QPainter& p) const
             }
             if(nullptr == ptPrev)
             {
-                setPen(p, pen, pt.flags);
+                setPen(p, pen, pt.getAct());
                 ptPrev = &pt;
                 continue;
             }
 
-            if((ptPrev->flags & CTrackData::trkpt_t::eActMask) != (pt.flags & CTrackData::trkpt_t::eActMask))
+            if(ptPrev->getAct() != pt.getAct())
             {
-                setPen(p, pen, pt.flags);
+                setPen(p, pen, pt.getAct());
             }
 
             p.drawLine(lineSimple[ptPrev->idxVisible], lineSimple[pt.idxVisible]);
@@ -2013,6 +2239,19 @@ void CGisItemTrk::drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis)
         // create trackpoint info text
         QString str = getInfoTrkPt(*mouseMoveFocus);
 
+        // search for track point information in the neighboring points
+        const int idxMin = qMax(mouseMoveFocus->idxTotal - 2, 0);
+        const int idxMax = qMin(mouseMoveFocus->idxTotal + 3, cntTotalPoints);
+        for(int idx = idxMin; idx < idxMax; idx++)
+        {
+            const QString& desc = trk.getTrkPtByTotalIndex(idx)->desc;
+            if(!desc.isEmpty())
+            {
+                str += "\nInfo: " + desc;
+                break;
+            }
+        }
+
         // calculate bounding box of text
         QFont f = CMainWindow::self().getMapFont();
         QFontMetrics fm(f);
@@ -2041,9 +2280,14 @@ void CGisItemTrk::drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis)
         p.setFont(f);
 
         // draw the bubble
+        QWidget widget;
+        const QPalette& pal = widget.palette();
+        const QColor& colorBg = pal.color(QPalette::Background);
+        const QColor& colorFg = pal.color(QPalette::Foreground);
+
         QRect box(0, 0, w, h);
         box.moveBottomLeft(anchor.toPoint() + QPoint(-50, -50));
-        CDraw::bubble(p, box, anchor.toPoint(), 18 /* px */, 21 /* px */);
+        CDraw::bubble(p, box, anchor.toPoint(), colorBg, 18 /* px */, 21 /* px */);
 
         p.save();
         p.translate(box.topLeft());
@@ -2093,7 +2337,7 @@ void CGisItemTrk::drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis)
 
         // draw text
         p.translate(0, fm.height() + 8);
-        p.setPen(Qt::darkBlue);
+        p.setPen(colorFg);
         p.drawText(rectText, Qt::AlignLeft|Qt::AlignTop|Qt::TextWordWrap, str);
 
         p.restore();
@@ -2118,10 +2362,17 @@ void CGisItemTrk::drawLabel(QPainter& p, const QPolygonF&, QList<QRectF>& blocke
         return;
     }
 
-    if(CMainWindow::self().isMinMaxTrackValues())
+    const CMainWindow& w = CMainWindow::self();
+    if(w.isShowMinMaxTrackLabels())
     {
         for(const QString& key : extrema.keys())
         {
+            if(key == CKnownExtension::internalSpeedTime)
+            {
+                // already handled by CKnownExtension::internalSpeedDist
+                continue;
+            }
+
             const CKnownExtension& ext = CKnownExtension::get(key);
             const limits_t& limit = extrema[key];
             QPointF posMin = limit.posMin * DEG_TO_RAD;
@@ -2133,6 +2384,8 @@ void CGisItemTrk::drawLabel(QPainter& p, const QPolygonF&, QList<QRectF>& blocke
             QString labelMin = ext.toString(limit.min, true, key);
             QString labelMax = ext.toString(limit.max, true, key);
 
+            qDebug() << labelMin << labelMax;
+
             if(!labelMin.isEmpty())
             {
                 drawLimitLabels(eLimitTypeMin, labelMin, posMin, p, fm, blockedAreas);
@@ -2161,7 +2414,7 @@ void CGisItemTrk::drawHighlight(QPainter& p)
     QList<QPolygonF> lines;
     splitLineToViewport(lineSimple, p.viewport(), lines);
 
-    p.setPen(QPen(QColor(255,0,0,100), penWidthHi, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
+    p.setPen(QPen(QColor(255, 0, 0, 100), penWidthHi, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
 
     for(const QPolygonF &line : lines)
     {
@@ -2241,8 +2494,7 @@ void CGisItemTrk::setLinks(const QList<link_t>& links)
 
 void CGisItemTrk::setElevation(qint32 idx, qint32 ele)
 {
-    auto condition = [idx](const CTrackData::trkpt_t &pt) { return pt.idxTotal == idx; };
-    CTrackData::trkpt_t * trkpt = trk.getTrkPtByCondition(condition);
+    CTrackData::trkpt_t * trkpt = trk.getTrkPtByTotalIndex(idx);
     if((trkpt != nullptr) && (trkpt->ele != ele))
     {
         trkpt->ele = ele;
@@ -2253,35 +2505,34 @@ void CGisItemTrk::setElevation(qint32 idx, qint32 ele)
 
 void CGisItemTrk::setColor(int idx)
 {
-    if(idx < TRK_N_COLORS)
+    if(idx < IGisItem::getColorMap().count())
     {
-        setColor(IGisItem::colorMap[idx].color);
+        setColor(IGisItem::getColorMap()[idx].color);
         updateHistory(eVisualColorLegend|eVisualDetails);
     }
 }
 
-void CGisItemTrk::setActivity(quint32 flag)
+void CGisItemTrk::setActivity(trkact_t act)
 {
     for(CTrackData::trkpt_t& trkpt : trk)
     {
-        trkpt.unsetFlag(CTrackData::trkpt_t::eActMask);
-        trkpt.setFlag((enum CTrackData::trkpt_t::flag_e) flag);
+        trkpt.setAct(act);
     }
 
     deriveSecondaryData();
 
-    const CActivityTrk::desc_t &desc = CActivityTrk::getDescriptor(flag);
+    const CActivityTrk::desc_t &desc = CActivityTrk::getDescriptor(act);
     changed(tr("Changed activity to '%1' for complete track.").arg(desc.name), desc.iconLarge);
 }
 
-void CGisItemTrk::setActivityRange(quint32 flags)
+void CGisItemTrk::setActivityRange(trkact_t act)
 {
     if(!setReadOnlyMode(false))
     {
         return;
     }
 
-    const CActivityTrk::desc_t &desc = CActivityTrk::getDescriptor(flags);
+    const CActivityTrk::desc_t &desc = CActivityTrk::getDescriptor(act);
 
     // read start/stop indices
     qint32 idx1, idx2;
@@ -2303,8 +2554,7 @@ void CGisItemTrk::setActivityRange(quint32 flags)
     {
         if((idx1 <= trkpt.idxTotal) && (trkpt.idxTotal < idx2))
         {
-            trkpt.unsetFlag(CTrackData::trkpt_t::eActMask);
-            trkpt.setFlag((enum CTrackData::trkpt_t::flag_e) flags);
+            trkpt.setAct(act);
         }
     }
 
@@ -2316,17 +2566,20 @@ void CGisItemTrk::setActivityRange(quint32 flags)
 void CGisItemTrk::setColor(const QColor& c)
 {
     colorIdx = DEFAULT_COLOR;
-    for(int n = 0; n < TRK_N_COLORS; n++)
+
+    const QVector<IGisItem::color_t>& colorMap = IGisItem::getColorMap();
+    const int N = colorMap.count();
+    for(int n = 0; n < N; n++)
     {
-        if(c == IGisItem::colorMap[n].color)
+        if(c == colorMap[n].color)
         {
             colorIdx = n;
             break;
         }
     }
 
-    color  = IGisItem::colorMap[colorIdx].color;
-    bullet = QPixmap(IGisItem::colorMap[colorIdx].bullet);
+    color  = colorMap[colorIdx].color;
+    bullet = QPixmap(colorMap[colorIdx].bullet);
 
     setIcon(color2str(color));
 }
@@ -2339,7 +2592,7 @@ void CGisItemTrk::setIcon(const QString& iconColor)
     QPixmap mask( icon.size() );
     mask.fill( str2color(iconColor) );
     mask.setMask( icon.createMaskFromColor( Qt::transparent ) );
-    IGisItem::setIcon(mask.scaled(22,22, Qt::KeepAspectRatio, Qt::SmoothTransformation));
+    IGisItem::setIcon(mask.scaled(22, 22, Qt::KeepAspectRatio, Qt::SmoothTransformation));
 }
 
 bool CGisItemTrk::setMouseFocusByDistance(qreal dist, focusmode_e fmode, const QString &owner)
@@ -2570,6 +2823,13 @@ void CGisItemTrk::updateVisuals(quint32 visuals, const QString& who)
             visual->updateData();
         }
     }
+
+    const CMainWindow& main = CMainWindow::self();
+    const QList<CCanvas*>& allCanvas = main.getCanvas();
+    for(CCanvas * canvas : allCanvas)
+    {
+        canvas->slotUpdateTrackInfo(false);
+    }
 }
 
 void CGisItemTrk::setMouseFocusVisuals(const CTrackData::trkpt_t * pt)
@@ -2623,7 +2883,7 @@ void CGisItemTrk::setupInterpolation(bool on, qint32 q)
     }
 
     const qint32 N = getNumberOfVisiblePoints();
-    alglib::real_1d_array x,y;
+    alglib::real_1d_array x, y;
     x.setlength(N);
     y.setlength(N);
 
@@ -2720,3 +2980,19 @@ bool CGisItemTrk::findPolylineCloseBy(const QPointF& pt1, const QPointF& pt2, qi
 
     return !polyline.isEmpty();
 }
+
+void CGisItemTrk::checkForInvalidPoints()
+{
+    IGisProject * project = getParentProject();
+    if(project && project->getInvalidDataOk())
+    {
+        return;
+    }
+
+    if((cntInvalidPoints != 0) && (cntInvalidPoints < cntVisiblePoints))
+    {
+        CInvalidTrk dlg(*this, CMainWindow::self().getBestWidgetForParent());
+        dlg.exec();
+    }
+}
+
