@@ -200,6 +200,15 @@ void CGisItemWpt::setSymbol()
     setIcon();
 }
 
+void CGisItemWpt::genKey() const
+{
+    if(geocache.hasData)
+    {
+        key.item=QString::number(geocache.id);
+    }
+    IGisItem::genKey();
+}
+
 QString CGisItemWpt::getLastName(const QString& name)
 {
     SETTINGS;
@@ -292,16 +301,6 @@ QString CGisItemWpt::getInfo(quint32 feature) const
         str = "<b>" + getName() + "</b>";
     }
 
-    if(geocache.hasData)
-    {
-        if(str.size() > initialSize)
-        {
-            str += "<br/>\n";
-        }
-
-        str += QString(" %4 (%1, D %2, T %3)").arg(geocache.container).arg(geocache.difficulty, 0,'f',1).arg(geocache.terrain, 0,'f',1).arg(geocache.name);
-    }
-
     if(wpt.ele != NOINT)
     {
         if(str.size() > initialSize)
@@ -325,20 +324,32 @@ QString CGisItemWpt::getInfo(quint32 feature) const
     }
 
     QString desc = removeHtml(wpt.desc).simplified();
-    if(desc.count())
+    if(geocache.hasData)
     {
         if(str.size() > initialSize)
         {
             str += "<br/>\n";
         }
 
-        if((feature & eFeatureShowFullText) || (desc.count() < 300))
+        str += QString(" %4 (%1, D %2, T %3)").arg(geocache.container).arg(geocache.difficulty, 0, 'f', 1).arg(geocache.terrain, 0, 'f', 1).arg(geocache.name);
+    }
+    else
+    {
+        if(desc.count())
         {
-            str += desc;
-        }
-        else
-        {
-            str += desc.left(297) + "...";
+            if(str.size() > initialSize)
+            {
+                str += "<br/>\n";
+            }
+
+            if((feature & eFeatureShowFullText) || (desc.count() < 300))
+            {
+                str += desc;
+            }
+            else
+            {
+                str += desc.left(297) + "...";
+            }
         }
     }
 
@@ -411,7 +422,7 @@ QPointF CGisItemWpt::getPointCloseBy(const QPoint& point)
     if (closeToRadius)
     {
         QPointF l = (QPointF(point) - posScreen);
-        return posScreen + l * (radius / sqrt(QPointF::dotProduct(l,l)));
+        return posScreen + l * (radius / sqrt(QPointF::dotProduct(l, l)));
     }
     else
     {
@@ -439,7 +450,7 @@ void CGisItemWpt::setName(const QString& str)
     setText(CGisListWks::eColumnName, str);
 
     wpt.name = str;
-    changed(tr("Changed name"),"://icons/48x48/EditText.png");
+    changed(tr("Changed name"), "://icons/48x48/EditText.png");
 }
 
 void CGisItemWpt::setPosition(const QPointF& pos)
@@ -449,13 +460,13 @@ void CGisItemWpt::setPosition(const QPointF& pos)
 
     detBoundingRect();
 
-    changed(tr("Changed position"),"://icons/48x48/WptMove.png");
+    changed(tr("Changed position"), "://icons/48x48/WptMove.png");
 }
 
 void CGisItemWpt::setElevation(qint32 val)
 {
     wpt.ele = val;
-    changed(tr("Changed elevation"),"://icons/48x48/SetEle.png");
+    changed(tr("Changed elevation"), "://icons/48x48/SetEle.png");
 }
 
 void CGisItemWpt::setProximity(qreal val)
@@ -464,12 +475,12 @@ void CGisItemWpt::setProximity(qreal val)
     {
         proximity = NOFLOAT;
         setNogoFlag(false);
-        changed(tr("Removed proximity"),"://icons/48x48/WptDelProx.png");
+        changed(tr("Removed proximity"), "://icons/48x48/WptDelProx.png");
     }
     else
     {
         proximity = qRound(val);
-        changed(tr("Changed proximity"),"://icons/48x48/WptEditProx.png");
+        changed(tr("Changed proximity"), "://icons/48x48/WptEditProx.png");
     }
 
     detBoundingRect();
@@ -541,7 +552,7 @@ bool CGisItemWpt::isCloseTo(const QPointF& pos)
         return false;
     }
 
-    closeToRadius = abs(QPointF::dotProduct(dist,dist)/radius - radius) < 22;
+    closeToRadius = abs(QPointF::dotProduct(dist, dist)/radius - radius) < 22;
     return closeToRadius;
 }
 
@@ -598,7 +609,7 @@ void CGisItemWpt::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>
     else
     {
         //remember radius for isCloseTo-method
-        radius = calcRadius(QPointF(wpt.lon * DEG_TO_RAD, wpt.lat * DEG_TO_RAD),posScreen,proximity,gis);
+        radius = calcRadius(QPointF(wpt.lon * DEG_TO_RAD, wpt.lat * DEG_TO_RAD), posScreen, proximity, gis);
 
         drawCircle(p, posScreen, radius, !hideArea && isNogo(), false);
     }
@@ -621,18 +632,18 @@ void CGisItemWpt::drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis)
         QRect barTop(rectBubble.topLeft(), QSize(rectBubble.width(), 26));
         QRect barBottom(barTop);
         barBottom.moveBottomLeft(rectBubble.bottomLeft());
-        barBottom.adjust(1,0,-1,-1);
-        barTop.adjust(1,1,-1,0);
+        barBottom.adjust(1, 0, -1, -1);
+        barTop.adjust(1, 1, -1, 0);
 
         p.setPen(Qt::NoPen);
-        p.setBrush(QColor(200,200,255,150));
+        p.setBrush(QColor(200, 200, 255, 150));
         p.drawRect(barTop);
         p.drawRect(barBottom);
 
         p.setBrush(Qt::white);
-        p.drawRoundedRect(rectBubbleMove.adjusted(-2,-2,2,2), RECT_RADIUS, RECT_RADIUS);
-        p.drawRoundedRect(rectBubbleEdit.adjusted(-2,-2,2,2), RECT_RADIUS, RECT_RADIUS);
-        p.drawRoundedRect(rectBubbleSize.adjusted(-2,-2,2,2), RECT_RADIUS, RECT_RADIUS);
+        p.drawRoundedRect(rectBubbleMove.adjusted(-2, -2, 2, 2), RECT_RADIUS, RECT_RADIUS);
+        p.drawRoundedRect(rectBubbleEdit.adjusted(-2, -2, 2, 2), RECT_RADIUS, RECT_RADIUS);
+        p.drawRoundedRect(rectBubbleSize.adjusted(-2, -2, 2, 2), RECT_RADIUS, RECT_RADIUS);
 
         p.drawPixmap(rectBubbleMove, QPixmap("://icons/32x32/MoveArrow.png"));
         p.drawPixmap(rectBubbleEdit, QPixmap("://icons/32x32/EditDetails.png"));
@@ -656,7 +667,7 @@ void CGisItemWpt::drawLabel(QPainter& p, const QPolygonF &viewport, QList<QRectF
     QPointF pt = posScreen - focus;
 
     QRectF rect = fm.boundingRect(wpt.name);
-    rect.adjust(-2,-2,2,2);
+    rect.adjust(-2, -2, 2, 2);
 
     // place label on top
     rect.moveCenter(pt + QPointF(icon.width()/2, -fm.height()));
@@ -681,7 +692,7 @@ void CGisItemWpt::drawLabel(QPainter& p, const QPolygonF &viewport, QList<QRectF
         }
     }
 
-    CDraw::text(wpt.name,p,rect.toRect(), Qt::darkBlue);
+    CDraw::text(wpt.name, p, rect.toRect(), Qt::darkBlue);
     blockedAreas << rect;
 }
 
@@ -698,7 +709,7 @@ void CGisItemWpt::drawHighlight(QPainter& p)
     }
     else
     {
-        p.drawImage(posScreen - QPointF(31,31), QImage("://cursors/wptHighlightRed.png"));
+        p.drawImage(posScreen - QPointF(31, 31), QImage("://cursors/wptHighlightRed.png"));
     }
 }
 
@@ -731,9 +742,9 @@ void CGisItemWpt::drawBubble(QPainter& p)
     QPoint posBubble = posScreen.toPoint() + offsetBubble;
     rectBubble.moveTopLeft(posBubble);
 
-    rectBubbleMove.moveTopLeft(rectBubble.topLeft() + QPoint(5,5));
-    rectBubbleEdit.moveTopLeft(rectBubbleMove.topRight() + QPoint(7,0));
-    rectBubbleSize.moveBottomRight(rectBubble.bottomRight() - QPoint(5,5));
+    rectBubbleMove.moveTopLeft(rectBubble.topLeft() + QPoint(5, 5));
+    rectBubbleEdit.moveTopLeft(rectBubbleMove.topRight() + QPoint(7, 0));
+    rectBubbleSize.moveBottomRight(rectBubble.bottomRight() - QPoint(5, 5));
 
     QPolygonF frame = makePolyline(posScreen, rectBubble);
     p.setPen(CDraw::penBorderGray);
@@ -754,13 +765,13 @@ void CGisItemWpt::drawCircle(QPainter& p, const QPointF& pos, const qreal& r, co
     p.setBrush(Qt::NoBrush);
     if (selected)
     {
-        p.setPen(QPen(Qt::red,3));
+        p.setPen(QPen(Qt::red, 3));
     }
     else
     {
-        p.setPen(QPen(Qt::white,3));
+        p.setPen(QPen(Qt::white, 3));
         p.drawEllipse(circle);
-        p.setPen(QPen(Qt::red,1));
+        p.setPen(QPen(Qt::red, 1));
     }
     p.drawEllipse(circle);
     if (nogo)
@@ -800,23 +811,23 @@ QPolygonF CGisItemWpt::makePolyline(const QPointF& anchor, const QRectF& r)
             h = 30;
         }
 
-        w = h = qMin(w,h);
+        w = h = qMin(w, h);
 
         if(anchor.x() < r.left())
         {
-            poly2 << anchor << (r.center() + QPoint(0,-h)) << (r.center() + QPoint(0,h)) << anchor;
+            poly2 << anchor << (r.center() + QPoint(0, -h)) << (r.center() + QPoint(0, h)) << anchor;
         }
         else if(r.right() < anchor.x())
         {
-            poly2 << anchor << (r.center() + QPoint(0,-h)) << (r.center() + QPoint(0,h)) << anchor;
+            poly2 << anchor << (r.center() + QPoint(0, -h)) << (r.center() + QPoint(0, h)) << anchor;
         }
         else if(anchor.y() < r.top())
         {
-            poly2 << anchor << (r.center() + QPoint(-w,0)) << (r.center() + QPoint(w,0)) << anchor;
+            poly2 << anchor << (r.center() + QPoint(-w, 0)) << (r.center() + QPoint(w, 0)) << anchor;
         }
         else if(r.bottom() < anchor.y())
         {
-            poly2 << anchor << (r.center() + QPoint(-w,0)) << (r.center() + QPoint(w,0)) << anchor;
+            poly2 << anchor << (r.center() + QPoint(-w, 0)) << (r.center() + QPoint(w, 0)) << anchor;
         }
 
         QPainterPath path1;
@@ -981,7 +992,7 @@ void CGisItemWpt::detBoundingRect()
 {
     if(proximity == NOFLOAT)
     {
-        boundingRect = QRectF(QPointF(wpt.lon,wpt.lat)*DEG_TO_RAD,QPointF(wpt.lon,wpt.lat)*DEG_TO_RAD);
+        boundingRect = QRectF(QPointF(wpt.lon, wpt.lat)*DEG_TO_RAD, QPointF(wpt.lon, wpt.lat)*DEG_TO_RAD);
     }
     else
     {
@@ -991,6 +1002,77 @@ void CGisItemWpt::detBoundingRect()
         QPointF pt1 = GPS_Math_Wpt_Projection(cent, diag, 225 * DEG_TO_RAD);
         QPointF pt2 = GPS_Math_Wpt_Projection(cent, diag, 45 * DEG_TO_RAD);
 
-        boundingRect = QRectF(pt1,pt2);
+        boundingRect = QRectF(pt1, pt2);
     }
 }
+
+const QVector<QString> CGisItemWpt::geocache_t::attributeMeanings = {
+    "QMS Attribute Flag",         //Not to be serialized in GPX files
+    "Dogs",
+    "Access or parking fee",
+    "Climbing gear",
+    "Boat",
+    "Scuba gear",
+    "Recommended for kids",
+    "Takes less than an hour",
+    "Scenic view",
+    "Significant hike",
+    "Difficult climbing",
+    "May require wading",
+    "May require swimming",
+    "Available at all times",
+    "Recommended at night",
+    "Available during winter",
+    "",
+    "Poison plants",
+    "Dangerous Animals",
+    "Ticks",
+    "Abandoned mines",
+    "Cliff / falling rocks",
+    "Hunting",
+    "Dangerous area",
+    "Wheelchair accessible",
+    "Parking available",
+    "Public transportation",
+    "Drinking water nearby",
+    "Public restrooms nearby",
+    "Telephone nearby",
+    "Picnic tables nearby",
+    "Camping available",
+    "Bicycles",
+    "Motorcycles",
+    "Quads",
+    "Off-road vehicles",
+    "Snowmobiles",
+    "Horses",
+    "Campfires",
+    "Thorns",
+    "Stealth required",
+    "Stroller accessible",
+    "Needs maintenance",
+    "Watch for livestock",
+    "Flashlight required",
+    "",
+    "Truck Driver/RV",
+    "Field Puzzle",
+    "UV Light Required",
+    "Snowshoes",
+    "Cross Country Skis",
+    "Special Tool Required",
+    "Night Cache",
+    "Park and Grab",
+    "Abandoned Structure",
+    "Short hike (less than 1km)",
+    "Medium hike (1km-10km)",
+    "Long Hike (+10km)",
+    "Fuel Nearby",
+    "Food Nearby",
+    "Wireless Beacon",
+    "Partnership cache",
+    "Seasonal Access",
+    "Tourist Friendly",
+    "Tree Climbing",
+    "Front Yard (Private Residence)",
+    "Teamwork Required",
+    "GeoTour"
+};
