@@ -1,6 +1,7 @@
 /**********************************************************************************************
     Copyright (C) 2014 Oliver Eichler oliver.eichler@gmx.de
     Copyright (C) 2017 Norbert Truchsess norbert.truchsess@t-online.de
+    Copyright (C) 2019 Henri Hornburg hrnbg@t-online.de
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -70,6 +71,8 @@ public:
         QString ownerId;
         QString type;
         QString container;
+        //See ZGeocacheAttributes.txt for meanings.
+        QMap<quint8, bool> attributes;
         bool shortDescIsHtml = false;
         QString shortDesc;
         bool longDescIsHtml = false;
@@ -79,6 +82,8 @@ public:
         QString state;
         QString locale;
         QList<geocachelog_t> logs;
+
+        const static QVector<QString> attributeMeanings;
     };
 
     struct image_t
@@ -335,6 +340,8 @@ public:
         hideArea = hide;
     }
 
+    void genKey() const override;
+
     static QString getLastName(const QString &name);
     static void newWpt(QPointF& pt, const QString& name, const QString& desc, IGisProject *project);
     static bool getIconAndName(QString& icon, QString& name);
@@ -381,9 +388,9 @@ private:
     bool doBubbleSize      = false;
     bool mouseIsOverBubble = false;
     QRect rectBubble;
-    QRect rectBubbleMove {0,0,16,16};
-    QRect rectBubbleEdit {0,0,16,16};
-    QRect rectBubbleSize {0,0,16,16};
+    QRect rectBubbleMove {0, 0, 16, 16};
+    QRect rectBubbleEdit {0, 0, 16, 16};
+    QRect rectBubbleSize {0, 0, 16, 16};
 
     QPoint offsetMouse;
     QPoint offsetBubble {-320, -150};
