@@ -55,12 +55,13 @@ CPrintDialog::CPrintDialog(type_e type, const QRectF& area, CCanvas *source)
     QLayout * layout = new QVBoxLayout(frameCanvas);
     layout->addWidget(canvas);
     layout->setSpacing(0);
-    layout->setContentsMargins(0,0,0,0);
+    layout->setContentsMargins(0, 0, 0, 0);
 
-    connect(canvas,    &CCanvas::sigZoom,     this, &CPrintDialog::slotUpdateMetrics);
-    connect(canvas,    &CCanvas::sigMove,     this, &CPrintDialog::slotUpdateMetrics);
-    connect(pushPrint, &QPushButton::pressed, this, &CPrintDialog::slotPrint);
-    connect(pushSave,  &QPushButton::pressed, this, &CPrintDialog::slotSave);
+    connect(canvas,         &CCanvas::sigZoom,     this, &CPrintDialog::slotUpdateMetrics);
+    connect(canvas,         &CCanvas::sigMove,     this, &CPrintDialog::slotUpdateMetrics);
+    connect(pushPrint,      &QPushButton::pressed, this, &CPrintDialog::slotPrint);
+    connect(pushSave,       &QPushButton::pressed, this, &CPrintDialog::slotSave);
+    connect(checkScaleOnAll, &QCheckBox::toggled,   this, &CPrintDialog::slotScaleOnAllChanged);
 
     if(type == eTypePrint)
     {
@@ -143,7 +144,7 @@ void CPrintDialog::slotUpdateMetrics()
     qreal h = rectPrinterPage.height() * scale;
 
     // the page rectangle
-    QRectF rectTile(1,1, w-2, h-2);
+    QRectF rectTile(1, 1, w-2, h-2);
 
     // paint page matrix
     QPainter p(&img);
@@ -159,15 +160,19 @@ void CPrintDialog::slotUpdateMetrics()
         }
     }
     p.setBrush(Qt::NoBrush);
-    p.drawRect(img.rect().adjusted(0,0,-1,-1));
+    p.drawRect(img.rect().adjusted(0, 0, -1, -1));
 
-    p.setPen(QPen(Qt::darkGreen,2));
+    p.setPen(QPen(Qt::darkGreen, 2));
     p.setBrush(Qt::BDiagPattern);
-    p.drawRect(0,0, rectSelAreaPixel.width() * scale, rectSelAreaPixel.height() * scale);
+    p.drawRect(0, 0, rectSelAreaPixel.width() * scale, rectSelAreaPixel.height() * scale);
+
+    qreal xPageSizeMm = printer.pageRect(QPrinter::Millimeter).width();
 
     labelPages->setPixmap(img);
-    labelPagesText->setText(tr("Pages: %1 x %2").arg(xPages,0,'f',1).arg(yPages,0,'f',1));
-    labelMapInfo->setText(tr("Zoom with mouse wheel on map below to change resolution:\n\n%1x%2 pixel\nx: %3 m/px\ny: %4 m/px").arg(rectSelAreaPixel.width()).arg(rectSelAreaPixel.height()).arg(mWidth/rectSelAreaPixel.width(),0,'f',1).arg(mHeight/rectSelAreaPixel.height(),0,'f',1));
+    labelPagesText->setText(tr("Pages: %1 x %2").arg(xPages, 0, 'f', 1).arg(yPages, 0, 'f', 1));
+    QString labelMapInfoText = tr("Zoom with mouse wheel on map below to change resolution:\n\n%1x%2 pixel\nx: %3 m/px\ny: %4 m/px").arg(rectSelAreaPixel.width()).arg(rectSelAreaPixel.height()).arg(mWidth/rectSelAreaPixel.width(), 0, 'f', 1).arg(mHeight/rectSelAreaPixel.height(), 0, 'f', 1);
+    labelMapInfoText += tr("\n This equals to a scale of approx. 1:") + QString::number((int)(mWidth*1000/(xPages*xPageSizeMm)));
+    labelMapInfo->setText(labelMapInfoText);
 }
 
 void CPrintDialog::slotPrint()
@@ -193,7 +198,7 @@ void CPrintDialog::slotPrint()
         qreal xoff = 0;
         for(int x = 0; x < qCeil(xPages); x++)
         {
-            QPointF center = pxCenter0 + QPointF(xoff,yoff);
+            QPointF center = pxCenter0 + QPointF(xoff, yoff);
             canvas->convertPx2Rad(center);
             centers << center;
 
@@ -203,13 +208,13 @@ void CPrintDialog::slotPrint()
         yoff += hPage;
     }
 
-    QRectF rectPage(0,0,wPage, hPage);
+    QRectF rectPage(0, 0, wPage, hPage);
 
     bool first = true;
     QPainter p;
     p.begin(&printer);
     p.setClipRect(rectPage);
-    USE_ANTI_ALIASING(p,true);
+    USE_ANTI_ALIASING(p, true);
 
     int N = centers.size();
     int n = 0;
@@ -225,8 +230,14 @@ void CPrintDialog::slotPrint()
         {
             first = false;
         }
-
-        canvas->print(p, rectPage, pt);
+        if(printScaleOnAllPages || pt == centers.last())
+        {
+            canvas->print(p, rectPage, pt, true);
+        }
+        else
+        {
+            canvas->print(p, rectPage, pt, false);
+        }
         PROGRESS(++n, break);
     }
 
@@ -247,7 +258,7 @@ void CPrintDialog::slotSave()
     QImage img(rect.size().toSize(), QImage::Format_ARGB32);
 
     QPainter p(&img);
-    USE_ANTI_ALIASING(p,true);
+    USE_ANTI_ALIASING(p, true);
 
     canvas->print(p, rect, rectSelArea.center());
 
@@ -285,3 +296,8 @@ void CPrintDialog::slotSave()
 
     QDialog::accept();
 }
+
+void CPrintDialog::slotScaleOnAllChanged(bool checked)
+{
+    printScaleOnAllPages=checked;
+}
